import{r as j}from"./chunk-BsAOyGYT.js";import{f as S,w as B}from"./chunk-B_1l0x81.js";import"./chunk-tuBEHo31.js";/* empty css              */import"./chunk-BXl3LOEh.js";function H({premultipliedAlpha:t,clock:o,autoUpdate:r,preUpdate:n,originDistance:d,maxStdDev:u,minPixelRadius:p,maxPixelRadius:m,minAlpha:c,enable2DGS:s,preBlurAmount:v,blurAmount:f,focalDistance:a,apertureSize:h,falloff:A,clipXY:x,focalAdjustment:D,view:G}){const R=S(i=>i.gl),b=S(i=>i.scene),P=S(i=>i.invalidate),U=(i,l)=>i===void 0||l===void 0||l<=0?0:2*Math.atan(.5*i/l),e=j.useMemo(()=>{let i=null;if(b.traverse(g=>{g instanceof B&&(i=g)}),i)return i;const l=new B({renderer:R,premultipliedAlpha:t,clock:o,autoUpdate:r,preUpdate:n,originDistance:d,maxStdDev:u,minPixelRadius:p,maxPixelRadius:m,minAlpha:c,enable2DGS:s,preBlurAmount:v,blurAmount:f,focalDistance:a,apertureAngle:U(h,a),falloff:A,clipXY:x,focalAdjustment:D,view:G});return b.add(l),l},[R]);return j.useEffect(()=>{t!==void 0&&(e.premultipliedAlpha=t),o!==void 0&&(e.clock=o),r!==void 0&&(e.autoUpdate=r),n!==void 0&&(e.preUpdate=n),d!==void 0&&(e.originDistance=d),u!==void 0&&(e.maxStdDev=u),p!==void 0&&(e.minPixelRadius=p),m!==void 0&&(e.maxPixelRadius=m),c!==void 0&&(e.minAlpha=c),s!==void 0&&(e.enable2DGS=s),v!==void 0&&(e.preBlurAmount=v),f!==void 0&&(e.blurAmount=f),a!==void 0&&(e.focalDistance=a),e.apertureAngle=U(h,a),A!==void 0&&(e.falloff=A),x!==void 0&&(e.clipXY=x),D!==void 0&&(e.focalAdjustment=D),P()},[e,P,t,o,r,n,d,u,p,m,c,s,v,f,a,h,A,x,D]),null}export{H as default};
