const __vite__mapDeps=(i,m=__vite__mapDeps,d=(m.f||(m.f=["assets/chunks/chunk-BEdNaCT-.js","assets/chunks/chunk-B_1l0x81.js","assets/chunks/chunk-BsAOyGYT.js","assets/chunks/chunk-tuBEHo31.js","assets/static/pages_index-c6b9ae3c.Clummn1-.css","assets/chunks/chunk-BXl3LOEh.js"])))=>i.map(i=>d[i]);
import{_ as c}from"./chunk-BXl3LOEh.js";import{r,j as i}from"./chunk-BsAOyGYT.js";import{e as l}from"./chunk-B_1l0x81.js";import"./chunk-tuBEHo31.js";/* empty css              */const m={all:255,foregrounds:2,backgrounds:1};let s=!1;async function p(o=t=>{s=t}){if(typeof window>"u")return;const{LumaSplatsThree:t}=await c(async()=>{const{LumaSplatsThree:a}=await import("./chunk-BEdNaCT-.js");return{LumaSplatsThree:a}},__vite__mapDeps([0,1,2,3,4,5]));l({LumaSplats:t}),o(!0)}function x({src:o,semantics:t="all",...a}){const[n,u]=r.useState(s);return r.useEffect(()=>{s||(async()=>(console.log("registering LumaSplats component. This should occur only once."),await p(e=>{u(e),s=e})))()},[]),n?i.jsx("lumaSplats",{semanticsMask:m[t],source:o,...a}):null}export{x as default};
