import{p as $v}from"./chunk-tuBEHo31.js";/* empty css              */function dm(f,r){for(var m=0;m<r.length;m++){const s=r[m];if(typeof s!="string"&&!Array.isArray(s)){for(const A in s)if(A!=="default"&&!(A in f)){const p=Object.getOwnPropertyDescriptor(s,A);p&&Object.defineProperty(f,A,p.get?p:{enumerable:!0,get:()=>s[A]})}}}return Object.freeze(Object.defineProperty(f,Symbol.toStringTag,{value:"Module"}))}function qa(f){return f&&f.__esModule&&Object.prototype.hasOwnProperty.call(f,"default")?f.default:f}var Sf={exports:{}},Mu={},bf={exports:{}},Tf={};/**
 * @license React
 * scheduler.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Xd;function Wv(){return Xd||(Xd=1,(function(f){function r(O,C){var x=O.length;O.push(C);t:for(;0<x;){var ot=x-1>>>1,ht=O[ot];if(0<A(ht,C))O[ot]=C,O[x]=ht,x=ot;else break t}}function m(O){return O.length===0?null:O[0]}function s(O){if(O.length===0)return null;var C=O[0],x=O.pop();if(x!==C){O[0]=x;t:for(var ot=0,ht=O.length,y=ht>>>1;ot<y;){var D=2*(ot+1)-1,q=O[D],j=D+1,Z=O[j];if(0>A(q,x))j<ht&&0>A(Z,q)?(O[ot]=Z,O[j]=x,ot=j):(O[ot]=q,O[D]=x,ot=D);else if(j<ht&&0>A(Z,x))O[ot]=Z,O[j]=x,ot=j;else break t}}return C}function A(O,C){var x=O.sortIndex-C.sortIndex;return x!==0?x:O.id-C.id}if(f.unstable_now=void 0,typeof performance=="object"&&typeof performance.now=="function"){var p=performance;f.unstable_now=function(){return p.now()}}else{var U=Date,H=U.now();f.unstable_now=function(){return U.now()-H}}var T=[],v=[],R=1,N=null,K=3,tt=!1,Q=!1,et=!1,vt=!1,it=typeof setTimeout=="function"?setTimeout:null,qt=typeof clearTimeout=="function"?clearTimeout:null,At=typeof setImmediate<"u"?setImmediate:null;function Jt(O){for(var C=m(v);C!==null;){if(C.callback===null)s(v);else if(C.startTime<=O)s(v),C.sortIndex=C.expirationTime,r(T,C);else break;C=m(v)}}function wt(O){if(et=!1,Jt(O),!Q)if(m(T)!==null)Q=!0,Nt||(Nt=!0,w());else{var C=m(v);C!==null&&Xt(wt,C.startTime-O)}}var Nt=!1,$=-1,Gt=5,ul=-1;function Ml(){return vt?!0:!(f.unstable_now()-ul<Gt)}function vl(){if(vt=!1,Nt){var O=f.unstable_now();ul=O;var C=!0;try{t:{Q=!1,et&&(et=!1,qt($),$=-1),tt=!0;var x=K;try{l:{for(Jt(O),N=m(T);N!==null&&!(N.expirationTime>O&&Ml());){var ot=N.callback;if(typeof ot=="function"){N.callback=null,K=N.priorityLevel;var ht=ot(N.expirationTime<=O);if(O=f.unstable_now(),typeof ht=="function"){N.callback=ht,Jt(O),C=!0;break l}N===m(T)&&s(T),Jt(O)}else s(T);N=m(T)}if(N!==null)C=!0;else{var y=m(v);y!==null&&Xt(wt,y.startTime-O),C=!1}}break t}finally{N=null,K=x,tt=!1}C=void 0}}finally{C?w():Nt=!1}}}var w;if(typeof At=="function")w=function(){At(vl)};else if(typeof MessageChannel<"u"){var Tt=new MessageChannel,Dl=Tt.port2;Tt.port1.onmessage=vl,w=function(){Dl.postMessage(null)}}else w=function(){it(vl,0)};function Xt(O,C){$=it(function(){O(f.unstable_now())},C)}f.unstable_IdlePriority=5,f.unstable_ImmediatePriority=1,f.unstable_LowPriority=4,f.unstable_NormalPriority=3,f.unstable_Profiling=null,f.unstable_UserBlockingPriority=2,f.unstable_cancelCallback=function(O){O.callback=null},f.unstable_forceFrameRate=function(O){0>O||125<O?console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported"):Gt=0<O?Math.floor(1e3/O):5},f.unstable_getCurrentPriorityLevel=function(){return K},f.unstable_next=function(O){switch(K){case 1:case 2:case 3:var C=3;break;default:C=K}var x=K;K=C;try{return O()}finally{K=x}},f.unstable_requestPaint=function(){vt=!0},f.unstable_runWithPriority=function(O,C){switch(O){case 1:case 2:case 3:case 4:case 5:break;default:O=3}var x=K;K=O;try{return C()}finally{K=x}},f.unstable_scheduleCallback=function(O,C,x){var ot=f.unstable_now();switch(typeof x=="object"&&x!==null?(x=x.delay,x=typeof x=="number"&&0<x?ot+x:ot):x=ot,O){case 1:var ht=-1;break;case 2:ht=250;break;case 5:ht=1073741823;break;case 4:ht=1e4;break;default:ht=5e3}return ht=x+ht,O={id:R++,callback:C,priorityLevel:O,startTime:x,expirationTime:ht,sortIndex:-1},x>ot?(O.sortIndex=x,r(v,O),m(T)===null&&O===m(v)&&(et?(qt($),$=-1):et=!0,Xt(wt,x-ot))):(O.sortIndex=ht,r(T,O),Q||tt||(Q=!0,Nt||(Nt=!0,w()))),O},f.unstable_shouldYield=Ml,f.unstable_wrapCallback=function(O){var C=K;return function(){var x=K;K=C;try{return O.apply(this,arguments)}finally{K=x}}}})(Tf)),Tf}var xd;function kv(){return xd||(xd=1,bf.exports=Wv()),bf.exports}var Ef={exports:{}},L={};/**
 * @license React
 * react.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Qd;function Fv(){if(Qd)return L;Qd=1;var f=Symbol.for("react.transitional.element"),r=Symbol.for("react.portal"),m=Symbol.for("react.fragment"),s=Symbol.for("react.strict_mode"),A=Symbol.for("react.profiler"),p=Symbol.for("react.consumer"),U=Symbol.for("react.context"),H=Symbol.for("react.forward_ref"),T=Symbol.for("react.suspense"),v=Symbol.for("react.memo"),R=Symbol.for("react.lazy"),N=Symbol.for("react.activity"),K=Symbol.iterator;function tt(y){return y===null||typeof y!="object"?null:(y=K&&y[K]||y["@@iterator"],typeof y=="function"?y:null)}var Q={isMounted:function(){return!1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},et=Object.assign,vt={};function it(y,D,q){this.props=y,this.context=D,this.refs=vt,this.updater=q||Q}it.prototype.isReactComponent={},it.prototype.setState=function(y,D){if(typeof y!="object"&&typeof y!="function"&&y!=null)throw Error("takes an object of state variables to update or a function which returns an object of state variables.");this.updater.enqueueSetState(this,y,D,"setState")},it.prototype.forceUpdate=function(y){this.updater.enqueueForceUpdate(this,y,"forceUpdate")};function qt(){}qt.prototype=it.prototype;function At(y,D,q){this.props=y,this.context=D,this.refs=vt,this.updater=q||Q}var Jt=At.prototype=new qt;Jt.constructor=At,et(Jt,it.prototype),Jt.isPureReactComponent=!0;var wt=Array.isArray;function Nt(){}var $={H:null,A:null,T:null,S:null},Gt=Object.prototype.hasOwnProperty;function ul(y,D,q){var j=q.ref;return{$$typeof:f,type:y,key:D,ref:j!==void 0?j:null,props:q}}function Ml(y,D){return ul(y.type,D,y.props)}function vl(y){return typeof y=="object"&&y!==null&&y.$$typeof===f}function w(y){var D={"=":"=0",":":"=2"};return"$"+y.replace(/[=:]/g,function(q){return D[q]})}var Tt=/\/+/g;function Dl(y,D){return typeof y=="object"&&y!==null&&y.key!=null?w(""+y.key):D.toString(36)}function Xt(y){switch(y.status){case"fulfilled":return y.value;case"rejected":throw y.reason;default:switch(typeof y.status=="string"?y.then(Nt,Nt):(y.status="pending",y.then(function(D){y.status==="pending"&&(y.status="fulfilled",y.value=D)},function(D){y.status==="pending"&&(y.status="rejected",y.reason=D)})),y.status){case"fulfilled":return y.value;case"rejected":throw y.reason}}throw y}function O(y,D,q,j,Z){var W=typeof y;(W==="undefined"||W==="boolean")&&(y=null);var ct=!1;if(y===null)ct=!0;else switch(W){case"bigint":case"string":case"number":ct=!0;break;case"object":switch(y.$$typeof){case f:case r:ct=!0;break;case R:return ct=y._init,O(ct(y._payload),D,q,j,Z)}}if(ct)return Z=Z(y),ct=j===""?"."+Dl(y,0):j,wt(Z)?(q="",ct!=null&&(q=ct.replace(Tt,"$&/")+"/"),O(Z,D,q,"",function(Na){return Na})):Z!=null&&(vl(Z)&&(Z=Ml(Z,q+(Z.key==null||y&&y.key===Z.key?"":(""+Z.key).replace(Tt,"$&/")+"/")+ct)),D.push(Z)),1;ct=0;var $t=j===""?".":j+":";if(wt(y))for(var _t=0;_t<y.length;_t++)j=y[_t],W=$t+Dl(j,_t),ct+=O(j,D,q,W,Z);else if(_t=tt(y),typeof _t=="function")for(y=_t.call(y),_t=0;!(j=y.next()).done;)j=j.value,W=$t+Dl(j,_t++),ct+=O(j,D,q,W,Z);else if(W==="object"){if(typeof y.then=="function")return O(Xt(y),D,q,j,Z);throw D=String(y),Error("Objects are not valid as a React child (found: "+(D==="[object Object]"?"object with keys {"+Object.keys(y).join(", ")+"}":D)+"). If you meant to render a collection of children, use an array instead.")}return ct}function C(y,D,q){if(y==null)return y;var j=[],Z=0;return O(y,j,"","",function(W){return D.call(q,W,Z++)}),j}function x(y){if(y._status===-1){var D=y._result;D=D(),D.then(function(q){(y._status===0||y._status===-1)&&(y._status=1,y._result=q)},function(q){(y._status===0||y._status===-1)&&(y._status=2,y._result=q)}),y._status===-1&&(y._status=0,y._result=D)}if(y._status===1)return y._result.default;throw y._result}var ot=typeof reportError=="function"?reportError:function(y){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var D=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof y=="object"&&y!==null&&typeof y.message=="string"?String(y.message):String(y),error:y});if(!window.dispatchEvent(D))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",y);return}console.error(y)},ht={map:C,forEach:function(y,D,q){C(y,function(){D.apply(this,arguments)},q)},count:function(y){var D=0;return C(y,function(){D++}),D},toArray:function(y){return C(y,function(D){return D})||[]},only:function(y){if(!vl(y))throw Error("React.Children.only expected to receive a single React element child.");return y}};return L.Activity=N,L.Children=ht,L.Component=it,L.Fragment=m,L.Profiler=A,L.PureComponent=At,L.StrictMode=s,L.Suspense=T,L.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=$,L.__COMPILER_RUNTIME={__proto__:null,c:function(y){return $.H.useMemoCache(y)}},L.cache=function(y){return function(){return y.apply(null,arguments)}},L.cacheSignal=function(){return null},L.cloneElement=function(y,D,q){if(y==null)throw Error("The argument must be a React element, but you passed "+y+".");var j=et({},y.props),Z=y.key;if(D!=null)for(W in D.key!==void 0&&(Z=""+D.key),D)!Gt.call(D,W)||W==="key"||W==="__self"||W==="__source"||W==="ref"&&D.ref===void 0||(j[W]=D[W]);var W=arguments.length-2;if(W===1)j.children=q;else if(1<W){for(var ct=Array(W),$t=0;$t<W;$t++)ct[$t]=arguments[$t+2];j.children=ct}return ul(y.type,Z,j)},L.createContext=function(y){return y={$$typeof:U,_currentValue:y,_currentValue2:y,_threadCount:0,Provider:null,Consumer:null},y.Provider=y,y.Consumer={$$typeof:p,_context:y},y},L.createElement=function(y,D,q){var j,Z={},W=null;if(D!=null)for(j in D.key!==void 0&&(W=""+D.key),D)Gt.call(D,j)&&j!=="key"&&j!=="__self"&&j!=="__source"&&(Z[j]=D[j]);var ct=arguments.length-2;if(ct===1)Z.children=q;else if(1<ct){for(var $t=Array(ct),_t=0;_t<ct;_t++)$t[_t]=arguments[_t+2];Z.children=$t}if(y&&y.defaultProps)for(j in ct=y.defaultProps,ct)Z[j]===void 0&&(Z[j]=ct[j]);return ul(y,W,Z)},L.createRef=function(){return{current:null}},L.forwardRef=function(y){return{$$typeof:H,render:y}},L.isValidElement=vl,L.lazy=function(y){return{$$typeof:R,_payload:{_status:-1,_result:y},_init:x}},L.memo=function(y,D){return{$$typeof:v,type:y,compare:D===void 0?null:D}},L.startTransition=function(y){var D=$.T,q={};$.T=q;try{var j=y(),Z=$.S;Z!==null&&Z(q,j),typeof j=="object"&&j!==null&&typeof j.then=="function"&&j.then(Nt,ot)}catch(W){ot(W)}finally{D!==null&&q.types!==null&&(D.types=q.types),$.T=D}},L.unstable_useCacheRefresh=function(){return $.H.useCacheRefresh()},L.use=function(y){return $.H.use(y)},L.useActionState=function(y,D,q){return $.H.useActionState(y,D,q)},L.useCallback=function(y,D){return $.H.useCallback(y,D)},L.useContext=function(y){return $.H.useContext(y)},L.useDebugValue=function(){},L.useDeferredValue=function(y,D){return $.H.useDeferredValue(y,D)},L.useEffect=function(y,D){return $.H.useEffect(y,D)},L.useEffectEvent=function(y){return $.H.useEffectEvent(y)},L.useId=function(){return $.H.useId()},L.useImperativeHandle=function(y,D,q){return $.H.useImperativeHandle(y,D,q)},L.useInsertionEffect=function(y,D){return $.H.useInsertionEffect(y,D)},L.useLayoutEffect=function(y,D){return $.H.useLayoutEffect(y,D)},L.useMemo=function(y,D){return $.H.useMemo(y,D)},L.useOptimistic=function(y,D){return $.H.useOptimistic(y,D)},L.useReducer=function(y,D,q){return $.H.useReducer(y,D,q)},L.useRef=function(y){return $.H.useRef(y)},L.useState=function(y){return $.H.useState(y)},L.useSyncExternalStore=function(y,D,q){return $.H.useSyncExternalStore(y,D,q)},L.useTransition=function(){return $.H.useTransition()},L.version="19.2.4",L}var Ld;function Nf(){return Ld||(Ld=1,Ef.exports=Fv()),Ef.exports}var Af={exports:{}},Kt={};/**
 * @license React
 * react-dom.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Zd;function Iv(){if(Zd)return Kt;Zd=1;var f=Nf();function r(T){var v="https://react.dev/errors/"+T;if(1<arguments.length){v+="?args[]="+encodeURIComponent(arguments[1]);for(var R=2;R<arguments.length;R++)v+="&args[]="+encodeURIComponent(arguments[R])}return"Minified React error #"+T+"; visit "+v+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function m(){}var s={d:{f:m,r:function(){throw Error(r(522))},D:m,C:m,L:m,m,X:m,S:m,M:m},p:0,findDOMNode:null},A=Symbol.for("react.portal");function p(T,v,R){var N=3<arguments.length&&arguments[3]!==void 0?arguments[3]:null;return{$$typeof:A,key:N==null?null:""+N,children:T,containerInfo:v,implementation:R}}var U=f.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE;function H(T,v){if(T==="font")return"";if(typeof v=="string")return v==="use-credentials"?v:""}return Kt.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=s,Kt.createPortal=function(T,v){var R=2<arguments.length&&arguments[2]!==void 0?arguments[2]:null;if(!v||v.nodeType!==1&&v.nodeType!==9&&v.nodeType!==11)throw Error(r(299));return p(T,v,null,R)},Kt.flushSync=function(T){var v=U.T,R=s.p;try{if(U.T=null,s.p=2,T)return T()}finally{U.T=v,s.p=R,s.d.f()}},Kt.preconnect=function(T,v){typeof T=="string"&&(v?(v=v.crossOrigin,v=typeof v=="string"?v==="use-credentials"?v:"":void 0):v=null,s.d.C(T,v))},Kt.prefetchDNS=function(T){typeof T=="string"&&s.d.D(T)},Kt.preinit=function(T,v){if(typeof T=="string"&&v&&typeof v.as=="string"){var R=v.as,N=H(R,v.crossOrigin),K=typeof v.integrity=="string"?v.integrity:void 0,tt=typeof v.fetchPriority=="string"?v.fetchPriority:void 0;R==="style"?s.d.S(T,typeof v.precedence=="string"?v.precedence:void 0,{crossOrigin:N,integrity:K,fetchPriority:tt}):R==="script"&&s.d.X(T,{crossOrigin:N,integrity:K,fetchPriority:tt,nonce:typeof v.nonce=="string"?v.nonce:void 0})}},Kt.preinitModule=function(T,v){if(typeof T=="string")if(typeof v=="object"&&v!==null){if(v.as==null||v.as==="script"){var R=H(v.as,v.crossOrigin);s.d.M(T,{crossOrigin:R,integrity:typeof v.integrity=="string"?v.integrity:void 0,nonce:typeof v.nonce=="string"?v.nonce:void 0})}}else v==null&&s.d.M(T)},Kt.preload=function(T,v){if(typeof T=="string"&&typeof v=="object"&&v!==null&&typeof v.as=="string"){var R=v.as,N=H(R,v.crossOrigin);s.d.L(T,R,{crossOrigin:N,integrity:typeof v.integrity=="string"?v.integrity:void 0,nonce:typeof v.nonce=="string"?v.nonce:void 0,type:typeof v.type=="string"?v.type:void 0,fetchPriority:typeof v.fetchPriority=="string"?v.fetchPriority:void 0,referrerPolicy:typeof v.referrerPolicy=="string"?v.referrerPolicy:void 0,imageSrcSet:typeof v.imageSrcSet=="string"?v.imageSrcSet:void 0,imageSizes:typeof v.imageSizes=="string"?v.imageSizes:void 0,media:typeof v.media=="string"?v.media:void 0})}},Kt.preloadModule=function(T,v){if(typeof T=="string")if(v){var R=H(v.as,v.crossOrigin);s.d.m(T,{as:typeof v.as=="string"&&v.as!=="script"?v.as:void 0,crossOrigin:R,integrity:typeof v.integrity=="string"?v.integrity:void 0})}else s.d.m(T)},Kt.requestFormReset=function(T){s.d.r(T)},Kt.unstable_batchedUpdates=function(T,v){return T(v)},Kt.useFormState=function(T,v,R){return U.H.useFormState(T,v,R)},Kt.useFormStatus=function(){return U.H.useHostTransitionStatus()},Kt.version="19.2.4",Kt}var Vd;function Pv(){if(Vd)return Af.exports;Vd=1;function f(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(f)}catch(r){console.error(r)}}return f(),Af.exports=Iv(),Af.exports}/**
 * @license React
 * react-dom-client.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Kd;function t0(){if(Kd)return Mu;Kd=1;var f=kv(),r=Nf(),m=Pv();function s(t){var l="https://react.dev/errors/"+t;if(1<arguments.length){l+="?args[]="+encodeURIComponent(arguments[1]);for(var e=2;e<arguments.length;e++)l+="&args[]="+encodeURIComponent(arguments[e])}return"Minified React error #"+t+"; visit "+l+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function A(t){return!(!t||t.nodeType!==1&&t.nodeType!==9&&t.nodeType!==11)}function p(t){var l=t,e=t;if(t.alternate)for(;l.return;)l=l.return;else{t=l;do l=t,(l.flags&4098)!==0&&(e=l.return),t=l.return;while(t)}return l.tag===3?e:null}function U(t){if(t.tag===13){var l=t.memoizedState;if(l===null&&(t=t.alternate,t!==null&&(l=t.memoizedState)),l!==null)return l.dehydrated}return null}function H(t){if(t.tag===31){var l=t.memoizedState;if(l===null&&(t=t.alternate,t!==null&&(l=t.memoizedState)),l!==null)return l.dehydrated}return null}function T(t){if(p(t)!==t)throw Error(s(188))}function v(t){var l=t.alternate;if(!l){if(l=p(t),l===null)throw Error(s(188));return l!==t?null:t}for(var e=t,a=l;;){var u=e.return;if(u===null)break;var n=u.alternate;if(n===null){if(a=u.return,a!==null){e=a;continue}break}if(u.child===n.child){for(n=u.child;n;){if(n===e)return T(u),t;if(n===a)return T(u),l;n=n.sibling}throw Error(s(188))}if(e.return!==a.return)e=u,a=n;else{for(var i=!1,c=u.child;c;){if(c===e){i=!0,e=u,a=n;break}if(c===a){i=!0,a=u,e=n;break}c=c.sibling}if(!i){for(c=n.child;c;){if(c===e){i=!0,e=n,a=u;break}if(c===a){i=!0,a=n,e=u;break}c=c.sibling}if(!i)throw Error(s(189))}}if(e.alternate!==a)throw Error(s(190))}if(e.tag!==3)throw Error(s(188));return e.stateNode.current===e?t:l}function R(t){var l=t.tag;if(l===5||l===26||l===27||l===6)return t;for(t=t.child;t!==null;){if(l=R(t),l!==null)return l;t=t.sibling}return null}var N=Object.assign,K=Symbol.for("react.element"),tt=Symbol.for("react.transitional.element"),Q=Symbol.for("react.portal"),et=Symbol.for("react.fragment"),vt=Symbol.for("react.strict_mode"),it=Symbol.for("react.profiler"),qt=Symbol.for("react.consumer"),At=Symbol.for("react.context"),Jt=Symbol.for("react.forward_ref"),wt=Symbol.for("react.suspense"),Nt=Symbol.for("react.suspense_list"),$=Symbol.for("react.memo"),Gt=Symbol.for("react.lazy"),ul=Symbol.for("react.activity"),Ml=Symbol.for("react.memo_cache_sentinel"),vl=Symbol.iterator;function w(t){return t===null||typeof t!="object"?null:(t=vl&&t[vl]||t["@@iterator"],typeof t=="function"?t:null)}var Tt=Symbol.for("react.client.reference");function Dl(t){if(t==null)return null;if(typeof t=="function")return t.$$typeof===Tt?null:t.displayName||t.name||null;if(typeof t=="string")return t;switch(t){case et:return"Fragment";case it:return"Profiler";case vt:return"StrictMode";case wt:return"Suspense";case Nt:return"SuspenseList";case ul:return"Activity"}if(typeof t=="object")switch(t.$$typeof){case Q:return"Portal";case At:return t.displayName||"Context";case qt:return(t._context.displayName||"Context")+".Consumer";case Jt:var l=t.render;return t=t.displayName,t||(t=l.displayName||l.name||"",t=t!==""?"ForwardRef("+t+")":"ForwardRef"),t;case $:return l=t.displayName||null,l!==null?l:Dl(t.type)||"Memo";case Gt:l=t._payload,t=t._init;try{return Dl(t(l))}catch{}}return null}var Xt=Array.isArray,O=r.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,C=m.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,x={pending:!1,data:null,method:null,action:null},ot=[],ht=-1;function y(t){return{current:t}}function D(t){0>ht||(t.current=ot[ht],ot[ht]=null,ht--)}function q(t,l){ht++,ot[ht]=t.current,t.current=l}var j=y(null),Z=y(null),W=y(null),ct=y(null);function $t(t,l){switch(q(W,l),q(Z,t),q(j,null),l.nodeType){case 9:case 11:t=(t=l.documentElement)&&(t=t.namespaceURI)?fd(t):0;break;default:if(t=l.tagName,l=l.namespaceURI)l=fd(l),t=sd(l,t);else switch(t){case"svg":t=1;break;case"math":t=2;break;default:t=0}}D(j),q(j,t)}function _t(){D(j),D(Z),D(W)}function Na(t){t.memoizedState!==null&&q(ct,t);var l=j.current,e=sd(l,t.type);l!==e&&(q(Z,t),q(j,e))}function Cu(t){Z.current===t&&(D(j),D(Z)),ct.current===t&&(D(ct),pu._currentValue=x)}var Pn,Yf;function Me(t){if(Pn===void 0)try{throw Error()}catch(e){var l=e.stack.trim().match(/\n( *(at )?)/);Pn=l&&l[1]||"",Yf=-1<e.stack.indexOf(`
    at`)?" (<anonymous>)":-1<e.stack.indexOf("@")?"@unknown:0:0":""}return`
`+Pn+t+Yf}var ti=!1;function li(t,l){if(!t||ti)return"";ti=!0;var e=Error.prepareStackTrace;Error.prepareStackTrace=void 0;try{var a={DetermineComponentFrameRoot:function(){try{if(l){var M=function(){throw Error()};if(Object.defineProperty(M.prototype,"props",{set:function(){throw Error()}}),typeof Reflect=="object"&&Reflect.construct){try{Reflect.construct(M,[])}catch(E){var b=E}Reflect.construct(t,[],M)}else{try{M.call()}catch(E){b=E}t.call(M.prototype)}}else{try{throw Error()}catch(E){b=E}(M=t())&&typeof M.catch=="function"&&M.catch(function(){})}}catch(E){if(E&&b&&typeof E.stack=="string")return[E.stack,b.stack]}return[null,null]}};a.DetermineComponentFrameRoot.displayName="DetermineComponentFrameRoot";var u=Object.getOwnPropertyDescriptor(a.DetermineComponentFrameRoot,"name");u&&u.configurable&&Object.defineProperty(a.DetermineComponentFrameRoot,"name",{value:"DetermineComponentFrameRoot"});var n=a.DetermineComponentFrameRoot(),i=n[0],c=n[1];if(i&&c){var o=i.split(`
`),S=c.split(`
`);for(u=a=0;a<o.length&&!o[a].includes("DetermineComponentFrameRoot");)a++;for(;u<S.length&&!S[u].includes("DetermineComponentFrameRoot");)u++;if(a===o.length||u===S.length)for(a=o.length-1,u=S.length-1;1<=a&&0<=u&&o[a]!==S[u];)u--;for(;1<=a&&0<=u;a--,u--)if(o[a]!==S[u]){if(a!==1||u!==1)do if(a--,u--,0>u||o[a]!==S[u]){var z=`
`+o[a].replace(" at new "," at ");return t.displayName&&z.includes("<anonymous>")&&(z=z.replace("<anonymous>",t.displayName)),z}while(1<=a&&0<=u);break}}}finally{ti=!1,Error.prepareStackTrace=e}return(e=t?t.displayName||t.name:"")?Me(e):""}function Om(t,l){switch(t.tag){case 26:case 27:case 5:return Me(t.type);case 16:return Me("Lazy");case 13:return t.child!==l&&l!==null?Me("Suspense Fallback"):Me("Suspense");case 19:return Me("SuspenseList");case 0:case 15:return li(t.type,!1);case 11:return li(t.type.render,!1);case 1:return li(t.type,!0);case 31:return Me("Activity");default:return""}}function Gf(t){try{var l="",e=null;do l+=Om(t,e),e=t,t=t.return;while(t);return l}catch(a){return`
Error generating stack: `+a.message+`
`+a.stack}}var ei=Object.prototype.hasOwnProperty,ai=f.unstable_scheduleCallback,ui=f.unstable_cancelCallback,_m=f.unstable_shouldYield,Mm=f.unstable_requestPaint,nl=f.unstable_now,Dm=f.unstable_getCurrentPriorityLevel,Xf=f.unstable_ImmediatePriority,xf=f.unstable_UserBlockingPriority,Ru=f.unstable_NormalPriority,Um=f.unstable_LowPriority,Qf=f.unstable_IdlePriority,Hm=f.log,Cm=f.unstable_setDisableYieldValue,Ba=null,il=null;function le(t){if(typeof Hm=="function"&&Cm(t),il&&typeof il.setStrictMode=="function")try{il.setStrictMode(Ba,t)}catch{}}var cl=Math.clz32?Math.clz32:Nm,Rm=Math.log,qm=Math.LN2;function Nm(t){return t>>>=0,t===0?32:31-(Rm(t)/qm|0)|0}var qu=256,Nu=262144,Bu=4194304;function De(t){var l=t&42;if(l!==0)return l;switch(t&-t){case 1:return 1;case 2:return 2;case 4:return 4;case 8:return 8;case 16:return 16;case 32:return 32;case 64:return 64;case 128:return 128;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:return t&261888;case 262144:case 524288:case 1048576:case 2097152:return t&3932160;case 4194304:case 8388608:case 16777216:case 33554432:return t&62914560;case 67108864:return 67108864;case 134217728:return 134217728;case 268435456:return 268435456;case 536870912:return 536870912;case 1073741824:return 0;default:return t}}function ju(t,l,e){var a=t.pendingLanes;if(a===0)return 0;var u=0,n=t.suspendedLanes,i=t.pingedLanes;t=t.warmLanes;var c=a&134217727;return c!==0?(a=c&~n,a!==0?u=De(a):(i&=c,i!==0?u=De(i):e||(e=c&~t,e!==0&&(u=De(e))))):(c=a&~n,c!==0?u=De(c):i!==0?u=De(i):e||(e=a&~t,e!==0&&(u=De(e)))),u===0?0:l!==0&&l!==u&&(l&n)===0&&(n=u&-u,e=l&-l,n>=e||n===32&&(e&4194048)!==0)?l:u}function ja(t,l){return(t.pendingLanes&~(t.suspendedLanes&~t.pingedLanes)&l)===0}function Bm(t,l){switch(t){case 1:case 2:case 4:case 8:case 64:return l+250;case 16:case 32:case 128:case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return l+5e3;case 4194304:case 8388608:case 16777216:case 33554432:return-1;case 67108864:case 134217728:case 268435456:case 536870912:case 1073741824:return-1;default:return-1}}function Lf(){var t=Bu;return Bu<<=1,(Bu&62914560)===0&&(Bu=4194304),t}function ni(t){for(var l=[],e=0;31>e;e++)l.push(t);return l}function Ya(t,l){t.pendingLanes|=l,l!==268435456&&(t.suspendedLanes=0,t.pingedLanes=0,t.warmLanes=0)}function jm(t,l,e,a,u,n){var i=t.pendingLanes;t.pendingLanes=e,t.suspendedLanes=0,t.pingedLanes=0,t.warmLanes=0,t.expiredLanes&=e,t.entangledLanes&=e,t.errorRecoveryDisabledLanes&=e,t.shellSuspendCounter=0;var c=t.entanglements,o=t.expirationTimes,S=t.hiddenUpdates;for(e=i&~e;0<e;){var z=31-cl(e),M=1<<z;c[z]=0,o[z]=-1;var b=S[z];if(b!==null)for(S[z]=null,z=0;z<b.length;z++){var E=b[z];E!==null&&(E.lane&=-536870913)}e&=~M}a!==0&&Zf(t,a,0),n!==0&&u===0&&t.tag!==0&&(t.suspendedLanes|=n&~(i&~l))}function Zf(t,l,e){t.pendingLanes|=l,t.suspendedLanes&=~l;var a=31-cl(l);t.entangledLanes|=l,t.entanglements[a]=t.entanglements[a]|1073741824|e&261930}function Vf(t,l){var e=t.entangledLanes|=l;for(t=t.entanglements;e;){var a=31-cl(e),u=1<<a;u&l|t[a]&l&&(t[a]|=l),e&=~u}}function Kf(t,l){var e=l&-l;return e=(e&42)!==0?1:ii(e),(e&(t.suspendedLanes|l))!==0?0:e}function ii(t){switch(t){case 2:t=1;break;case 8:t=4;break;case 32:t=16;break;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:case 4194304:case 8388608:case 16777216:case 33554432:t=128;break;case 268435456:t=134217728;break;default:t=0}return t}function ci(t){return t&=-t,2<t?8<t?(t&134217727)!==0?32:268435456:8:2}function Jf(){var t=C.p;return t!==0?t:(t=window.event,t===void 0?32:Rd(t.type))}function wf(t,l){var e=C.p;try{return C.p=t,l()}finally{C.p=e}}var ee=Math.random().toString(36).slice(2),xt="__reactFiber$"+ee,kt="__reactProps$"+ee,Je="__reactContainer$"+ee,fi="__reactEvents$"+ee,Ym="__reactListeners$"+ee,Gm="__reactHandles$"+ee,$f="__reactResources$"+ee,Ga="__reactMarker$"+ee;function si(t){delete t[xt],delete t[kt],delete t[fi],delete t[Ym],delete t[Gm]}function we(t){var l=t[xt];if(l)return l;for(var e=t.parentNode;e;){if(l=e[Je]||e[xt]){if(e=l.alternate,l.child!==null||e!==null&&e.child!==null)for(t=hd(t);t!==null;){if(e=t[xt])return e;t=hd(t)}return l}t=e,e=t.parentNode}return null}function $e(t){if(t=t[xt]||t[Je]){var l=t.tag;if(l===5||l===6||l===13||l===31||l===26||l===27||l===3)return t}return null}function Xa(t){var l=t.tag;if(l===5||l===26||l===27||l===6)return t.stateNode;throw Error(s(33))}function We(t){var l=t[$f];return l||(l=t[$f]={hoistableStyles:new Map,hoistableScripts:new Map}),l}function jt(t){t[Ga]=!0}var Wf=new Set,kf={};function Ue(t,l){ke(t,l),ke(t+"Capture",l)}function ke(t,l){for(kf[t]=l,t=0;t<l.length;t++)Wf.add(l[t])}var Xm=RegExp("^[:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD][:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040]*$"),Ff={},If={};function xm(t){return ei.call(If,t)?!0:ei.call(Ff,t)?!1:Xm.test(t)?If[t]=!0:(Ff[t]=!0,!1)}function Yu(t,l,e){if(xm(l))if(e===null)t.removeAttribute(l);else{switch(typeof e){case"undefined":case"function":case"symbol":t.removeAttribute(l);return;case"boolean":var a=l.toLowerCase().slice(0,5);if(a!=="data-"&&a!=="aria-"){t.removeAttribute(l);return}}t.setAttribute(l,""+e)}}function Gu(t,l,e){if(e===null)t.removeAttribute(l);else{switch(typeof e){case"undefined":case"function":case"symbol":case"boolean":t.removeAttribute(l);return}t.setAttribute(l,""+e)}}function Yl(t,l,e,a){if(a===null)t.removeAttribute(e);else{switch(typeof a){case"undefined":case"function":case"symbol":case"boolean":t.removeAttribute(e);return}t.setAttributeNS(l,e,""+a)}}function hl(t){switch(typeof t){case"bigint":case"boolean":case"number":case"string":case"undefined":return t;case"object":return t;default:return""}}function Pf(t){var l=t.type;return(t=t.nodeName)&&t.toLowerCase()==="input"&&(l==="checkbox"||l==="radio")}function Qm(t,l,e){var a=Object.getOwnPropertyDescriptor(t.constructor.prototype,l);if(!t.hasOwnProperty(l)&&typeof a<"u"&&typeof a.get=="function"&&typeof a.set=="function"){var u=a.get,n=a.set;return Object.defineProperty(t,l,{configurable:!0,get:function(){return u.call(this)},set:function(i){e=""+i,n.call(this,i)}}),Object.defineProperty(t,l,{enumerable:a.enumerable}),{getValue:function(){return e},setValue:function(i){e=""+i},stopTracking:function(){t._valueTracker=null,delete t[l]}}}}function oi(t){if(!t._valueTracker){var l=Pf(t)?"checked":"value";t._valueTracker=Qm(t,l,""+t[l])}}function ts(t){if(!t)return!1;var l=t._valueTracker;if(!l)return!0;var e=l.getValue(),a="";return t&&(a=Pf(t)?t.checked?"true":"false":t.value),t=a,t!==e?(l.setValue(t),!0):!1}function Xu(t){if(t=t||(typeof document<"u"?document:void 0),typeof t>"u")return null;try{return t.activeElement||t.body}catch{return t.body}}var Lm=/[\n"\\]/g;function gl(t){return t.replace(Lm,function(l){return"\\"+l.charCodeAt(0).toString(16)+" "})}function ri(t,l,e,a,u,n,i,c){t.name="",i!=null&&typeof i!="function"&&typeof i!="symbol"&&typeof i!="boolean"?t.type=i:t.removeAttribute("type"),l!=null?i==="number"?(l===0&&t.value===""||t.value!=l)&&(t.value=""+hl(l)):t.value!==""+hl(l)&&(t.value=""+hl(l)):i!=="submit"&&i!=="reset"||t.removeAttribute("value"),l!=null?di(t,i,hl(l)):e!=null?di(t,i,hl(e)):a!=null&&t.removeAttribute("value"),u==null&&n!=null&&(t.defaultChecked=!!n),u!=null&&(t.checked=u&&typeof u!="function"&&typeof u!="symbol"),c!=null&&typeof c!="function"&&typeof c!="symbol"&&typeof c!="boolean"?t.name=""+hl(c):t.removeAttribute("name")}function ls(t,l,e,a,u,n,i,c){if(n!=null&&typeof n!="function"&&typeof n!="symbol"&&typeof n!="boolean"&&(t.type=n),l!=null||e!=null){if(!(n!=="submit"&&n!=="reset"||l!=null)){oi(t);return}e=e!=null?""+hl(e):"",l=l!=null?""+hl(l):e,c||l===t.value||(t.value=l),t.defaultValue=l}a=a??u,a=typeof a!="function"&&typeof a!="symbol"&&!!a,t.checked=c?t.checked:!!a,t.defaultChecked=!!a,i!=null&&typeof i!="function"&&typeof i!="symbol"&&typeof i!="boolean"&&(t.name=i),oi(t)}function di(t,l,e){l==="number"&&Xu(t.ownerDocument)===t||t.defaultValue===""+e||(t.defaultValue=""+e)}function Fe(t,l,e,a){if(t=t.options,l){l={};for(var u=0;u<e.length;u++)l["$"+e[u]]=!0;for(e=0;e<t.length;e++)u=l.hasOwnProperty("$"+t[e].value),t[e].selected!==u&&(t[e].selected=u),u&&a&&(t[e].defaultSelected=!0)}else{for(e=""+hl(e),l=null,u=0;u<t.length;u++){if(t[u].value===e){t[u].selected=!0,a&&(t[u].defaultSelected=!0);return}l!==null||t[u].disabled||(l=t[u])}l!==null&&(l.selected=!0)}}function es(t,l,e){if(l!=null&&(l=""+hl(l),l!==t.value&&(t.value=l),e==null)){t.defaultValue!==l&&(t.defaultValue=l);return}t.defaultValue=e!=null?""+hl(e):""}function as(t,l,e,a){if(l==null){if(a!=null){if(e!=null)throw Error(s(92));if(Xt(a)){if(1<a.length)throw Error(s(93));a=a[0]}e=a}e==null&&(e=""),l=e}e=hl(l),t.defaultValue=e,a=t.textContent,a===e&&a!==""&&a!==null&&(t.value=a),oi(t)}function Ie(t,l){if(l){var e=t.firstChild;if(e&&e===t.lastChild&&e.nodeType===3){e.nodeValue=l;return}}t.textContent=l}var Zm=new Set("animationIterationCount aspectRatio borderImageOutset borderImageSlice borderImageWidth boxFlex boxFlexGroup boxOrdinalGroup columnCount columns flex flexGrow flexPositive flexShrink flexNegative flexOrder gridArea gridRow gridRowEnd gridRowSpan gridRowStart gridColumn gridColumnEnd gridColumnSpan gridColumnStart fontWeight lineClamp lineHeight opacity order orphans scale tabSize widows zIndex zoom fillOpacity floodOpacity stopOpacity strokeDasharray strokeDashoffset strokeMiterlimit strokeOpacity strokeWidth MozAnimationIterationCount MozBoxFlex MozBoxFlexGroup MozLineClamp msAnimationIterationCount msFlex msZoom msFlexGrow msFlexNegative msFlexOrder msFlexPositive msFlexShrink msGridColumn msGridColumnSpan msGridRow msGridRowSpan WebkitAnimationIterationCount WebkitBoxFlex WebKitBoxFlexGroup WebkitBoxOrdinalGroup WebkitColumnCount WebkitColumns WebkitFlex WebkitFlexGrow WebkitFlexPositive WebkitFlexShrink WebkitLineClamp".split(" "));function us(t,l,e){var a=l.indexOf("--")===0;e==null||typeof e=="boolean"||e===""?a?t.setProperty(l,""):l==="float"?t.cssFloat="":t[l]="":a?t.setProperty(l,e):typeof e!="number"||e===0||Zm.has(l)?l==="float"?t.cssFloat=e:t[l]=(""+e).trim():t[l]=e+"px"}function ns(t,l,e){if(l!=null&&typeof l!="object")throw Error(s(62));if(t=t.style,e!=null){for(var a in e)!e.hasOwnProperty(a)||l!=null&&l.hasOwnProperty(a)||(a.indexOf("--")===0?t.setProperty(a,""):a==="float"?t.cssFloat="":t[a]="");for(var u in l)a=l[u],l.hasOwnProperty(u)&&e[u]!==a&&us(t,u,a)}else for(var n in l)l.hasOwnProperty(n)&&us(t,n,l[n])}function mi(t){if(t.indexOf("-")===-1)return!1;switch(t){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}var Vm=new Map([["acceptCharset","accept-charset"],["htmlFor","for"],["httpEquiv","http-equiv"],["crossOrigin","crossorigin"],["accentHeight","accent-height"],["alignmentBaseline","alignment-baseline"],["arabicForm","arabic-form"],["baselineShift","baseline-shift"],["capHeight","cap-height"],["clipPath","clip-path"],["clipRule","clip-rule"],["colorInterpolation","color-interpolation"],["colorInterpolationFilters","color-interpolation-filters"],["colorProfile","color-profile"],["colorRendering","color-rendering"],["dominantBaseline","dominant-baseline"],["enableBackground","enable-background"],["fillOpacity","fill-opacity"],["fillRule","fill-rule"],["floodColor","flood-color"],["floodOpacity","flood-opacity"],["fontFamily","font-family"],["fontSize","font-size"],["fontSizeAdjust","font-size-adjust"],["fontStretch","font-stretch"],["fontStyle","font-style"],["fontVariant","font-variant"],["fontWeight","font-weight"],["glyphName","glyph-name"],["glyphOrientationHorizontal","glyph-orientation-horizontal"],["glyphOrientationVertical","glyph-orientation-vertical"],["horizAdvX","horiz-adv-x"],["horizOriginX","horiz-origin-x"],["imageRendering","image-rendering"],["letterSpacing","letter-spacing"],["lightingColor","lighting-color"],["markerEnd","marker-end"],["markerMid","marker-mid"],["markerStart","marker-start"],["overlinePosition","overline-position"],["overlineThickness","overline-thickness"],["paintOrder","paint-order"],["panose-1","panose-1"],["pointerEvents","pointer-events"],["renderingIntent","rendering-intent"],["shapeRendering","shape-rendering"],["stopColor","stop-color"],["stopOpacity","stop-opacity"],["strikethroughPosition","strikethrough-position"],["strikethroughThickness","strikethrough-thickness"],["strokeDasharray","stroke-dasharray"],["strokeDashoffset","stroke-dashoffset"],["strokeLinecap","stroke-linecap"],["strokeLinejoin","stroke-linejoin"],["strokeMiterlimit","stroke-miterlimit"],["strokeOpacity","stroke-opacity"],["strokeWidth","stroke-width"],["textAnchor","text-anchor"],["textDecoration","text-decoration"],["textRendering","text-rendering"],["transformOrigin","transform-origin"],["underlinePosition","underline-position"],["underlineThickness","underline-thickness"],["unicodeBidi","unicode-bidi"],["unicodeRange","unicode-range"],["unitsPerEm","units-per-em"],["vAlphabetic","v-alphabetic"],["vHanging","v-hanging"],["vIdeographic","v-ideographic"],["vMathematical","v-mathematical"],["vectorEffect","vector-effect"],["vertAdvY","vert-adv-y"],["vertOriginX","vert-origin-x"],["vertOriginY","vert-origin-y"],["wordSpacing","word-spacing"],["writingMode","writing-mode"],["xmlnsXlink","xmlns:xlink"],["xHeight","x-height"]]),Km=/^[\u0000-\u001F ]*j[\r\n\t]*a[\r\n\t]*v[\r\n\t]*a[\r\n\t]*s[\r\n\t]*c[\r\n\t]*r[\r\n\t]*i[\r\n\t]*p[\r\n\t]*t[\r\n\t]*:/i;function xu(t){return Km.test(""+t)?"javascript:throw new Error('React has blocked a javascript: URL as a security precaution.')":t}function Gl(){}var yi=null;function vi(t){return t=t.target||t.srcElement||window,t.correspondingUseElement&&(t=t.correspondingUseElement),t.nodeType===3?t.parentNode:t}var Pe=null,ta=null;function is(t){var l=$e(t);if(l&&(t=l.stateNode)){var e=t[kt]||null;t:switch(t=l.stateNode,l.type){case"input":if(ri(t,e.value,e.defaultValue,e.defaultValue,e.checked,e.defaultChecked,e.type,e.name),l=e.name,e.type==="radio"&&l!=null){for(e=t;e.parentNode;)e=e.parentNode;for(e=e.querySelectorAll('input[name="'+gl(""+l)+'"][type="radio"]'),l=0;l<e.length;l++){var a=e[l];if(a!==t&&a.form===t.form){var u=a[kt]||null;if(!u)throw Error(s(90));ri(a,u.value,u.defaultValue,u.defaultValue,u.checked,u.defaultChecked,u.type,u.name)}}for(l=0;l<e.length;l++)a=e[l],a.form===t.form&&ts(a)}break t;case"textarea":es(t,e.value,e.defaultValue);break t;case"select":l=e.value,l!=null&&Fe(t,!!e.multiple,l,!1)}}}var hi=!1;function cs(t,l,e){if(hi)return t(l,e);hi=!0;try{var a=t(l);return a}finally{if(hi=!1,(Pe!==null||ta!==null)&&(Dn(),Pe&&(l=Pe,t=ta,ta=Pe=null,is(l),t)))for(l=0;l<t.length;l++)is(t[l])}}function xa(t,l){var e=t.stateNode;if(e===null)return null;var a=e[kt]||null;if(a===null)return null;e=a[l];t:switch(l){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":case"onMouseEnter":(a=!a.disabled)||(t=t.type,a=!(t==="button"||t==="input"||t==="select"||t==="textarea")),t=!a;break t;default:t=!1}if(t)return null;if(e&&typeof e!="function")throw Error(s(231,l,typeof e));return e}var Xl=!(typeof window>"u"||typeof window.document>"u"||typeof window.document.createElement>"u"),gi=!1;if(Xl)try{var Qa={};Object.defineProperty(Qa,"passive",{get:function(){gi=!0}}),window.addEventListener("test",Qa,Qa),window.removeEventListener("test",Qa,Qa)}catch{gi=!1}var ae=null,Si=null,Qu=null;function fs(){if(Qu)return Qu;var t,l=Si,e=l.length,a,u="value"in ae?ae.value:ae.textContent,n=u.length;for(t=0;t<e&&l[t]===u[t];t++);var i=e-t;for(a=1;a<=i&&l[e-a]===u[n-a];a++);return Qu=u.slice(t,1<a?1-a:void 0)}function Lu(t){var l=t.keyCode;return"charCode"in t?(t=t.charCode,t===0&&l===13&&(t=13)):t=l,t===10&&(t=13),32<=t||t===13?t:0}function Zu(){return!0}function ss(){return!1}function Ft(t){function l(e,a,u,n,i){this._reactName=e,this._targetInst=u,this.type=a,this.nativeEvent=n,this.target=i,this.currentTarget=null;for(var c in t)t.hasOwnProperty(c)&&(e=t[c],this[c]=e?e(n):n[c]);return this.isDefaultPrevented=(n.defaultPrevented!=null?n.defaultPrevented:n.returnValue===!1)?Zu:ss,this.isPropagationStopped=ss,this}return N(l.prototype,{preventDefault:function(){this.defaultPrevented=!0;var e=this.nativeEvent;e&&(e.preventDefault?e.preventDefault():typeof e.returnValue!="unknown"&&(e.returnValue=!1),this.isDefaultPrevented=Zu)},stopPropagation:function(){var e=this.nativeEvent;e&&(e.stopPropagation?e.stopPropagation():typeof e.cancelBubble!="unknown"&&(e.cancelBubble=!0),this.isPropagationStopped=Zu)},persist:function(){},isPersistent:Zu}),l}var He={eventPhase:0,bubbles:0,cancelable:0,timeStamp:function(t){return t.timeStamp||Date.now()},defaultPrevented:0,isTrusted:0},Vu=Ft(He),La=N({},He,{view:0,detail:0}),Jm=Ft(La),bi,Ti,Za,Ku=N({},La,{screenX:0,screenY:0,clientX:0,clientY:0,pageX:0,pageY:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,getModifierState:Ai,button:0,buttons:0,relatedTarget:function(t){return t.relatedTarget===void 0?t.fromElement===t.srcElement?t.toElement:t.fromElement:t.relatedTarget},movementX:function(t){return"movementX"in t?t.movementX:(t!==Za&&(Za&&t.type==="mousemove"?(bi=t.screenX-Za.screenX,Ti=t.screenY-Za.screenY):Ti=bi=0,Za=t),bi)},movementY:function(t){return"movementY"in t?t.movementY:Ti}}),os=Ft(Ku),wm=N({},Ku,{dataTransfer:0}),$m=Ft(wm),Wm=N({},La,{relatedTarget:0}),Ei=Ft(Wm),km=N({},He,{animationName:0,elapsedTime:0,pseudoElement:0}),Fm=Ft(km),Im=N({},He,{clipboardData:function(t){return"clipboardData"in t?t.clipboardData:window.clipboardData}}),Pm=Ft(Im),ty=N({},He,{data:0}),rs=Ft(ty),ly={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},ey={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},ay={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function uy(t){var l=this.nativeEvent;return l.getModifierState?l.getModifierState(t):(t=ay[t])?!!l[t]:!1}function Ai(){return uy}var ny=N({},La,{key:function(t){if(t.key){var l=ly[t.key]||t.key;if(l!=="Unidentified")return l}return t.type==="keypress"?(t=Lu(t),t===13?"Enter":String.fromCharCode(t)):t.type==="keydown"||t.type==="keyup"?ey[t.keyCode]||"Unidentified":""},code:0,location:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,repeat:0,locale:0,getModifierState:Ai,charCode:function(t){return t.type==="keypress"?Lu(t):0},keyCode:function(t){return t.type==="keydown"||t.type==="keyup"?t.keyCode:0},which:function(t){return t.type==="keypress"?Lu(t):t.type==="keydown"||t.type==="keyup"?t.keyCode:0}}),iy=Ft(ny),cy=N({},Ku,{pointerId:0,width:0,height:0,pressure:0,tangentialPressure:0,tiltX:0,tiltY:0,twist:0,pointerType:0,isPrimary:0}),ds=Ft(cy),fy=N({},La,{touches:0,targetTouches:0,changedTouches:0,altKey:0,metaKey:0,ctrlKey:0,shiftKey:0,getModifierState:Ai}),sy=Ft(fy),oy=N({},He,{propertyName:0,elapsedTime:0,pseudoElement:0}),ry=Ft(oy),dy=N({},Ku,{deltaX:function(t){return"deltaX"in t?t.deltaX:"wheelDeltaX"in t?-t.wheelDeltaX:0},deltaY:function(t){return"deltaY"in t?t.deltaY:"wheelDeltaY"in t?-t.wheelDeltaY:"wheelDelta"in t?-t.wheelDelta:0},deltaZ:0,deltaMode:0}),my=Ft(dy),yy=N({},He,{newState:0,oldState:0}),vy=Ft(yy),hy=[9,13,27,32],pi=Xl&&"CompositionEvent"in window,Va=null;Xl&&"documentMode"in document&&(Va=document.documentMode);var gy=Xl&&"TextEvent"in window&&!Va,ms=Xl&&(!pi||Va&&8<Va&&11>=Va),ys=" ",vs=!1;function hs(t,l){switch(t){case"keyup":return hy.indexOf(l.keyCode)!==-1;case"keydown":return l.keyCode!==229;case"keypress":case"mousedown":case"focusout":return!0;default:return!1}}function gs(t){return t=t.detail,typeof t=="object"&&"data"in t?t.data:null}var la=!1;function Sy(t,l){switch(t){case"compositionend":return gs(l);case"keypress":return l.which!==32?null:(vs=!0,ys);case"textInput":return t=l.data,t===ys&&vs?null:t;default:return null}}function by(t,l){if(la)return t==="compositionend"||!pi&&hs(t,l)?(t=fs(),Qu=Si=ae=null,la=!1,t):null;switch(t){case"paste":return null;case"keypress":if(!(l.ctrlKey||l.altKey||l.metaKey)||l.ctrlKey&&l.altKey){if(l.char&&1<l.char.length)return l.char;if(l.which)return String.fromCharCode(l.which)}return null;case"compositionend":return ms&&l.locale!=="ko"?null:l.data;default:return null}}var Ty={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function Ss(t){var l=t&&t.nodeName&&t.nodeName.toLowerCase();return l==="input"?!!Ty[t.type]:l==="textarea"}function bs(t,l,e,a){Pe?ta?ta.push(a):ta=[a]:Pe=a,l=Bn(l,"onChange"),0<l.length&&(e=new Vu("onChange","change",null,e,a),t.push({event:e,listeners:l}))}var Ka=null,Ja=null;function Ey(t){ed(t,0)}function Ju(t){var l=Xa(t);if(ts(l))return t}function Ts(t,l){if(t==="change")return l}var Es=!1;if(Xl){var zi;if(Xl){var Oi="oninput"in document;if(!Oi){var As=document.createElement("div");As.setAttribute("oninput","return;"),Oi=typeof As.oninput=="function"}zi=Oi}else zi=!1;Es=zi&&(!document.documentMode||9<document.documentMode)}function ps(){Ka&&(Ka.detachEvent("onpropertychange",zs),Ja=Ka=null)}function zs(t){if(t.propertyName==="value"&&Ju(Ja)){var l=[];bs(l,Ja,t,vi(t)),cs(Ey,l)}}function Ay(t,l,e){t==="focusin"?(ps(),Ka=l,Ja=e,Ka.attachEvent("onpropertychange",zs)):t==="focusout"&&ps()}function py(t){if(t==="selectionchange"||t==="keyup"||t==="keydown")return Ju(Ja)}function zy(t,l){if(t==="click")return Ju(l)}function Oy(t,l){if(t==="input"||t==="change")return Ju(l)}function _y(t,l){return t===l&&(t!==0||1/t===1/l)||t!==t&&l!==l}var fl=typeof Object.is=="function"?Object.is:_y;function wa(t,l){if(fl(t,l))return!0;if(typeof t!="object"||t===null||typeof l!="object"||l===null)return!1;var e=Object.keys(t),a=Object.keys(l);if(e.length!==a.length)return!1;for(a=0;a<e.length;a++){var u=e[a];if(!ei.call(l,u)||!fl(t[u],l[u]))return!1}return!0}function Os(t){for(;t&&t.firstChild;)t=t.firstChild;return t}function _s(t,l){var e=Os(t);t=0;for(var a;e;){if(e.nodeType===3){if(a=t+e.textContent.length,t<=l&&a>=l)return{node:e,offset:l-t};t=a}t:{for(;e;){if(e.nextSibling){e=e.nextSibling;break t}e=e.parentNode}e=void 0}e=Os(e)}}function Ms(t,l){return t&&l?t===l?!0:t&&t.nodeType===3?!1:l&&l.nodeType===3?Ms(t,l.parentNode):"contains"in t?t.contains(l):t.compareDocumentPosition?!!(t.compareDocumentPosition(l)&16):!1:!1}function Ds(t){t=t!=null&&t.ownerDocument!=null&&t.ownerDocument.defaultView!=null?t.ownerDocument.defaultView:window;for(var l=Xu(t.document);l instanceof t.HTMLIFrameElement;){try{var e=typeof l.contentWindow.location.href=="string"}catch{e=!1}if(e)t=l.contentWindow;else break;l=Xu(t.document)}return l}function _i(t){var l=t&&t.nodeName&&t.nodeName.toLowerCase();return l&&(l==="input"&&(t.type==="text"||t.type==="search"||t.type==="tel"||t.type==="url"||t.type==="password")||l==="textarea"||t.contentEditable==="true")}var My=Xl&&"documentMode"in document&&11>=document.documentMode,ea=null,Mi=null,$a=null,Di=!1;function Us(t,l,e){var a=e.window===e?e.document:e.nodeType===9?e:e.ownerDocument;Di||ea==null||ea!==Xu(a)||(a=ea,"selectionStart"in a&&_i(a)?a={start:a.selectionStart,end:a.selectionEnd}:(a=(a.ownerDocument&&a.ownerDocument.defaultView||window).getSelection(),a={anchorNode:a.anchorNode,anchorOffset:a.anchorOffset,focusNode:a.focusNode,focusOffset:a.focusOffset}),$a&&wa($a,a)||($a=a,a=Bn(Mi,"onSelect"),0<a.length&&(l=new Vu("onSelect","select",null,l,e),t.push({event:l,listeners:a}),l.target=ea)))}function Ce(t,l){var e={};return e[t.toLowerCase()]=l.toLowerCase(),e["Webkit"+t]="webkit"+l,e["Moz"+t]="moz"+l,e}var aa={animationend:Ce("Animation","AnimationEnd"),animationiteration:Ce("Animation","AnimationIteration"),animationstart:Ce("Animation","AnimationStart"),transitionrun:Ce("Transition","TransitionRun"),transitionstart:Ce("Transition","TransitionStart"),transitioncancel:Ce("Transition","TransitionCancel"),transitionend:Ce("Transition","TransitionEnd")},Ui={},Hs={};Xl&&(Hs=document.createElement("div").style,"AnimationEvent"in window||(delete aa.animationend.animation,delete aa.animationiteration.animation,delete aa.animationstart.animation),"TransitionEvent"in window||delete aa.transitionend.transition);function Re(t){if(Ui[t])return Ui[t];if(!aa[t])return t;var l=aa[t],e;for(e in l)if(l.hasOwnProperty(e)&&e in Hs)return Ui[t]=l[e];return t}var Cs=Re("animationend"),Rs=Re("animationiteration"),qs=Re("animationstart"),Dy=Re("transitionrun"),Uy=Re("transitionstart"),Hy=Re("transitioncancel"),Ns=Re("transitionend"),Bs=new Map,Hi="abort auxClick beforeToggle cancel canPlay canPlayThrough click close contextMenu copy cut drag dragEnd dragEnter dragExit dragLeave dragOver dragStart drop durationChange emptied encrypted ended error gotPointerCapture input invalid keyDown keyPress keyUp load loadedData loadedMetadata loadStart lostPointerCapture mouseDown mouseMove mouseOut mouseOver mouseUp paste pause play playing pointerCancel pointerDown pointerMove pointerOut pointerOver pointerUp progress rateChange reset resize seeked seeking stalled submit suspend timeUpdate touchCancel touchEnd touchStart volumeChange scroll toggle touchMove waiting wheel".split(" ");Hi.push("scrollEnd");function Ul(t,l){Bs.set(t,l),Ue(l,[t])}var wu=typeof reportError=="function"?reportError:function(t){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var l=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof t=="object"&&t!==null&&typeof t.message=="string"?String(t.message):String(t),error:t});if(!window.dispatchEvent(l))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",t);return}console.error(t)},Sl=[],ua=0,Ci=0;function $u(){for(var t=ua,l=Ci=ua=0;l<t;){var e=Sl[l];Sl[l++]=null;var a=Sl[l];Sl[l++]=null;var u=Sl[l];Sl[l++]=null;var n=Sl[l];if(Sl[l++]=null,a!==null&&u!==null){var i=a.pending;i===null?u.next=u:(u.next=i.next,i.next=u),a.pending=u}n!==0&&js(e,u,n)}}function Wu(t,l,e,a){Sl[ua++]=t,Sl[ua++]=l,Sl[ua++]=e,Sl[ua++]=a,Ci|=a,t.lanes|=a,t=t.alternate,t!==null&&(t.lanes|=a)}function Ri(t,l,e,a){return Wu(t,l,e,a),ku(t)}function qe(t,l){return Wu(t,null,null,l),ku(t)}function js(t,l,e){t.lanes|=e;var a=t.alternate;a!==null&&(a.lanes|=e);for(var u=!1,n=t.return;n!==null;)n.childLanes|=e,a=n.alternate,a!==null&&(a.childLanes|=e),n.tag===22&&(t=n.stateNode,t===null||t._visibility&1||(u=!0)),t=n,n=n.return;return t.tag===3?(n=t.stateNode,u&&l!==null&&(u=31-cl(e),t=n.hiddenUpdates,a=t[u],a===null?t[u]=[l]:a.push(l),l.lane=e|536870912),n):null}function ku(t){if(50<hu)throw hu=0,Qc=null,Error(s(185));for(var l=t.return;l!==null;)t=l,l=t.return;return t.tag===3?t.stateNode:null}var na={};function Cy(t,l,e,a){this.tag=t,this.key=e,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.refCleanup=this.ref=null,this.pendingProps=l,this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=a,this.subtreeFlags=this.flags=0,this.deletions=null,this.childLanes=this.lanes=0,this.alternate=null}function sl(t,l,e,a){return new Cy(t,l,e,a)}function qi(t){return t=t.prototype,!(!t||!t.isReactComponent)}function xl(t,l){var e=t.alternate;return e===null?(e=sl(t.tag,l,t.key,t.mode),e.elementType=t.elementType,e.type=t.type,e.stateNode=t.stateNode,e.alternate=t,t.alternate=e):(e.pendingProps=l,e.type=t.type,e.flags=0,e.subtreeFlags=0,e.deletions=null),e.flags=t.flags&65011712,e.childLanes=t.childLanes,e.lanes=t.lanes,e.child=t.child,e.memoizedProps=t.memoizedProps,e.memoizedState=t.memoizedState,e.updateQueue=t.updateQueue,l=t.dependencies,e.dependencies=l===null?null:{lanes:l.lanes,firstContext:l.firstContext},e.sibling=t.sibling,e.index=t.index,e.ref=t.ref,e.refCleanup=t.refCleanup,e}function Ys(t,l){t.flags&=65011714;var e=t.alternate;return e===null?(t.childLanes=0,t.lanes=l,t.child=null,t.subtreeFlags=0,t.memoizedProps=null,t.memoizedState=null,t.updateQueue=null,t.dependencies=null,t.stateNode=null):(t.childLanes=e.childLanes,t.lanes=e.lanes,t.child=e.child,t.subtreeFlags=0,t.deletions=null,t.memoizedProps=e.memoizedProps,t.memoizedState=e.memoizedState,t.updateQueue=e.updateQueue,t.type=e.type,l=e.dependencies,t.dependencies=l===null?null:{lanes:l.lanes,firstContext:l.firstContext}),t}function Fu(t,l,e,a,u,n){var i=0;if(a=t,typeof t=="function")qi(t)&&(i=1);else if(typeof t=="string")i=jv(t,e,j.current)?26:t==="html"||t==="head"||t==="body"?27:5;else t:switch(t){case ul:return t=sl(31,e,l,u),t.elementType=ul,t.lanes=n,t;case et:return Ne(e.children,u,n,l);case vt:i=8,u|=24;break;case it:return t=sl(12,e,l,u|2),t.elementType=it,t.lanes=n,t;case wt:return t=sl(13,e,l,u),t.elementType=wt,t.lanes=n,t;case Nt:return t=sl(19,e,l,u),t.elementType=Nt,t.lanes=n,t;default:if(typeof t=="object"&&t!==null)switch(t.$$typeof){case At:i=10;break t;case qt:i=9;break t;case Jt:i=11;break t;case $:i=14;break t;case Gt:i=16,a=null;break t}i=29,e=Error(s(130,t===null?"null":typeof t,"")),a=null}return l=sl(i,e,l,u),l.elementType=t,l.type=a,l.lanes=n,l}function Ne(t,l,e,a){return t=sl(7,t,a,l),t.lanes=e,t}function Ni(t,l,e){return t=sl(6,t,null,l),t.lanes=e,t}function Gs(t){var l=sl(18,null,null,0);return l.stateNode=t,l}function Bi(t,l,e){return l=sl(4,t.children!==null?t.children:[],t.key,l),l.lanes=e,l.stateNode={containerInfo:t.containerInfo,pendingChildren:null,implementation:t.implementation},l}var Xs=new WeakMap;function bl(t,l){if(typeof t=="object"&&t!==null){var e=Xs.get(t);return e!==void 0?e:(l={value:t,source:l,stack:Gf(l)},Xs.set(t,l),l)}return{value:t,source:l,stack:Gf(l)}}var ia=[],ca=0,Iu=null,Wa=0,Tl=[],El=0,ue=null,ql=1,Nl="";function Ql(t,l){ia[ca++]=Wa,ia[ca++]=Iu,Iu=t,Wa=l}function xs(t,l,e){Tl[El++]=ql,Tl[El++]=Nl,Tl[El++]=ue,ue=t;var a=ql;t=Nl;var u=32-cl(a)-1;a&=~(1<<u),e+=1;var n=32-cl(l)+u;if(30<n){var i=u-u%5;n=(a&(1<<i)-1).toString(32),a>>=i,u-=i,ql=1<<32-cl(l)+u|e<<u|a,Nl=n+t}else ql=1<<n|e<<u|a,Nl=t}function ji(t){t.return!==null&&(Ql(t,1),xs(t,1,0))}function Yi(t){for(;t===Iu;)Iu=ia[--ca],ia[ca]=null,Wa=ia[--ca],ia[ca]=null;for(;t===ue;)ue=Tl[--El],Tl[El]=null,Nl=Tl[--El],Tl[El]=null,ql=Tl[--El],Tl[El]=null}function Qs(t,l){Tl[El++]=ql,Tl[El++]=Nl,Tl[El++]=ue,ql=l.id,Nl=l.overflow,ue=t}var Qt=null,St=null,lt=!1,ne=null,Al=!1,Gi=Error(s(519));function ie(t){var l=Error(s(418,1<arguments.length&&arguments[1]!==void 0&&arguments[1]?"text":"HTML",""));throw ka(bl(l,t)),Gi}function Ls(t){var l=t.stateNode,e=t.type,a=t.memoizedProps;switch(l[xt]=t,l[kt]=a,e){case"dialog":F("cancel",l),F("close",l);break;case"iframe":case"object":case"embed":F("load",l);break;case"video":case"audio":for(e=0;e<Su.length;e++)F(Su[e],l);break;case"source":F("error",l);break;case"img":case"image":case"link":F("error",l),F("load",l);break;case"details":F("toggle",l);break;case"input":F("invalid",l),ls(l,a.value,a.defaultValue,a.checked,a.defaultChecked,a.type,a.name,!0);break;case"select":F("invalid",l);break;case"textarea":F("invalid",l),as(l,a.value,a.defaultValue,a.children)}e=a.children,typeof e!="string"&&typeof e!="number"&&typeof e!="bigint"||l.textContent===""+e||a.suppressHydrationWarning===!0||id(l.textContent,e)?(a.popover!=null&&(F("beforetoggle",l),F("toggle",l)),a.onScroll!=null&&F("scroll",l),a.onScrollEnd!=null&&F("scrollend",l),a.onClick!=null&&(l.onclick=Gl),l=!0):l=!1,l||ie(t,!0)}function Zs(t){for(Qt=t.return;Qt;)switch(Qt.tag){case 5:case 31:case 13:Al=!1;return;case 27:case 3:Al=!0;return;default:Qt=Qt.return}}function fa(t){if(t!==Qt)return!1;if(!lt)return Zs(t),lt=!0,!1;var l=t.tag,e;if((e=l!==3&&l!==27)&&((e=l===5)&&(e=t.type,e=!(e!=="form"&&e!=="button")||ef(t.type,t.memoizedProps)),e=!e),e&&St&&ie(t),Zs(t),l===13){if(t=t.memoizedState,t=t!==null?t.dehydrated:null,!t)throw Error(s(317));St=vd(t)}else if(l===31){if(t=t.memoizedState,t=t!==null?t.dehydrated:null,!t)throw Error(s(317));St=vd(t)}else l===27?(l=St,Te(t.type)?(t=ff,ff=null,St=t):St=l):St=Qt?zl(t.stateNode.nextSibling):null;return!0}function Be(){St=Qt=null,lt=!1}function Xi(){var t=ne;return t!==null&&(ll===null?ll=t:ll.push.apply(ll,t),ne=null),t}function ka(t){ne===null?ne=[t]:ne.push(t)}var xi=y(null),je=null,Ll=null;function ce(t,l,e){q(xi,l._currentValue),l._currentValue=e}function Zl(t){t._currentValue=xi.current,D(xi)}function Qi(t,l,e){for(;t!==null;){var a=t.alternate;if((t.childLanes&l)!==l?(t.childLanes|=l,a!==null&&(a.childLanes|=l)):a!==null&&(a.childLanes&l)!==l&&(a.childLanes|=l),t===e)break;t=t.return}}function Li(t,l,e,a){var u=t.child;for(u!==null&&(u.return=t);u!==null;){var n=u.dependencies;if(n!==null){var i=u.child;n=n.firstContext;t:for(;n!==null;){var c=n;n=u;for(var o=0;o<l.length;o++)if(c.context===l[o]){n.lanes|=e,c=n.alternate,c!==null&&(c.lanes|=e),Qi(n.return,e,t),a||(i=null);break t}n=c.next}}else if(u.tag===18){if(i=u.return,i===null)throw Error(s(341));i.lanes|=e,n=i.alternate,n!==null&&(n.lanes|=e),Qi(i,e,t),i=null}else i=u.child;if(i!==null)i.return=u;else for(i=u;i!==null;){if(i===t){i=null;break}if(u=i.sibling,u!==null){u.return=i.return,i=u;break}i=i.return}u=i}}function sa(t,l,e,a){t=null;for(var u=l,n=!1;u!==null;){if(!n){if((u.flags&524288)!==0)n=!0;else if((u.flags&262144)!==0)break}if(u.tag===10){var i=u.alternate;if(i===null)throw Error(s(387));if(i=i.memoizedProps,i!==null){var c=u.type;fl(u.pendingProps.value,i.value)||(t!==null?t.push(c):t=[c])}}else if(u===ct.current){if(i=u.alternate,i===null)throw Error(s(387));i.memoizedState.memoizedState!==u.memoizedState.memoizedState&&(t!==null?t.push(pu):t=[pu])}u=u.return}t!==null&&Li(l,t,e,a),l.flags|=262144}function Pu(t){for(t=t.firstContext;t!==null;){if(!fl(t.context._currentValue,t.memoizedValue))return!0;t=t.next}return!1}function Ye(t){je=t,Ll=null,t=t.dependencies,t!==null&&(t.firstContext=null)}function Lt(t){return Vs(je,t)}function tn(t,l){return je===null&&Ye(t),Vs(t,l)}function Vs(t,l){var e=l._currentValue;if(l={context:l,memoizedValue:e,next:null},Ll===null){if(t===null)throw Error(s(308));Ll=l,t.dependencies={lanes:0,firstContext:l},t.flags|=524288}else Ll=Ll.next=l;return e}var Ry=typeof AbortController<"u"?AbortController:function(){var t=[],l=this.signal={aborted:!1,addEventListener:function(e,a){t.push(a)}};this.abort=function(){l.aborted=!0,t.forEach(function(e){return e()})}},qy=f.unstable_scheduleCallback,Ny=f.unstable_NormalPriority,Ut={$$typeof:At,Consumer:null,Provider:null,_currentValue:null,_currentValue2:null,_threadCount:0};function Zi(){return{controller:new Ry,data:new Map,refCount:0}}function Fa(t){t.refCount--,t.refCount===0&&qy(Ny,function(){t.controller.abort()})}var Ia=null,Vi=0,oa=0,ra=null;function By(t,l){if(Ia===null){var e=Ia=[];Vi=0,oa=wc(),ra={status:"pending",value:void 0,then:function(a){e.push(a)}}}return Vi++,l.then(Ks,Ks),l}function Ks(){if(--Vi===0&&Ia!==null){ra!==null&&(ra.status="fulfilled");var t=Ia;Ia=null,oa=0,ra=null;for(var l=0;l<t.length;l++)(0,t[l])()}}function jy(t,l){var e=[],a={status:"pending",value:null,reason:null,then:function(u){e.push(u)}};return t.then(function(){a.status="fulfilled",a.value=l;for(var u=0;u<e.length;u++)(0,e[u])(l)},function(u){for(a.status="rejected",a.reason=u,u=0;u<e.length;u++)(0,e[u])(void 0)}),a}var Js=O.S;O.S=function(t,l){Hr=nl(),typeof l=="object"&&l!==null&&typeof l.then=="function"&&By(t,l),Js!==null&&Js(t,l)};var Ge=y(null);function Ki(){var t=Ge.current;return t!==null?t:gt.pooledCache}function ln(t,l){l===null?q(Ge,Ge.current):q(Ge,l.pool)}function ws(){var t=Ki();return t===null?null:{parent:Ut._currentValue,pool:t}}var da=Error(s(460)),Ji=Error(s(474)),en=Error(s(542)),an={then:function(){}};function $s(t){return t=t.status,t==="fulfilled"||t==="rejected"}function Ws(t,l,e){switch(e=t[e],e===void 0?t.push(l):e!==l&&(l.then(Gl,Gl),l=e),l.status){case"fulfilled":return l.value;case"rejected":throw t=l.reason,Fs(t),t;default:if(typeof l.status=="string")l.then(Gl,Gl);else{if(t=gt,t!==null&&100<t.shellSuspendCounter)throw Error(s(482));t=l,t.status="pending",t.then(function(a){if(l.status==="pending"){var u=l;u.status="fulfilled",u.value=a}},function(a){if(l.status==="pending"){var u=l;u.status="rejected",u.reason=a}})}switch(l.status){case"fulfilled":return l.value;case"rejected":throw t=l.reason,Fs(t),t}throw xe=l,da}}function Xe(t){try{var l=t._init;return l(t._payload)}catch(e){throw e!==null&&typeof e=="object"&&typeof e.then=="function"?(xe=e,da):e}}var xe=null;function ks(){if(xe===null)throw Error(s(459));var t=xe;return xe=null,t}function Fs(t){if(t===da||t===en)throw Error(s(483))}var ma=null,Pa=0;function un(t){var l=Pa;return Pa+=1,ma===null&&(ma=[]),Ws(ma,t,l)}function tu(t,l){l=l.props.ref,t.ref=l!==void 0?l:null}function nn(t,l){throw l.$$typeof===K?Error(s(525)):(t=Object.prototype.toString.call(l),Error(s(31,t==="[object Object]"?"object with keys {"+Object.keys(l).join(", ")+"}":t)))}function Is(t){function l(h,d){if(t){var g=h.deletions;g===null?(h.deletions=[d],h.flags|=16):g.push(d)}}function e(h,d){if(!t)return null;for(;d!==null;)l(h,d),d=d.sibling;return null}function a(h){for(var d=new Map;h!==null;)h.key!==null?d.set(h.key,h):d.set(h.index,h),h=h.sibling;return d}function u(h,d){return h=xl(h,d),h.index=0,h.sibling=null,h}function n(h,d,g){return h.index=g,t?(g=h.alternate,g!==null?(g=g.index,g<d?(h.flags|=67108866,d):g):(h.flags|=67108866,d)):(h.flags|=1048576,d)}function i(h){return t&&h.alternate===null&&(h.flags|=67108866),h}function c(h,d,g,_){return d===null||d.tag!==6?(d=Ni(g,h.mode,_),d.return=h,d):(d=u(d,g),d.return=h,d)}function o(h,d,g,_){var G=g.type;return G===et?z(h,d,g.props.children,_,g.key):d!==null&&(d.elementType===G||typeof G=="object"&&G!==null&&G.$$typeof===Gt&&Xe(G)===d.type)?(d=u(d,g.props),tu(d,g),d.return=h,d):(d=Fu(g.type,g.key,g.props,null,h.mode,_),tu(d,g),d.return=h,d)}function S(h,d,g,_){return d===null||d.tag!==4||d.stateNode.containerInfo!==g.containerInfo||d.stateNode.implementation!==g.implementation?(d=Bi(g,h.mode,_),d.return=h,d):(d=u(d,g.children||[]),d.return=h,d)}function z(h,d,g,_,G){return d===null||d.tag!==7?(d=Ne(g,h.mode,_,G),d.return=h,d):(d=u(d,g),d.return=h,d)}function M(h,d,g){if(typeof d=="string"&&d!==""||typeof d=="number"||typeof d=="bigint")return d=Ni(""+d,h.mode,g),d.return=h,d;if(typeof d=="object"&&d!==null){switch(d.$$typeof){case tt:return g=Fu(d.type,d.key,d.props,null,h.mode,g),tu(g,d),g.return=h,g;case Q:return d=Bi(d,h.mode,g),d.return=h,d;case Gt:return d=Xe(d),M(h,d,g)}if(Xt(d)||w(d))return d=Ne(d,h.mode,g,null),d.return=h,d;if(typeof d.then=="function")return M(h,un(d),g);if(d.$$typeof===At)return M(h,tn(h,d),g);nn(h,d)}return null}function b(h,d,g,_){var G=d!==null?d.key:null;if(typeof g=="string"&&g!==""||typeof g=="number"||typeof g=="bigint")return G!==null?null:c(h,d,""+g,_);if(typeof g=="object"&&g!==null){switch(g.$$typeof){case tt:return g.key===G?o(h,d,g,_):null;case Q:return g.key===G?S(h,d,g,_):null;case Gt:return g=Xe(g),b(h,d,g,_)}if(Xt(g)||w(g))return G!==null?null:z(h,d,g,_,null);if(typeof g.then=="function")return b(h,d,un(g),_);if(g.$$typeof===At)return b(h,d,tn(h,g),_);nn(h,g)}return null}function E(h,d,g,_,G){if(typeof _=="string"&&_!==""||typeof _=="number"||typeof _=="bigint")return h=h.get(g)||null,c(d,h,""+_,G);if(typeof _=="object"&&_!==null){switch(_.$$typeof){case tt:return h=h.get(_.key===null?g:_.key)||null,o(d,h,_,G);case Q:return h=h.get(_.key===null?g:_.key)||null,S(d,h,_,G);case Gt:return _=Xe(_),E(h,d,g,_,G)}if(Xt(_)||w(_))return h=h.get(g)||null,z(d,h,_,G,null);if(typeof _.then=="function")return E(h,d,g,un(_),G);if(_.$$typeof===At)return E(h,d,g,tn(d,_),G);nn(d,_)}return null}function B(h,d,g,_){for(var G=null,at=null,Y=d,J=d=0,P=null;Y!==null&&J<g.length;J++){Y.index>J?(P=Y,Y=null):P=Y.sibling;var ut=b(h,Y,g[J],_);if(ut===null){Y===null&&(Y=P);break}t&&Y&&ut.alternate===null&&l(h,Y),d=n(ut,d,J),at===null?G=ut:at.sibling=ut,at=ut,Y=P}if(J===g.length)return e(h,Y),lt&&Ql(h,J),G;if(Y===null){for(;J<g.length;J++)Y=M(h,g[J],_),Y!==null&&(d=n(Y,d,J),at===null?G=Y:at.sibling=Y,at=Y);return lt&&Ql(h,J),G}for(Y=a(Y);J<g.length;J++)P=E(Y,h,J,g[J],_),P!==null&&(t&&P.alternate!==null&&Y.delete(P.key===null?J:P.key),d=n(P,d,J),at===null?G=P:at.sibling=P,at=P);return t&&Y.forEach(function(Oe){return l(h,Oe)}),lt&&Ql(h,J),G}function X(h,d,g,_){if(g==null)throw Error(s(151));for(var G=null,at=null,Y=d,J=d=0,P=null,ut=g.next();Y!==null&&!ut.done;J++,ut=g.next()){Y.index>J?(P=Y,Y=null):P=Y.sibling;var Oe=b(h,Y,ut.value,_);if(Oe===null){Y===null&&(Y=P);break}t&&Y&&Oe.alternate===null&&l(h,Y),d=n(Oe,d,J),at===null?G=Oe:at.sibling=Oe,at=Oe,Y=P}if(ut.done)return e(h,Y),lt&&Ql(h,J),G;if(Y===null){for(;!ut.done;J++,ut=g.next())ut=M(h,ut.value,_),ut!==null&&(d=n(ut,d,J),at===null?G=ut:at.sibling=ut,at=ut);return lt&&Ql(h,J),G}for(Y=a(Y);!ut.done;J++,ut=g.next())ut=E(Y,h,J,ut.value,_),ut!==null&&(t&&ut.alternate!==null&&Y.delete(ut.key===null?J:ut.key),d=n(ut,d,J),at===null?G=ut:at.sibling=ut,at=ut);return t&&Y.forEach(function(wv){return l(h,wv)}),lt&&Ql(h,J),G}function mt(h,d,g,_){if(typeof g=="object"&&g!==null&&g.type===et&&g.key===null&&(g=g.props.children),typeof g=="object"&&g!==null){switch(g.$$typeof){case tt:t:{for(var G=g.key;d!==null;){if(d.key===G){if(G=g.type,G===et){if(d.tag===7){e(h,d.sibling),_=u(d,g.props.children),_.return=h,h=_;break t}}else if(d.elementType===G||typeof G=="object"&&G!==null&&G.$$typeof===Gt&&Xe(G)===d.type){e(h,d.sibling),_=u(d,g.props),tu(_,g),_.return=h,h=_;break t}e(h,d);break}else l(h,d);d=d.sibling}g.type===et?(_=Ne(g.props.children,h.mode,_,g.key),_.return=h,h=_):(_=Fu(g.type,g.key,g.props,null,h.mode,_),tu(_,g),_.return=h,h=_)}return i(h);case Q:t:{for(G=g.key;d!==null;){if(d.key===G)if(d.tag===4&&d.stateNode.containerInfo===g.containerInfo&&d.stateNode.implementation===g.implementation){e(h,d.sibling),_=u(d,g.children||[]),_.return=h,h=_;break t}else{e(h,d);break}else l(h,d);d=d.sibling}_=Bi(g,h.mode,_),_.return=h,h=_}return i(h);case Gt:return g=Xe(g),mt(h,d,g,_)}if(Xt(g))return B(h,d,g,_);if(w(g)){if(G=w(g),typeof G!="function")throw Error(s(150));return g=G.call(g),X(h,d,g,_)}if(typeof g.then=="function")return mt(h,d,un(g),_);if(g.$$typeof===At)return mt(h,d,tn(h,g),_);nn(h,g)}return typeof g=="string"&&g!==""||typeof g=="number"||typeof g=="bigint"?(g=""+g,d!==null&&d.tag===6?(e(h,d.sibling),_=u(d,g),_.return=h,h=_):(e(h,d),_=Ni(g,h.mode,_),_.return=h,h=_),i(h)):e(h,d)}return function(h,d,g,_){try{Pa=0;var G=mt(h,d,g,_);return ma=null,G}catch(Y){if(Y===da||Y===en)throw Y;var at=sl(29,Y,null,h.mode);return at.lanes=_,at.return=h,at}finally{}}}var Qe=Is(!0),Ps=Is(!1),fe=!1;function wi(t){t.updateQueue={baseState:t.memoizedState,firstBaseUpdate:null,lastBaseUpdate:null,shared:{pending:null,lanes:0,hiddenCallbacks:null},callbacks:null}}function $i(t,l){t=t.updateQueue,l.updateQueue===t&&(l.updateQueue={baseState:t.baseState,firstBaseUpdate:t.firstBaseUpdate,lastBaseUpdate:t.lastBaseUpdate,shared:t.shared,callbacks:null})}function se(t){return{lane:t,tag:0,payload:null,callback:null,next:null}}function oe(t,l,e){var a=t.updateQueue;if(a===null)return null;if(a=a.shared,(nt&2)!==0){var u=a.pending;return u===null?l.next=l:(l.next=u.next,u.next=l),a.pending=l,l=ku(t),js(t,null,e),l}return Wu(t,a,l,e),ku(t)}function lu(t,l,e){if(l=l.updateQueue,l!==null&&(l=l.shared,(e&4194048)!==0)){var a=l.lanes;a&=t.pendingLanes,e|=a,l.lanes=e,Vf(t,e)}}function Wi(t,l){var e=t.updateQueue,a=t.alternate;if(a!==null&&(a=a.updateQueue,e===a)){var u=null,n=null;if(e=e.firstBaseUpdate,e!==null){do{var i={lane:e.lane,tag:e.tag,payload:e.payload,callback:null,next:null};n===null?u=n=i:n=n.next=i,e=e.next}while(e!==null);n===null?u=n=l:n=n.next=l}else u=n=l;e={baseState:a.baseState,firstBaseUpdate:u,lastBaseUpdate:n,shared:a.shared,callbacks:a.callbacks},t.updateQueue=e;return}t=e.lastBaseUpdate,t===null?e.firstBaseUpdate=l:t.next=l,e.lastBaseUpdate=l}var ki=!1;function eu(){if(ki){var t=ra;if(t!==null)throw t}}function au(t,l,e,a){ki=!1;var u=t.updateQueue;fe=!1;var n=u.firstBaseUpdate,i=u.lastBaseUpdate,c=u.shared.pending;if(c!==null){u.shared.pending=null;var o=c,S=o.next;o.next=null,i===null?n=S:i.next=S,i=o;var z=t.alternate;z!==null&&(z=z.updateQueue,c=z.lastBaseUpdate,c!==i&&(c===null?z.firstBaseUpdate=S:c.next=S,z.lastBaseUpdate=o))}if(n!==null){var M=u.baseState;i=0,z=S=o=null,c=n;do{var b=c.lane&-536870913,E=b!==c.lane;if(E?(I&b)===b:(a&b)===b){b!==0&&b===oa&&(ki=!0),z!==null&&(z=z.next={lane:0,tag:c.tag,payload:c.payload,callback:null,next:null});t:{var B=t,X=c;b=l;var mt=e;switch(X.tag){case 1:if(B=X.payload,typeof B=="function"){M=B.call(mt,M,b);break t}M=B;break t;case 3:B.flags=B.flags&-65537|128;case 0:if(B=X.payload,b=typeof B=="function"?B.call(mt,M,b):B,b==null)break t;M=N({},M,b);break t;case 2:fe=!0}}b=c.callback,b!==null&&(t.flags|=64,E&&(t.flags|=8192),E=u.callbacks,E===null?u.callbacks=[b]:E.push(b))}else E={lane:b,tag:c.tag,payload:c.payload,callback:c.callback,next:null},z===null?(S=z=E,o=M):z=z.next=E,i|=b;if(c=c.next,c===null){if(c=u.shared.pending,c===null)break;E=c,c=E.next,E.next=null,u.lastBaseUpdate=E,u.shared.pending=null}}while(!0);z===null&&(o=M),u.baseState=o,u.firstBaseUpdate=S,u.lastBaseUpdate=z,n===null&&(u.shared.lanes=0),ve|=i,t.lanes=i,t.memoizedState=M}}function to(t,l){if(typeof t!="function")throw Error(s(191,t));t.call(l)}function lo(t,l){var e=t.callbacks;if(e!==null)for(t.callbacks=null,t=0;t<e.length;t++)to(e[t],l)}var ya=y(null),cn=y(0);function eo(t,l){t=Il,q(cn,t),q(ya,l),Il=t|l.baseLanes}function Fi(){q(cn,Il),q(ya,ya.current)}function Ii(){Il=cn.current,D(ya),D(cn)}var ol=y(null),pl=null;function re(t){var l=t.alternate;q(Mt,Mt.current&1),q(ol,t),pl===null&&(l===null||ya.current!==null||l.memoizedState!==null)&&(pl=t)}function Pi(t){q(Mt,Mt.current),q(ol,t),pl===null&&(pl=t)}function ao(t){t.tag===22?(q(Mt,Mt.current),q(ol,t),pl===null&&(pl=t)):de()}function de(){q(Mt,Mt.current),q(ol,ol.current)}function rl(t){D(ol),pl===t&&(pl=null),D(Mt)}var Mt=y(0);function fn(t){for(var l=t;l!==null;){if(l.tag===13){var e=l.memoizedState;if(e!==null&&(e=e.dehydrated,e===null||nf(e)||cf(e)))return l}else if(l.tag===19&&(l.memoizedProps.revealOrder==="forwards"||l.memoizedProps.revealOrder==="backwards"||l.memoizedProps.revealOrder==="unstable_legacy-backwards"||l.memoizedProps.revealOrder==="together")){if((l.flags&128)!==0)return l}else if(l.child!==null){l.child.return=l,l=l.child;continue}if(l===t)break;for(;l.sibling===null;){if(l.return===null||l.return===t)return null;l=l.return}l.sibling.return=l.return,l=l.sibling}return null}var Vl=0,V=null,rt=null,Ht=null,sn=!1,va=!1,Le=!1,on=0,uu=0,ha=null,Yy=0;function zt(){throw Error(s(321))}function tc(t,l){if(l===null)return!1;for(var e=0;e<l.length&&e<t.length;e++)if(!fl(t[e],l[e]))return!1;return!0}function lc(t,l,e,a,u,n){return Vl=n,V=l,l.memoizedState=null,l.updateQueue=null,l.lanes=0,O.H=t===null||t.memoizedState===null?Qo:hc,Le=!1,n=e(a,u),Le=!1,va&&(n=no(l,e,a,u)),uo(t),n}function uo(t){O.H=cu;var l=rt!==null&&rt.next!==null;if(Vl=0,Ht=rt=V=null,sn=!1,uu=0,ha=null,l)throw Error(s(300));t===null||Ct||(t=t.dependencies,t!==null&&Pu(t)&&(Ct=!0))}function no(t,l,e,a){V=t;var u=0;do{if(va&&(ha=null),uu=0,va=!1,25<=u)throw Error(s(301));if(u+=1,Ht=rt=null,t.updateQueue!=null){var n=t.updateQueue;n.lastEffect=null,n.events=null,n.stores=null,n.memoCache!=null&&(n.memoCache.index=0)}O.H=Lo,n=l(e,a)}while(va);return n}function Gy(){var t=O.H,l=t.useState()[0];return l=typeof l.then=="function"?nu(l):l,t=t.useState()[0],(rt!==null?rt.memoizedState:null)!==t&&(V.flags|=1024),l}function ec(){var t=on!==0;return on=0,t}function ac(t,l,e){l.updateQueue=t.updateQueue,l.flags&=-2053,t.lanes&=~e}function uc(t){if(sn){for(t=t.memoizedState;t!==null;){var l=t.queue;l!==null&&(l.pending=null),t=t.next}sn=!1}Vl=0,Ht=rt=V=null,va=!1,uu=on=0,ha=null}function Wt(){var t={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};return Ht===null?V.memoizedState=Ht=t:Ht=Ht.next=t,Ht}function Dt(){if(rt===null){var t=V.alternate;t=t!==null?t.memoizedState:null}else t=rt.next;var l=Ht===null?V.memoizedState:Ht.next;if(l!==null)Ht=l,rt=t;else{if(t===null)throw V.alternate===null?Error(s(467)):Error(s(310));rt=t,t={memoizedState:rt.memoizedState,baseState:rt.baseState,baseQueue:rt.baseQueue,queue:rt.queue,next:null},Ht===null?V.memoizedState=Ht=t:Ht=Ht.next=t}return Ht}function rn(){return{lastEffect:null,events:null,stores:null,memoCache:null}}function nu(t){var l=uu;return uu+=1,ha===null&&(ha=[]),t=Ws(ha,t,l),l=V,(Ht===null?l.memoizedState:Ht.next)===null&&(l=l.alternate,O.H=l===null||l.memoizedState===null?Qo:hc),t}function dn(t){if(t!==null&&typeof t=="object"){if(typeof t.then=="function")return nu(t);if(t.$$typeof===At)return Lt(t)}throw Error(s(438,String(t)))}function nc(t){var l=null,e=V.updateQueue;if(e!==null&&(l=e.memoCache),l==null){var a=V.alternate;a!==null&&(a=a.updateQueue,a!==null&&(a=a.memoCache,a!=null&&(l={data:a.data.map(function(u){return u.slice()}),index:0})))}if(l==null&&(l={data:[],index:0}),e===null&&(e=rn(),V.updateQueue=e),e.memoCache=l,e=l.data[l.index],e===void 0)for(e=l.data[l.index]=Array(t),a=0;a<t;a++)e[a]=Ml;return l.index++,e}function Kl(t,l){return typeof l=="function"?l(t):l}function mn(t){var l=Dt();return ic(l,rt,t)}function ic(t,l,e){var a=t.queue;if(a===null)throw Error(s(311));a.lastRenderedReducer=e;var u=t.baseQueue,n=a.pending;if(n!==null){if(u!==null){var i=u.next;u.next=n.next,n.next=i}l.baseQueue=u=n,a.pending=null}if(n=t.baseState,u===null)t.memoizedState=n;else{l=u.next;var c=i=null,o=null,S=l,z=!1;do{var M=S.lane&-536870913;if(M!==S.lane?(I&M)===M:(Vl&M)===M){var b=S.revertLane;if(b===0)o!==null&&(o=o.next={lane:0,revertLane:0,gesture:null,action:S.action,hasEagerState:S.hasEagerState,eagerState:S.eagerState,next:null}),M===oa&&(z=!0);else if((Vl&b)===b){S=S.next,b===oa&&(z=!0);continue}else M={lane:0,revertLane:S.revertLane,gesture:null,action:S.action,hasEagerState:S.hasEagerState,eagerState:S.eagerState,next:null},o===null?(c=o=M,i=n):o=o.next=M,V.lanes|=b,ve|=b;M=S.action,Le&&e(n,M),n=S.hasEagerState?S.eagerState:e(n,M)}else b={lane:M,revertLane:S.revertLane,gesture:S.gesture,action:S.action,hasEagerState:S.hasEagerState,eagerState:S.eagerState,next:null},o===null?(c=o=b,i=n):o=o.next=b,V.lanes|=M,ve|=M;S=S.next}while(S!==null&&S!==l);if(o===null?i=n:o.next=c,!fl(n,t.memoizedState)&&(Ct=!0,z&&(e=ra,e!==null)))throw e;t.memoizedState=n,t.baseState=i,t.baseQueue=o,a.lastRenderedState=n}return u===null&&(a.lanes=0),[t.memoizedState,a.dispatch]}function cc(t){var l=Dt(),e=l.queue;if(e===null)throw Error(s(311));e.lastRenderedReducer=t;var a=e.dispatch,u=e.pending,n=l.memoizedState;if(u!==null){e.pending=null;var i=u=u.next;do n=t(n,i.action),i=i.next;while(i!==u);fl(n,l.memoizedState)||(Ct=!0),l.memoizedState=n,l.baseQueue===null&&(l.baseState=n),e.lastRenderedState=n}return[n,a]}function io(t,l,e){var a=V,u=Dt(),n=lt;if(n){if(e===void 0)throw Error(s(407));e=e()}else e=l();var i=!fl((rt||u).memoizedState,e);if(i&&(u.memoizedState=e,Ct=!0),u=u.queue,oc(so.bind(null,a,u,t),[t]),u.getSnapshot!==l||i||Ht!==null&&Ht.memoizedState.tag&1){if(a.flags|=2048,ga(9,{destroy:void 0},fo.bind(null,a,u,e,l),null),gt===null)throw Error(s(349));n||(Vl&127)!==0||co(a,l,e)}return e}function co(t,l,e){t.flags|=16384,t={getSnapshot:l,value:e},l=V.updateQueue,l===null?(l=rn(),V.updateQueue=l,l.stores=[t]):(e=l.stores,e===null?l.stores=[t]:e.push(t))}function fo(t,l,e,a){l.value=e,l.getSnapshot=a,oo(l)&&ro(t)}function so(t,l,e){return e(function(){oo(l)&&ro(t)})}function oo(t){var l=t.getSnapshot;t=t.value;try{var e=l();return!fl(t,e)}catch{return!0}}function ro(t){var l=qe(t,2);l!==null&&el(l,t,2)}function fc(t){var l=Wt();if(typeof t=="function"){var e=t;if(t=e(),Le){le(!0);try{e()}finally{le(!1)}}}return l.memoizedState=l.baseState=t,l.queue={pending:null,lanes:0,dispatch:null,lastRenderedReducer:Kl,lastRenderedState:t},l}function mo(t,l,e,a){return t.baseState=e,ic(t,rt,typeof a=="function"?a:Kl)}function Xy(t,l,e,a,u){if(hn(t))throw Error(s(485));if(t=l.action,t!==null){var n={payload:u,action:t,next:null,isTransition:!0,status:"pending",value:null,reason:null,listeners:[],then:function(i){n.listeners.push(i)}};O.T!==null?e(!0):n.isTransition=!1,a(n),e=l.pending,e===null?(n.next=l.pending=n,yo(l,n)):(n.next=e.next,l.pending=e.next=n)}}function yo(t,l){var e=l.action,a=l.payload,u=t.state;if(l.isTransition){var n=O.T,i={};O.T=i;try{var c=e(u,a),o=O.S;o!==null&&o(i,c),vo(t,l,c)}catch(S){sc(t,l,S)}finally{n!==null&&i.types!==null&&(n.types=i.types),O.T=n}}else try{n=e(u,a),vo(t,l,n)}catch(S){sc(t,l,S)}}function vo(t,l,e){e!==null&&typeof e=="object"&&typeof e.then=="function"?e.then(function(a){ho(t,l,a)},function(a){return sc(t,l,a)}):ho(t,l,e)}function ho(t,l,e){l.status="fulfilled",l.value=e,go(l),t.state=e,l=t.pending,l!==null&&(e=l.next,e===l?t.pending=null:(e=e.next,l.next=e,yo(t,e)))}function sc(t,l,e){var a=t.pending;if(t.pending=null,a!==null){a=a.next;do l.status="rejected",l.reason=e,go(l),l=l.next;while(l!==a)}t.action=null}function go(t){t=t.listeners;for(var l=0;l<t.length;l++)(0,t[l])()}function So(t,l){return l}function bo(t,l){if(lt){var e=gt.formState;if(e!==null){t:{var a=V;if(lt){if(St){l:{for(var u=St,n=Al;u.nodeType!==8;){if(!n){u=null;break l}if(u=zl(u.nextSibling),u===null){u=null;break l}}n=u.data,u=n==="F!"||n==="F"?u:null}if(u){St=zl(u.nextSibling),a=u.data==="F!";break t}}ie(a)}a=!1}a&&(l=e[0])}}return e=Wt(),e.memoizedState=e.baseState=l,a={pending:null,lanes:0,dispatch:null,lastRenderedReducer:So,lastRenderedState:l},e.queue=a,e=Go.bind(null,V,a),a.dispatch=e,a=fc(!1),n=vc.bind(null,V,!1,a.queue),a=Wt(),u={state:l,dispatch:null,action:t,pending:null},a.queue=u,e=Xy.bind(null,V,u,n,e),u.dispatch=e,a.memoizedState=t,[l,e,!1]}function To(t){var l=Dt();return Eo(l,rt,t)}function Eo(t,l,e){if(l=ic(t,l,So)[0],t=mn(Kl)[0],typeof l=="object"&&l!==null&&typeof l.then=="function")try{var a=nu(l)}catch(i){throw i===da?en:i}else a=l;l=Dt();var u=l.queue,n=u.dispatch;return e!==l.memoizedState&&(V.flags|=2048,ga(9,{destroy:void 0},xy.bind(null,u,e),null)),[a,n,t]}function xy(t,l){t.action=l}function Ao(t){var l=Dt(),e=rt;if(e!==null)return Eo(l,e,t);Dt(),l=l.memoizedState,e=Dt();var a=e.queue.dispatch;return e.memoizedState=t,[l,a,!1]}function ga(t,l,e,a){return t={tag:t,create:e,deps:a,inst:l,next:null},l=V.updateQueue,l===null&&(l=rn(),V.updateQueue=l),e=l.lastEffect,e===null?l.lastEffect=t.next=t:(a=e.next,e.next=t,t.next=a,l.lastEffect=t),t}function po(){return Dt().memoizedState}function yn(t,l,e,a){var u=Wt();V.flags|=t,u.memoizedState=ga(1|l,{destroy:void 0},e,a===void 0?null:a)}function vn(t,l,e,a){var u=Dt();a=a===void 0?null:a;var n=u.memoizedState.inst;rt!==null&&a!==null&&tc(a,rt.memoizedState.deps)?u.memoizedState=ga(l,n,e,a):(V.flags|=t,u.memoizedState=ga(1|l,n,e,a))}function zo(t,l){yn(8390656,8,t,l)}function oc(t,l){vn(2048,8,t,l)}function Qy(t){V.flags|=4;var l=V.updateQueue;if(l===null)l=rn(),V.updateQueue=l,l.events=[t];else{var e=l.events;e===null?l.events=[t]:e.push(t)}}function Oo(t){var l=Dt().memoizedState;return Qy({ref:l,nextImpl:t}),function(){if((nt&2)!==0)throw Error(s(440));return l.impl.apply(void 0,arguments)}}function _o(t,l){return vn(4,2,t,l)}function Mo(t,l){return vn(4,4,t,l)}function Do(t,l){if(typeof l=="function"){t=t();var e=l(t);return function(){typeof e=="function"?e():l(null)}}if(l!=null)return t=t(),l.current=t,function(){l.current=null}}function Uo(t,l,e){e=e!=null?e.concat([t]):null,vn(4,4,Do.bind(null,l,t),e)}function rc(){}function Ho(t,l){var e=Dt();l=l===void 0?null:l;var a=e.memoizedState;return l!==null&&tc(l,a[1])?a[0]:(e.memoizedState=[t,l],t)}function Co(t,l){var e=Dt();l=l===void 0?null:l;var a=e.memoizedState;if(l!==null&&tc(l,a[1]))return a[0];if(a=t(),Le){le(!0);try{t()}finally{le(!1)}}return e.memoizedState=[a,l],a}function dc(t,l,e){return e===void 0||(Vl&1073741824)!==0&&(I&261930)===0?t.memoizedState=l:(t.memoizedState=e,t=Rr(),V.lanes|=t,ve|=t,e)}function Ro(t,l,e,a){return fl(e,l)?e:ya.current!==null?(t=dc(t,e,a),fl(t,l)||(Ct=!0),t):(Vl&42)===0||(Vl&1073741824)!==0&&(I&261930)===0?(Ct=!0,t.memoizedState=e):(t=Rr(),V.lanes|=t,ve|=t,l)}function qo(t,l,e,a,u){var n=C.p;C.p=n!==0&&8>n?n:8;var i=O.T,c={};O.T=c,vc(t,!1,l,e);try{var o=u(),S=O.S;if(S!==null&&S(c,o),o!==null&&typeof o=="object"&&typeof o.then=="function"){var z=jy(o,a);iu(t,l,z,yl(t))}else iu(t,l,a,yl(t))}catch(M){iu(t,l,{then:function(){},status:"rejected",reason:M},yl())}finally{C.p=n,i!==null&&c.types!==null&&(i.types=c.types),O.T=i}}function Ly(){}function mc(t,l,e,a){if(t.tag!==5)throw Error(s(476));var u=No(t).queue;qo(t,u,l,x,e===null?Ly:function(){return Bo(t),e(a)})}function No(t){var l=t.memoizedState;if(l!==null)return l;l={memoizedState:x,baseState:x,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:Kl,lastRenderedState:x},next:null};var e={};return l.next={memoizedState:e,baseState:e,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:Kl,lastRenderedState:e},next:null},t.memoizedState=l,t=t.alternate,t!==null&&(t.memoizedState=l),l}function Bo(t){var l=No(t);l.next===null&&(l=t.alternate.memoizedState),iu(t,l.next.queue,{},yl())}function yc(){return Lt(pu)}function jo(){return Dt().memoizedState}function Yo(){return Dt().memoizedState}function Zy(t){for(var l=t.return;l!==null;){switch(l.tag){case 24:case 3:var e=yl();t=se(e);var a=oe(l,t,e);a!==null&&(el(a,l,e),lu(a,l,e)),l={cache:Zi()},t.payload=l;return}l=l.return}}function Vy(t,l,e){var a=yl();e={lane:a,revertLane:0,gesture:null,action:e,hasEagerState:!1,eagerState:null,next:null},hn(t)?Xo(l,e):(e=Ri(t,l,e,a),e!==null&&(el(e,t,a),xo(e,l,a)))}function Go(t,l,e){var a=yl();iu(t,l,e,a)}function iu(t,l,e,a){var u={lane:a,revertLane:0,gesture:null,action:e,hasEagerState:!1,eagerState:null,next:null};if(hn(t))Xo(l,u);else{var n=t.alternate;if(t.lanes===0&&(n===null||n.lanes===0)&&(n=l.lastRenderedReducer,n!==null))try{var i=l.lastRenderedState,c=n(i,e);if(u.hasEagerState=!0,u.eagerState=c,fl(c,i))return Wu(t,l,u,0),gt===null&&$u(),!1}catch{}finally{}if(e=Ri(t,l,u,a),e!==null)return el(e,t,a),xo(e,l,a),!0}return!1}function vc(t,l,e,a){if(a={lane:2,revertLane:wc(),gesture:null,action:a,hasEagerState:!1,eagerState:null,next:null},hn(t)){if(l)throw Error(s(479))}else l=Ri(t,e,a,2),l!==null&&el(l,t,2)}function hn(t){var l=t.alternate;return t===V||l!==null&&l===V}function Xo(t,l){va=sn=!0;var e=t.pending;e===null?l.next=l:(l.next=e.next,e.next=l),t.pending=l}function xo(t,l,e){if((e&4194048)!==0){var a=l.lanes;a&=t.pendingLanes,e|=a,l.lanes=e,Vf(t,e)}}var cu={readContext:Lt,use:dn,useCallback:zt,useContext:zt,useEffect:zt,useImperativeHandle:zt,useLayoutEffect:zt,useInsertionEffect:zt,useMemo:zt,useReducer:zt,useRef:zt,useState:zt,useDebugValue:zt,useDeferredValue:zt,useTransition:zt,useSyncExternalStore:zt,useId:zt,useHostTransitionStatus:zt,useFormState:zt,useActionState:zt,useOptimistic:zt,useMemoCache:zt,useCacheRefresh:zt};cu.useEffectEvent=zt;var Qo={readContext:Lt,use:dn,useCallback:function(t,l){return Wt().memoizedState=[t,l===void 0?null:l],t},useContext:Lt,useEffect:zo,useImperativeHandle:function(t,l,e){e=e!=null?e.concat([t]):null,yn(4194308,4,Do.bind(null,l,t),e)},useLayoutEffect:function(t,l){return yn(4194308,4,t,l)},useInsertionEffect:function(t,l){yn(4,2,t,l)},useMemo:function(t,l){var e=Wt();l=l===void 0?null:l;var a=t();if(Le){le(!0);try{t()}finally{le(!1)}}return e.memoizedState=[a,l],a},useReducer:function(t,l,e){var a=Wt();if(e!==void 0){var u=e(l);if(Le){le(!0);try{e(l)}finally{le(!1)}}}else u=l;return a.memoizedState=a.baseState=u,t={pending:null,lanes:0,dispatch:null,lastRenderedReducer:t,lastRenderedState:u},a.queue=t,t=t.dispatch=Vy.bind(null,V,t),[a.memoizedState,t]},useRef:function(t){var l=Wt();return t={current:t},l.memoizedState=t},useState:function(t){t=fc(t);var l=t.queue,e=Go.bind(null,V,l);return l.dispatch=e,[t.memoizedState,e]},useDebugValue:rc,useDeferredValue:function(t,l){var e=Wt();return dc(e,t,l)},useTransition:function(){var t=fc(!1);return t=qo.bind(null,V,t.queue,!0,!1),Wt().memoizedState=t,[!1,t]},useSyncExternalStore:function(t,l,e){var a=V,u=Wt();if(lt){if(e===void 0)throw Error(s(407));e=e()}else{if(e=l(),gt===null)throw Error(s(349));(I&127)!==0||co(a,l,e)}u.memoizedState=e;var n={value:e,getSnapshot:l};return u.queue=n,zo(so.bind(null,a,n,t),[t]),a.flags|=2048,ga(9,{destroy:void 0},fo.bind(null,a,n,e,l),null),e},useId:function(){var t=Wt(),l=gt.identifierPrefix;if(lt){var e=Nl,a=ql;e=(a&~(1<<32-cl(a)-1)).toString(32)+e,l="_"+l+"R_"+e,e=on++,0<e&&(l+="H"+e.toString(32)),l+="_"}else e=Yy++,l="_"+l+"r_"+e.toString(32)+"_";return t.memoizedState=l},useHostTransitionStatus:yc,useFormState:bo,useActionState:bo,useOptimistic:function(t){var l=Wt();l.memoizedState=l.baseState=t;var e={pending:null,lanes:0,dispatch:null,lastRenderedReducer:null,lastRenderedState:null};return l.queue=e,l=vc.bind(null,V,!0,e),e.dispatch=l,[t,l]},useMemoCache:nc,useCacheRefresh:function(){return Wt().memoizedState=Zy.bind(null,V)},useEffectEvent:function(t){var l=Wt(),e={impl:t};return l.memoizedState=e,function(){if((nt&2)!==0)throw Error(s(440));return e.impl.apply(void 0,arguments)}}},hc={readContext:Lt,use:dn,useCallback:Ho,useContext:Lt,useEffect:oc,useImperativeHandle:Uo,useInsertionEffect:_o,useLayoutEffect:Mo,useMemo:Co,useReducer:mn,useRef:po,useState:function(){return mn(Kl)},useDebugValue:rc,useDeferredValue:function(t,l){var e=Dt();return Ro(e,rt.memoizedState,t,l)},useTransition:function(){var t=mn(Kl)[0],l=Dt().memoizedState;return[typeof t=="boolean"?t:nu(t),l]},useSyncExternalStore:io,useId:jo,useHostTransitionStatus:yc,useFormState:To,useActionState:To,useOptimistic:function(t,l){var e=Dt();return mo(e,rt,t,l)},useMemoCache:nc,useCacheRefresh:Yo};hc.useEffectEvent=Oo;var Lo={readContext:Lt,use:dn,useCallback:Ho,useContext:Lt,useEffect:oc,useImperativeHandle:Uo,useInsertionEffect:_o,useLayoutEffect:Mo,useMemo:Co,useReducer:cc,useRef:po,useState:function(){return cc(Kl)},useDebugValue:rc,useDeferredValue:function(t,l){var e=Dt();return rt===null?dc(e,t,l):Ro(e,rt.memoizedState,t,l)},useTransition:function(){var t=cc(Kl)[0],l=Dt().memoizedState;return[typeof t=="boolean"?t:nu(t),l]},useSyncExternalStore:io,useId:jo,useHostTransitionStatus:yc,useFormState:Ao,useActionState:Ao,useOptimistic:function(t,l){var e=Dt();return rt!==null?mo(e,rt,t,l):(e.baseState=t,[t,e.queue.dispatch])},useMemoCache:nc,useCacheRefresh:Yo};Lo.useEffectEvent=Oo;function gc(t,l,e,a){l=t.memoizedState,e=e(a,l),e=e==null?l:N({},l,e),t.memoizedState=e,t.lanes===0&&(t.updateQueue.baseState=e)}var Sc={enqueueSetState:function(t,l,e){t=t._reactInternals;var a=yl(),u=se(a);u.payload=l,e!=null&&(u.callback=e),l=oe(t,u,a),l!==null&&(el(l,t,a),lu(l,t,a))},enqueueReplaceState:function(t,l,e){t=t._reactInternals;var a=yl(),u=se(a);u.tag=1,u.payload=l,e!=null&&(u.callback=e),l=oe(t,u,a),l!==null&&(el(l,t,a),lu(l,t,a))},enqueueForceUpdate:function(t,l){t=t._reactInternals;var e=yl(),a=se(e);a.tag=2,l!=null&&(a.callback=l),l=oe(t,a,e),l!==null&&(el(l,t,e),lu(l,t,e))}};function Zo(t,l,e,a,u,n,i){return t=t.stateNode,typeof t.shouldComponentUpdate=="function"?t.shouldComponentUpdate(a,n,i):l.prototype&&l.prototype.isPureReactComponent?!wa(e,a)||!wa(u,n):!0}function Vo(t,l,e,a){t=l.state,typeof l.componentWillReceiveProps=="function"&&l.componentWillReceiveProps(e,a),typeof l.UNSAFE_componentWillReceiveProps=="function"&&l.UNSAFE_componentWillReceiveProps(e,a),l.state!==t&&Sc.enqueueReplaceState(l,l.state,null)}function Ze(t,l){var e=l;if("ref"in l){e={};for(var a in l)a!=="ref"&&(e[a]=l[a])}if(t=t.defaultProps){e===l&&(e=N({},e));for(var u in t)e[u]===void 0&&(e[u]=t[u])}return e}function Ko(t){wu(t)}function Jo(t){console.error(t)}function wo(t){wu(t)}function gn(t,l){try{var e=t.onUncaughtError;e(l.value,{componentStack:l.stack})}catch(a){setTimeout(function(){throw a})}}function $o(t,l,e){try{var a=t.onCaughtError;a(e.value,{componentStack:e.stack,errorBoundary:l.tag===1?l.stateNode:null})}catch(u){setTimeout(function(){throw u})}}function bc(t,l,e){return e=se(e),e.tag=3,e.payload={element:null},e.callback=function(){gn(t,l)},e}function Wo(t){return t=se(t),t.tag=3,t}function ko(t,l,e,a){var u=e.type.getDerivedStateFromError;if(typeof u=="function"){var n=a.value;t.payload=function(){return u(n)},t.callback=function(){$o(l,e,a)}}var i=e.stateNode;i!==null&&typeof i.componentDidCatch=="function"&&(t.callback=function(){$o(l,e,a),typeof u!="function"&&(he===null?he=new Set([this]):he.add(this));var c=a.stack;this.componentDidCatch(a.value,{componentStack:c!==null?c:""})})}function Ky(t,l,e,a,u){if(e.flags|=32768,a!==null&&typeof a=="object"&&typeof a.then=="function"){if(l=e.alternate,l!==null&&sa(l,e,u,!0),e=ol.current,e!==null){switch(e.tag){case 31:case 13:return pl===null?Un():e.alternate===null&&Ot===0&&(Ot=3),e.flags&=-257,e.flags|=65536,e.lanes=u,a===an?e.flags|=16384:(l=e.updateQueue,l===null?e.updateQueue=new Set([a]):l.add(a),Vc(t,a,u)),!1;case 22:return e.flags|=65536,a===an?e.flags|=16384:(l=e.updateQueue,l===null?(l={transitions:null,markerInstances:null,retryQueue:new Set([a])},e.updateQueue=l):(e=l.retryQueue,e===null?l.retryQueue=new Set([a]):e.add(a)),Vc(t,a,u)),!1}throw Error(s(435,e.tag))}return Vc(t,a,u),Un(),!1}if(lt)return l=ol.current,l!==null?((l.flags&65536)===0&&(l.flags|=256),l.flags|=65536,l.lanes=u,a!==Gi&&(t=Error(s(422),{cause:a}),ka(bl(t,e)))):(a!==Gi&&(l=Error(s(423),{cause:a}),ka(bl(l,e))),t=t.current.alternate,t.flags|=65536,u&=-u,t.lanes|=u,a=bl(a,e),u=bc(t.stateNode,a,u),Wi(t,u),Ot!==4&&(Ot=2)),!1;var n=Error(s(520),{cause:a});if(n=bl(n,e),vu===null?vu=[n]:vu.push(n),Ot!==4&&(Ot=2),l===null)return!0;a=bl(a,e),e=l;do{switch(e.tag){case 3:return e.flags|=65536,t=u&-u,e.lanes|=t,t=bc(e.stateNode,a,t),Wi(e,t),!1;case 1:if(l=e.type,n=e.stateNode,(e.flags&128)===0&&(typeof l.getDerivedStateFromError=="function"||n!==null&&typeof n.componentDidCatch=="function"&&(he===null||!he.has(n))))return e.flags|=65536,u&=-u,e.lanes|=u,u=Wo(u),ko(u,t,e,a),Wi(e,u),!1}e=e.return}while(e!==null);return!1}var Tc=Error(s(461)),Ct=!1;function Zt(t,l,e,a){l.child=t===null?Ps(l,null,e,a):Qe(l,t.child,e,a)}function Fo(t,l,e,a,u){e=e.render;var n=l.ref;if("ref"in a){var i={};for(var c in a)c!=="ref"&&(i[c]=a[c])}else i=a;return Ye(l),a=lc(t,l,e,i,n,u),c=ec(),t!==null&&!Ct?(ac(t,l,u),Jl(t,l,u)):(lt&&c&&ji(l),l.flags|=1,Zt(t,l,a,u),l.child)}function Io(t,l,e,a,u){if(t===null){var n=e.type;return typeof n=="function"&&!qi(n)&&n.defaultProps===void 0&&e.compare===null?(l.tag=15,l.type=n,Po(t,l,n,a,u)):(t=Fu(e.type,null,a,l,l.mode,u),t.ref=l.ref,t.return=l,l.child=t)}if(n=t.child,!Dc(t,u)){var i=n.memoizedProps;if(e=e.compare,e=e!==null?e:wa,e(i,a)&&t.ref===l.ref)return Jl(t,l,u)}return l.flags|=1,t=xl(n,a),t.ref=l.ref,t.return=l,l.child=t}function Po(t,l,e,a,u){if(t!==null){var n=t.memoizedProps;if(wa(n,a)&&t.ref===l.ref)if(Ct=!1,l.pendingProps=a=n,Dc(t,u))(t.flags&131072)!==0&&(Ct=!0);else return l.lanes=t.lanes,Jl(t,l,u)}return Ec(t,l,e,a,u)}function tr(t,l,e,a){var u=a.children,n=t!==null?t.memoizedState:null;if(t===null&&l.stateNode===null&&(l.stateNode={_visibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null}),a.mode==="hidden"){if((l.flags&128)!==0){if(n=n!==null?n.baseLanes|e:e,t!==null){for(a=l.child=t.child,u=0;a!==null;)u=u|a.lanes|a.childLanes,a=a.sibling;a=u&~n}else a=0,l.child=null;return lr(t,l,n,e,a)}if((e&536870912)!==0)l.memoizedState={baseLanes:0,cachePool:null},t!==null&&ln(l,n!==null?n.cachePool:null),n!==null?eo(l,n):Fi(),ao(l);else return a=l.lanes=536870912,lr(t,l,n!==null?n.baseLanes|e:e,e,a)}else n!==null?(ln(l,n.cachePool),eo(l,n),de(),l.memoizedState=null):(t!==null&&ln(l,null),Fi(),de());return Zt(t,l,u,e),l.child}function fu(t,l){return t!==null&&t.tag===22||l.stateNode!==null||(l.stateNode={_visibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null}),l.sibling}function lr(t,l,e,a,u){var n=Ki();return n=n===null?null:{parent:Ut._currentValue,pool:n},l.memoizedState={baseLanes:e,cachePool:n},t!==null&&ln(l,null),Fi(),ao(l),t!==null&&sa(t,l,a,!0),l.childLanes=u,null}function Sn(t,l){return l=Tn({mode:l.mode,children:l.children},t.mode),l.ref=t.ref,t.child=l,l.return=t,l}function er(t,l,e){return Qe(l,t.child,null,e),t=Sn(l,l.pendingProps),t.flags|=2,rl(l),l.memoizedState=null,t}function Jy(t,l,e){var a=l.pendingProps,u=(l.flags&128)!==0;if(l.flags&=-129,t===null){if(lt){if(a.mode==="hidden")return t=Sn(l,a),l.lanes=536870912,fu(null,t);if(Pi(l),(t=St)?(t=yd(t,Al),t=t!==null&&t.data==="&"?t:null,t!==null&&(l.memoizedState={dehydrated:t,treeContext:ue!==null?{id:ql,overflow:Nl}:null,retryLane:536870912,hydrationErrors:null},e=Gs(t),e.return=l,l.child=e,Qt=l,St=null)):t=null,t===null)throw ie(l);return l.lanes=536870912,null}return Sn(l,a)}var n=t.memoizedState;if(n!==null){var i=n.dehydrated;if(Pi(l),u)if(l.flags&256)l.flags&=-257,l=er(t,l,e);else if(l.memoizedState!==null)l.child=t.child,l.flags|=128,l=null;else throw Error(s(558));else if(Ct||sa(t,l,e,!1),u=(e&t.childLanes)!==0,Ct||u){if(a=gt,a!==null&&(i=Kf(a,e),i!==0&&i!==n.retryLane))throw n.retryLane=i,qe(t,i),el(a,t,i),Tc;Un(),l=er(t,l,e)}else t=n.treeContext,St=zl(i.nextSibling),Qt=l,lt=!0,ne=null,Al=!1,t!==null&&Qs(l,t),l=Sn(l,a),l.flags|=4096;return l}return t=xl(t.child,{mode:a.mode,children:a.children}),t.ref=l.ref,l.child=t,t.return=l,t}function bn(t,l){var e=l.ref;if(e===null)t!==null&&t.ref!==null&&(l.flags|=4194816);else{if(typeof e!="function"&&typeof e!="object")throw Error(s(284));(t===null||t.ref!==e)&&(l.flags|=4194816)}}function Ec(t,l,e,a,u){return Ye(l),e=lc(t,l,e,a,void 0,u),a=ec(),t!==null&&!Ct?(ac(t,l,u),Jl(t,l,u)):(lt&&a&&ji(l),l.flags|=1,Zt(t,l,e,u),l.child)}function ar(t,l,e,a,u,n){return Ye(l),l.updateQueue=null,e=no(l,a,e,u),uo(t),a=ec(),t!==null&&!Ct?(ac(t,l,n),Jl(t,l,n)):(lt&&a&&ji(l),l.flags|=1,Zt(t,l,e,n),l.child)}function ur(t,l,e,a,u){if(Ye(l),l.stateNode===null){var n=na,i=e.contextType;typeof i=="object"&&i!==null&&(n=Lt(i)),n=new e(a,n),l.memoizedState=n.state!==null&&n.state!==void 0?n.state:null,n.updater=Sc,l.stateNode=n,n._reactInternals=l,n=l.stateNode,n.props=a,n.state=l.memoizedState,n.refs={},wi(l),i=e.contextType,n.context=typeof i=="object"&&i!==null?Lt(i):na,n.state=l.memoizedState,i=e.getDerivedStateFromProps,typeof i=="function"&&(gc(l,e,i,a),n.state=l.memoizedState),typeof e.getDerivedStateFromProps=="function"||typeof n.getSnapshotBeforeUpdate=="function"||typeof n.UNSAFE_componentWillMount!="function"&&typeof n.componentWillMount!="function"||(i=n.state,typeof n.componentWillMount=="function"&&n.componentWillMount(),typeof n.UNSAFE_componentWillMount=="function"&&n.UNSAFE_componentWillMount(),i!==n.state&&Sc.enqueueReplaceState(n,n.state,null),au(l,a,n,u),eu(),n.state=l.memoizedState),typeof n.componentDidMount=="function"&&(l.flags|=4194308),a=!0}else if(t===null){n=l.stateNode;var c=l.memoizedProps,o=Ze(e,c);n.props=o;var S=n.context,z=e.contextType;i=na,typeof z=="object"&&z!==null&&(i=Lt(z));var M=e.getDerivedStateFromProps;z=typeof M=="function"||typeof n.getSnapshotBeforeUpdate=="function",c=l.pendingProps!==c,z||typeof n.UNSAFE_componentWillReceiveProps!="function"&&typeof n.componentWillReceiveProps!="function"||(c||S!==i)&&Vo(l,n,a,i),fe=!1;var b=l.memoizedState;n.state=b,au(l,a,n,u),eu(),S=l.memoizedState,c||b!==S||fe?(typeof M=="function"&&(gc(l,e,M,a),S=l.memoizedState),(o=fe||Zo(l,e,o,a,b,S,i))?(z||typeof n.UNSAFE_componentWillMount!="function"&&typeof n.componentWillMount!="function"||(typeof n.componentWillMount=="function"&&n.componentWillMount(),typeof n.UNSAFE_componentWillMount=="function"&&n.UNSAFE_componentWillMount()),typeof n.componentDidMount=="function"&&(l.flags|=4194308)):(typeof n.componentDidMount=="function"&&(l.flags|=4194308),l.memoizedProps=a,l.memoizedState=S),n.props=a,n.state=S,n.context=i,a=o):(typeof n.componentDidMount=="function"&&(l.flags|=4194308),a=!1)}else{n=l.stateNode,$i(t,l),i=l.memoizedProps,z=Ze(e,i),n.props=z,M=l.pendingProps,b=n.context,S=e.contextType,o=na,typeof S=="object"&&S!==null&&(o=Lt(S)),c=e.getDerivedStateFromProps,(S=typeof c=="function"||typeof n.getSnapshotBeforeUpdate=="function")||typeof n.UNSAFE_componentWillReceiveProps!="function"&&typeof n.componentWillReceiveProps!="function"||(i!==M||b!==o)&&Vo(l,n,a,o),fe=!1,b=l.memoizedState,n.state=b,au(l,a,n,u),eu();var E=l.memoizedState;i!==M||b!==E||fe||t!==null&&t.dependencies!==null&&Pu(t.dependencies)?(typeof c=="function"&&(gc(l,e,c,a),E=l.memoizedState),(z=fe||Zo(l,e,z,a,b,E,o)||t!==null&&t.dependencies!==null&&Pu(t.dependencies))?(S||typeof n.UNSAFE_componentWillUpdate!="function"&&typeof n.componentWillUpdate!="function"||(typeof n.componentWillUpdate=="function"&&n.componentWillUpdate(a,E,o),typeof n.UNSAFE_componentWillUpdate=="function"&&n.UNSAFE_componentWillUpdate(a,E,o)),typeof n.componentDidUpdate=="function"&&(l.flags|=4),typeof n.getSnapshotBeforeUpdate=="function"&&(l.flags|=1024)):(typeof n.componentDidUpdate!="function"||i===t.memoizedProps&&b===t.memoizedState||(l.flags|=4),typeof n.getSnapshotBeforeUpdate!="function"||i===t.memoizedProps&&b===t.memoizedState||(l.flags|=1024),l.memoizedProps=a,l.memoizedState=E),n.props=a,n.state=E,n.context=o,a=z):(typeof n.componentDidUpdate!="function"||i===t.memoizedProps&&b===t.memoizedState||(l.flags|=4),typeof n.getSnapshotBeforeUpdate!="function"||i===t.memoizedProps&&b===t.memoizedState||(l.flags|=1024),a=!1)}return n=a,bn(t,l),a=(l.flags&128)!==0,n||a?(n=l.stateNode,e=a&&typeof e.getDerivedStateFromError!="function"?null:n.render(),l.flags|=1,t!==null&&a?(l.child=Qe(l,t.child,null,u),l.child=Qe(l,null,e,u)):Zt(t,l,e,u),l.memoizedState=n.state,t=l.child):t=Jl(t,l,u),t}function nr(t,l,e,a){return Be(),l.flags|=256,Zt(t,l,e,a),l.child}var Ac={dehydrated:null,treeContext:null,retryLane:0,hydrationErrors:null};function pc(t){return{baseLanes:t,cachePool:ws()}}function zc(t,l,e){return t=t!==null?t.childLanes&~e:0,l&&(t|=ml),t}function ir(t,l,e){var a=l.pendingProps,u=!1,n=(l.flags&128)!==0,i;if((i=n)||(i=t!==null&&t.memoizedState===null?!1:(Mt.current&2)!==0),i&&(u=!0,l.flags&=-129),i=(l.flags&32)!==0,l.flags&=-33,t===null){if(lt){if(u?re(l):de(),(t=St)?(t=yd(t,Al),t=t!==null&&t.data!=="&"?t:null,t!==null&&(l.memoizedState={dehydrated:t,treeContext:ue!==null?{id:ql,overflow:Nl}:null,retryLane:536870912,hydrationErrors:null},e=Gs(t),e.return=l,l.child=e,Qt=l,St=null)):t=null,t===null)throw ie(l);return cf(t)?l.lanes=32:l.lanes=536870912,null}var c=a.children;return a=a.fallback,u?(de(),u=l.mode,c=Tn({mode:"hidden",children:c},u),a=Ne(a,u,e,null),c.return=l,a.return=l,c.sibling=a,l.child=c,a=l.child,a.memoizedState=pc(e),a.childLanes=zc(t,i,e),l.memoizedState=Ac,fu(null,a)):(re(l),Oc(l,c))}var o=t.memoizedState;if(o!==null&&(c=o.dehydrated,c!==null)){if(n)l.flags&256?(re(l),l.flags&=-257,l=_c(t,l,e)):l.memoizedState!==null?(de(),l.child=t.child,l.flags|=128,l=null):(de(),c=a.fallback,u=l.mode,a=Tn({mode:"visible",children:a.children},u),c=Ne(c,u,e,null),c.flags|=2,a.return=l,c.return=l,a.sibling=c,l.child=a,Qe(l,t.child,null,e),a=l.child,a.memoizedState=pc(e),a.childLanes=zc(t,i,e),l.memoizedState=Ac,l=fu(null,a));else if(re(l),cf(c)){if(i=c.nextSibling&&c.nextSibling.dataset,i)var S=i.dgst;i=S,a=Error(s(419)),a.stack="",a.digest=i,ka({value:a,source:null,stack:null}),l=_c(t,l,e)}else if(Ct||sa(t,l,e,!1),i=(e&t.childLanes)!==0,Ct||i){if(i=gt,i!==null&&(a=Kf(i,e),a!==0&&a!==o.retryLane))throw o.retryLane=a,qe(t,a),el(i,t,a),Tc;nf(c)||Un(),l=_c(t,l,e)}else nf(c)?(l.flags|=192,l.child=t.child,l=null):(t=o.treeContext,St=zl(c.nextSibling),Qt=l,lt=!0,ne=null,Al=!1,t!==null&&Qs(l,t),l=Oc(l,a.children),l.flags|=4096);return l}return u?(de(),c=a.fallback,u=l.mode,o=t.child,S=o.sibling,a=xl(o,{mode:"hidden",children:a.children}),a.subtreeFlags=o.subtreeFlags&65011712,S!==null?c=xl(S,c):(c=Ne(c,u,e,null),c.flags|=2),c.return=l,a.return=l,a.sibling=c,l.child=a,fu(null,a),a=l.child,c=t.child.memoizedState,c===null?c=pc(e):(u=c.cachePool,u!==null?(o=Ut._currentValue,u=u.parent!==o?{parent:o,pool:o}:u):u=ws(),c={baseLanes:c.baseLanes|e,cachePool:u}),a.memoizedState=c,a.childLanes=zc(t,i,e),l.memoizedState=Ac,fu(t.child,a)):(re(l),e=t.child,t=e.sibling,e=xl(e,{mode:"visible",children:a.children}),e.return=l,e.sibling=null,t!==null&&(i=l.deletions,i===null?(l.deletions=[t],l.flags|=16):i.push(t)),l.child=e,l.memoizedState=null,e)}function Oc(t,l){return l=Tn({mode:"visible",children:l},t.mode),l.return=t,t.child=l}function Tn(t,l){return t=sl(22,t,null,l),t.lanes=0,t}function _c(t,l,e){return Qe(l,t.child,null,e),t=Oc(l,l.pendingProps.children),t.flags|=2,l.memoizedState=null,t}function cr(t,l,e){t.lanes|=l;var a=t.alternate;a!==null&&(a.lanes|=l),Qi(t.return,l,e)}function Mc(t,l,e,a,u,n){var i=t.memoizedState;i===null?t.memoizedState={isBackwards:l,rendering:null,renderingStartTime:0,last:a,tail:e,tailMode:u,treeForkCount:n}:(i.isBackwards=l,i.rendering=null,i.renderingStartTime=0,i.last=a,i.tail=e,i.tailMode=u,i.treeForkCount=n)}function fr(t,l,e){var a=l.pendingProps,u=a.revealOrder,n=a.tail;a=a.children;var i=Mt.current,c=(i&2)!==0;if(c?(i=i&1|2,l.flags|=128):i&=1,q(Mt,i),Zt(t,l,a,e),a=lt?Wa:0,!c&&t!==null&&(t.flags&128)!==0)t:for(t=l.child;t!==null;){if(t.tag===13)t.memoizedState!==null&&cr(t,e,l);else if(t.tag===19)cr(t,e,l);else if(t.child!==null){t.child.return=t,t=t.child;continue}if(t===l)break t;for(;t.sibling===null;){if(t.return===null||t.return===l)break t;t=t.return}t.sibling.return=t.return,t=t.sibling}switch(u){case"forwards":for(e=l.child,u=null;e!==null;)t=e.alternate,t!==null&&fn(t)===null&&(u=e),e=e.sibling;e=u,e===null?(u=l.child,l.child=null):(u=e.sibling,e.sibling=null),Mc(l,!1,u,e,n,a);break;case"backwards":case"unstable_legacy-backwards":for(e=null,u=l.child,l.child=null;u!==null;){if(t=u.alternate,t!==null&&fn(t)===null){l.child=u;break}t=u.sibling,u.sibling=e,e=u,u=t}Mc(l,!0,e,null,n,a);break;case"together":Mc(l,!1,null,null,void 0,a);break;default:l.memoizedState=null}return l.child}function Jl(t,l,e){if(t!==null&&(l.dependencies=t.dependencies),ve|=l.lanes,(e&l.childLanes)===0)if(t!==null){if(sa(t,l,e,!1),(e&l.childLanes)===0)return null}else return null;if(t!==null&&l.child!==t.child)throw Error(s(153));if(l.child!==null){for(t=l.child,e=xl(t,t.pendingProps),l.child=e,e.return=l;t.sibling!==null;)t=t.sibling,e=e.sibling=xl(t,t.pendingProps),e.return=l;e.sibling=null}return l.child}function Dc(t,l){return(t.lanes&l)!==0?!0:(t=t.dependencies,!!(t!==null&&Pu(t)))}function wy(t,l,e){switch(l.tag){case 3:$t(l,l.stateNode.containerInfo),ce(l,Ut,t.memoizedState.cache),Be();break;case 27:case 5:Na(l);break;case 4:$t(l,l.stateNode.containerInfo);break;case 10:ce(l,l.type,l.memoizedProps.value);break;case 31:if(l.memoizedState!==null)return l.flags|=128,Pi(l),null;break;case 13:var a=l.memoizedState;if(a!==null)return a.dehydrated!==null?(re(l),l.flags|=128,null):(e&l.child.childLanes)!==0?ir(t,l,e):(re(l),t=Jl(t,l,e),t!==null?t.sibling:null);re(l);break;case 19:var u=(t.flags&128)!==0;if(a=(e&l.childLanes)!==0,a||(sa(t,l,e,!1),a=(e&l.childLanes)!==0),u){if(a)return fr(t,l,e);l.flags|=128}if(u=l.memoizedState,u!==null&&(u.rendering=null,u.tail=null,u.lastEffect=null),q(Mt,Mt.current),a)break;return null;case 22:return l.lanes=0,tr(t,l,e,l.pendingProps);case 24:ce(l,Ut,t.memoizedState.cache)}return Jl(t,l,e)}function sr(t,l,e){if(t!==null)if(t.memoizedProps!==l.pendingProps)Ct=!0;else{if(!Dc(t,e)&&(l.flags&128)===0)return Ct=!1,wy(t,l,e);Ct=(t.flags&131072)!==0}else Ct=!1,lt&&(l.flags&1048576)!==0&&xs(l,Wa,l.index);switch(l.lanes=0,l.tag){case 16:t:{var a=l.pendingProps;if(t=Xe(l.elementType),l.type=t,typeof t=="function")qi(t)?(a=Ze(t,a),l.tag=1,l=ur(null,l,t,a,e)):(l.tag=0,l=Ec(null,l,t,a,e));else{if(t!=null){var u=t.$$typeof;if(u===Jt){l.tag=11,l=Fo(null,l,t,a,e);break t}else if(u===$){l.tag=14,l=Io(null,l,t,a,e);break t}}throw l=Dl(t)||t,Error(s(306,l,""))}}return l;case 0:return Ec(t,l,l.type,l.pendingProps,e);case 1:return a=l.type,u=Ze(a,l.pendingProps),ur(t,l,a,u,e);case 3:t:{if($t(l,l.stateNode.containerInfo),t===null)throw Error(s(387));a=l.pendingProps;var n=l.memoizedState;u=n.element,$i(t,l),au(l,a,null,e);var i=l.memoizedState;if(a=i.cache,ce(l,Ut,a),a!==n.cache&&Li(l,[Ut],e,!0),eu(),a=i.element,n.isDehydrated)if(n={element:a,isDehydrated:!1,cache:i.cache},l.updateQueue.baseState=n,l.memoizedState=n,l.flags&256){l=nr(t,l,a,e);break t}else if(a!==u){u=bl(Error(s(424)),l),ka(u),l=nr(t,l,a,e);break t}else{switch(t=l.stateNode.containerInfo,t.nodeType){case 9:t=t.body;break;default:t=t.nodeName==="HTML"?t.ownerDocument.body:t}for(St=zl(t.firstChild),Qt=l,lt=!0,ne=null,Al=!0,e=Ps(l,null,a,e),l.child=e;e;)e.flags=e.flags&-3|4096,e=e.sibling}else{if(Be(),a===u){l=Jl(t,l,e);break t}Zt(t,l,a,e)}l=l.child}return l;case 26:return bn(t,l),t===null?(e=Td(l.type,null,l.pendingProps,null))?l.memoizedState=e:lt||(e=l.type,t=l.pendingProps,a=jn(W.current).createElement(e),a[xt]=l,a[kt]=t,Vt(a,e,t),jt(a),l.stateNode=a):l.memoizedState=Td(l.type,t.memoizedProps,l.pendingProps,t.memoizedState),null;case 27:return Na(l),t===null&&lt&&(a=l.stateNode=gd(l.type,l.pendingProps,W.current),Qt=l,Al=!0,u=St,Te(l.type)?(ff=u,St=zl(a.firstChild)):St=u),Zt(t,l,l.pendingProps.children,e),bn(t,l),t===null&&(l.flags|=4194304),l.child;case 5:return t===null&&lt&&((u=a=St)&&(a=pv(a,l.type,l.pendingProps,Al),a!==null?(l.stateNode=a,Qt=l,St=zl(a.firstChild),Al=!1,u=!0):u=!1),u||ie(l)),Na(l),u=l.type,n=l.pendingProps,i=t!==null?t.memoizedProps:null,a=n.children,ef(u,n)?a=null:i!==null&&ef(u,i)&&(l.flags|=32),l.memoizedState!==null&&(u=lc(t,l,Gy,null,null,e),pu._currentValue=u),bn(t,l),Zt(t,l,a,e),l.child;case 6:return t===null&&lt&&((t=e=St)&&(e=zv(e,l.pendingProps,Al),e!==null?(l.stateNode=e,Qt=l,St=null,t=!0):t=!1),t||ie(l)),null;case 13:return ir(t,l,e);case 4:return $t(l,l.stateNode.containerInfo),a=l.pendingProps,t===null?l.child=Qe(l,null,a,e):Zt(t,l,a,e),l.child;case 11:return Fo(t,l,l.type,l.pendingProps,e);case 7:return Zt(t,l,l.pendingProps,e),l.child;case 8:return Zt(t,l,l.pendingProps.children,e),l.child;case 12:return Zt(t,l,l.pendingProps.children,e),l.child;case 10:return a=l.pendingProps,ce(l,l.type,a.value),Zt(t,l,a.children,e),l.child;case 9:return u=l.type._context,a=l.pendingProps.children,Ye(l),u=Lt(u),a=a(u),l.flags|=1,Zt(t,l,a,e),l.child;case 14:return Io(t,l,l.type,l.pendingProps,e);case 15:return Po(t,l,l.type,l.pendingProps,e);case 19:return fr(t,l,e);case 31:return Jy(t,l,e);case 22:return tr(t,l,e,l.pendingProps);case 24:return Ye(l),a=Lt(Ut),t===null?(u=Ki(),u===null&&(u=gt,n=Zi(),u.pooledCache=n,n.refCount++,n!==null&&(u.pooledCacheLanes|=e),u=n),l.memoizedState={parent:a,cache:u},wi(l),ce(l,Ut,u)):((t.lanes&e)!==0&&($i(t,l),au(l,null,null,e),eu()),u=t.memoizedState,n=l.memoizedState,u.parent!==a?(u={parent:a,cache:a},l.memoizedState=u,l.lanes===0&&(l.memoizedState=l.updateQueue.baseState=u),ce(l,Ut,a)):(a=n.cache,ce(l,Ut,a),a!==u.cache&&Li(l,[Ut],e,!0))),Zt(t,l,l.pendingProps.children,e),l.child;case 29:throw l.pendingProps}throw Error(s(156,l.tag))}function wl(t){t.flags|=4}function Uc(t,l,e,a,u){if((l=(t.mode&32)!==0)&&(l=!1),l){if(t.flags|=16777216,(u&335544128)===u)if(t.stateNode.complete)t.flags|=8192;else if(jr())t.flags|=8192;else throw xe=an,Ji}else t.flags&=-16777217}function or(t,l){if(l.type!=="stylesheet"||(l.state.loading&4)!==0)t.flags&=-16777217;else if(t.flags|=16777216,!Od(l))if(jr())t.flags|=8192;else throw xe=an,Ji}function En(t,l){l!==null&&(t.flags|=4),t.flags&16384&&(l=t.tag!==22?Lf():536870912,t.lanes|=l,Ea|=l)}function su(t,l){if(!lt)switch(t.tailMode){case"hidden":l=t.tail;for(var e=null;l!==null;)l.alternate!==null&&(e=l),l=l.sibling;e===null?t.tail=null:e.sibling=null;break;case"collapsed":e=t.tail;for(var a=null;e!==null;)e.alternate!==null&&(a=e),e=e.sibling;a===null?l||t.tail===null?t.tail=null:t.tail.sibling=null:a.sibling=null}}function bt(t){var l=t.alternate!==null&&t.alternate.child===t.child,e=0,a=0;if(l)for(var u=t.child;u!==null;)e|=u.lanes|u.childLanes,a|=u.subtreeFlags&65011712,a|=u.flags&65011712,u.return=t,u=u.sibling;else for(u=t.child;u!==null;)e|=u.lanes|u.childLanes,a|=u.subtreeFlags,a|=u.flags,u.return=t,u=u.sibling;return t.subtreeFlags|=a,t.childLanes=e,l}function $y(t,l,e){var a=l.pendingProps;switch(Yi(l),l.tag){case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return bt(l),null;case 1:return bt(l),null;case 3:return e=l.stateNode,a=null,t!==null&&(a=t.memoizedState.cache),l.memoizedState.cache!==a&&(l.flags|=2048),Zl(Ut),_t(),e.pendingContext&&(e.context=e.pendingContext,e.pendingContext=null),(t===null||t.child===null)&&(fa(l)?wl(l):t===null||t.memoizedState.isDehydrated&&(l.flags&256)===0||(l.flags|=1024,Xi())),bt(l),null;case 26:var u=l.type,n=l.memoizedState;return t===null?(wl(l),n!==null?(bt(l),or(l,n)):(bt(l),Uc(l,u,null,a,e))):n?n!==t.memoizedState?(wl(l),bt(l),or(l,n)):(bt(l),l.flags&=-16777217):(t=t.memoizedProps,t!==a&&wl(l),bt(l),Uc(l,u,t,a,e)),null;case 27:if(Cu(l),e=W.current,u=l.type,t!==null&&l.stateNode!=null)t.memoizedProps!==a&&wl(l);else{if(!a){if(l.stateNode===null)throw Error(s(166));return bt(l),null}t=j.current,fa(l)?Ls(l):(t=gd(u,a,e),l.stateNode=t,wl(l))}return bt(l),null;case 5:if(Cu(l),u=l.type,t!==null&&l.stateNode!=null)t.memoizedProps!==a&&wl(l);else{if(!a){if(l.stateNode===null)throw Error(s(166));return bt(l),null}if(n=j.current,fa(l))Ls(l);else{var i=jn(W.current);switch(n){case 1:n=i.createElementNS("http://www.w3.org/2000/svg",u);break;case 2:n=i.createElementNS("http://www.w3.org/1998/Math/MathML",u);break;default:switch(u){case"svg":n=i.createElementNS("http://www.w3.org/2000/svg",u);break;case"math":n=i.createElementNS("http://www.w3.org/1998/Math/MathML",u);break;case"script":n=i.createElement("div"),n.innerHTML="<script><\/script>",n=n.removeChild(n.firstChild);break;case"select":n=typeof a.is=="string"?i.createElement("select",{is:a.is}):i.createElement("select"),a.multiple?n.multiple=!0:a.size&&(n.size=a.size);break;default:n=typeof a.is=="string"?i.createElement(u,{is:a.is}):i.createElement(u)}}n[xt]=l,n[kt]=a;t:for(i=l.child;i!==null;){if(i.tag===5||i.tag===6)n.appendChild(i.stateNode);else if(i.tag!==4&&i.tag!==27&&i.child!==null){i.child.return=i,i=i.child;continue}if(i===l)break t;for(;i.sibling===null;){if(i.return===null||i.return===l)break t;i=i.return}i.sibling.return=i.return,i=i.sibling}l.stateNode=n;t:switch(Vt(n,u,a),u){case"button":case"input":case"select":case"textarea":a=!!a.autoFocus;break t;case"img":a=!0;break t;default:a=!1}a&&wl(l)}}return bt(l),Uc(l,l.type,t===null?null:t.memoizedProps,l.pendingProps,e),null;case 6:if(t&&l.stateNode!=null)t.memoizedProps!==a&&wl(l);else{if(typeof a!="string"&&l.stateNode===null)throw Error(s(166));if(t=W.current,fa(l)){if(t=l.stateNode,e=l.memoizedProps,a=null,u=Qt,u!==null)switch(u.tag){case 27:case 5:a=u.memoizedProps}t[xt]=l,t=!!(t.nodeValue===e||a!==null&&a.suppressHydrationWarning===!0||id(t.nodeValue,e)),t||ie(l,!0)}else t=jn(t).createTextNode(a),t[xt]=l,l.stateNode=t}return bt(l),null;case 31:if(e=l.memoizedState,t===null||t.memoizedState!==null){if(a=fa(l),e!==null){if(t===null){if(!a)throw Error(s(318));if(t=l.memoizedState,t=t!==null?t.dehydrated:null,!t)throw Error(s(557));t[xt]=l}else Be(),(l.flags&128)===0&&(l.memoizedState=null),l.flags|=4;bt(l),t=!1}else e=Xi(),t!==null&&t.memoizedState!==null&&(t.memoizedState.hydrationErrors=e),t=!0;if(!t)return l.flags&256?(rl(l),l):(rl(l),null);if((l.flags&128)!==0)throw Error(s(558))}return bt(l),null;case 13:if(a=l.memoizedState,t===null||t.memoizedState!==null&&t.memoizedState.dehydrated!==null){if(u=fa(l),a!==null&&a.dehydrated!==null){if(t===null){if(!u)throw Error(s(318));if(u=l.memoizedState,u=u!==null?u.dehydrated:null,!u)throw Error(s(317));u[xt]=l}else Be(),(l.flags&128)===0&&(l.memoizedState=null),l.flags|=4;bt(l),u=!1}else u=Xi(),t!==null&&t.memoizedState!==null&&(t.memoizedState.hydrationErrors=u),u=!0;if(!u)return l.flags&256?(rl(l),l):(rl(l),null)}return rl(l),(l.flags&128)!==0?(l.lanes=e,l):(e=a!==null,t=t!==null&&t.memoizedState!==null,e&&(a=l.child,u=null,a.alternate!==null&&a.alternate.memoizedState!==null&&a.alternate.memoizedState.cachePool!==null&&(u=a.alternate.memoizedState.cachePool.pool),n=null,a.memoizedState!==null&&a.memoizedState.cachePool!==null&&(n=a.memoizedState.cachePool.pool),n!==u&&(a.flags|=2048)),e!==t&&e&&(l.child.flags|=8192),En(l,l.updateQueue),bt(l),null);case 4:return _t(),t===null&&Fc(l.stateNode.containerInfo),bt(l),null;case 10:return Zl(l.type),bt(l),null;case 19:if(D(Mt),a=l.memoizedState,a===null)return bt(l),null;if(u=(l.flags&128)!==0,n=a.rendering,n===null)if(u)su(a,!1);else{if(Ot!==0||t!==null&&(t.flags&128)!==0)for(t=l.child;t!==null;){if(n=fn(t),n!==null){for(l.flags|=128,su(a,!1),t=n.updateQueue,l.updateQueue=t,En(l,t),l.subtreeFlags=0,t=e,e=l.child;e!==null;)Ys(e,t),e=e.sibling;return q(Mt,Mt.current&1|2),lt&&Ql(l,a.treeForkCount),l.child}t=t.sibling}a.tail!==null&&nl()>_n&&(l.flags|=128,u=!0,su(a,!1),l.lanes=4194304)}else{if(!u)if(t=fn(n),t!==null){if(l.flags|=128,u=!0,t=t.updateQueue,l.updateQueue=t,En(l,t),su(a,!0),a.tail===null&&a.tailMode==="hidden"&&!n.alternate&&!lt)return bt(l),null}else 2*nl()-a.renderingStartTime>_n&&e!==536870912&&(l.flags|=128,u=!0,su(a,!1),l.lanes=4194304);a.isBackwards?(n.sibling=l.child,l.child=n):(t=a.last,t!==null?t.sibling=n:l.child=n,a.last=n)}return a.tail!==null?(t=a.tail,a.rendering=t,a.tail=t.sibling,a.renderingStartTime=nl(),t.sibling=null,e=Mt.current,q(Mt,u?e&1|2:e&1),lt&&Ql(l,a.treeForkCount),t):(bt(l),null);case 22:case 23:return rl(l),Ii(),a=l.memoizedState!==null,t!==null?t.memoizedState!==null!==a&&(l.flags|=8192):a&&(l.flags|=8192),a?(e&536870912)!==0&&(l.flags&128)===0&&(bt(l),l.subtreeFlags&6&&(l.flags|=8192)):bt(l),e=l.updateQueue,e!==null&&En(l,e.retryQueue),e=null,t!==null&&t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(e=t.memoizedState.cachePool.pool),a=null,l.memoizedState!==null&&l.memoizedState.cachePool!==null&&(a=l.memoizedState.cachePool.pool),a!==e&&(l.flags|=2048),t!==null&&D(Ge),null;case 24:return e=null,t!==null&&(e=t.memoizedState.cache),l.memoizedState.cache!==e&&(l.flags|=2048),Zl(Ut),bt(l),null;case 25:return null;case 30:return null}throw Error(s(156,l.tag))}function Wy(t,l){switch(Yi(l),l.tag){case 1:return t=l.flags,t&65536?(l.flags=t&-65537|128,l):null;case 3:return Zl(Ut),_t(),t=l.flags,(t&65536)!==0&&(t&128)===0?(l.flags=t&-65537|128,l):null;case 26:case 27:case 5:return Cu(l),null;case 31:if(l.memoizedState!==null){if(rl(l),l.alternate===null)throw Error(s(340));Be()}return t=l.flags,t&65536?(l.flags=t&-65537|128,l):null;case 13:if(rl(l),t=l.memoizedState,t!==null&&t.dehydrated!==null){if(l.alternate===null)throw Error(s(340));Be()}return t=l.flags,t&65536?(l.flags=t&-65537|128,l):null;case 19:return D(Mt),null;case 4:return _t(),null;case 10:return Zl(l.type),null;case 22:case 23:return rl(l),Ii(),t!==null&&D(Ge),t=l.flags,t&65536?(l.flags=t&-65537|128,l):null;case 24:return Zl(Ut),null;case 25:return null;default:return null}}function rr(t,l){switch(Yi(l),l.tag){case 3:Zl(Ut),_t();break;case 26:case 27:case 5:Cu(l);break;case 4:_t();break;case 31:l.memoizedState!==null&&rl(l);break;case 13:rl(l);break;case 19:D(Mt);break;case 10:Zl(l.type);break;case 22:case 23:rl(l),Ii(),t!==null&&D(Ge);break;case 24:Zl(Ut)}}function ou(t,l){try{var e=l.updateQueue,a=e!==null?e.lastEffect:null;if(a!==null){var u=a.next;e=u;do{if((e.tag&t)===t){a=void 0;var n=e.create,i=e.inst;a=n(),i.destroy=a}e=e.next}while(e!==u)}}catch(c){st(l,l.return,c)}}function me(t,l,e){try{var a=l.updateQueue,u=a!==null?a.lastEffect:null;if(u!==null){var n=u.next;a=n;do{if((a.tag&t)===t){var i=a.inst,c=i.destroy;if(c!==void 0){i.destroy=void 0,u=l;var o=e,S=c;try{S()}catch(z){st(u,o,z)}}}a=a.next}while(a!==n)}}catch(z){st(l,l.return,z)}}function dr(t){var l=t.updateQueue;if(l!==null){var e=t.stateNode;try{lo(l,e)}catch(a){st(t,t.return,a)}}}function mr(t,l,e){e.props=Ze(t.type,t.memoizedProps),e.state=t.memoizedState;try{e.componentWillUnmount()}catch(a){st(t,l,a)}}function ru(t,l){try{var e=t.ref;if(e!==null){switch(t.tag){case 26:case 27:case 5:var a=t.stateNode;break;case 30:a=t.stateNode;break;default:a=t.stateNode}typeof e=="function"?t.refCleanup=e(a):e.current=a}}catch(u){st(t,l,u)}}function Bl(t,l){var e=t.ref,a=t.refCleanup;if(e!==null)if(typeof a=="function")try{a()}catch(u){st(t,l,u)}finally{t.refCleanup=null,t=t.alternate,t!=null&&(t.refCleanup=null)}else if(typeof e=="function")try{e(null)}catch(u){st(t,l,u)}else e.current=null}function yr(t){var l=t.type,e=t.memoizedProps,a=t.stateNode;try{t:switch(l){case"button":case"input":case"select":case"textarea":e.autoFocus&&a.focus();break t;case"img":e.src?a.src=e.src:e.srcSet&&(a.srcset=e.srcSet)}}catch(u){st(t,t.return,u)}}function Hc(t,l,e){try{var a=t.stateNode;gv(a,t.type,e,l),a[kt]=l}catch(u){st(t,t.return,u)}}function vr(t){return t.tag===5||t.tag===3||t.tag===26||t.tag===27&&Te(t.type)||t.tag===4}function Cc(t){t:for(;;){for(;t.sibling===null;){if(t.return===null||vr(t.return))return null;t=t.return}for(t.sibling.return=t.return,t=t.sibling;t.tag!==5&&t.tag!==6&&t.tag!==18;){if(t.tag===27&&Te(t.type)||t.flags&2||t.child===null||t.tag===4)continue t;t.child.return=t,t=t.child}if(!(t.flags&2))return t.stateNode}}function Rc(t,l,e){var a=t.tag;if(a===5||a===6)t=t.stateNode,l?(e.nodeType===9?e.body:e.nodeName==="HTML"?e.ownerDocument.body:e).insertBefore(t,l):(l=e.nodeType===9?e.body:e.nodeName==="HTML"?e.ownerDocument.body:e,l.appendChild(t),e=e._reactRootContainer,e!=null||l.onclick!==null||(l.onclick=Gl));else if(a!==4&&(a===27&&Te(t.type)&&(e=t.stateNode,l=null),t=t.child,t!==null))for(Rc(t,l,e),t=t.sibling;t!==null;)Rc(t,l,e),t=t.sibling}function An(t,l,e){var a=t.tag;if(a===5||a===6)t=t.stateNode,l?e.insertBefore(t,l):e.appendChild(t);else if(a!==4&&(a===27&&Te(t.type)&&(e=t.stateNode),t=t.child,t!==null))for(An(t,l,e),t=t.sibling;t!==null;)An(t,l,e),t=t.sibling}function hr(t){var l=t.stateNode,e=t.memoizedProps;try{for(var a=t.type,u=l.attributes;u.length;)l.removeAttributeNode(u[0]);Vt(l,a,e),l[xt]=t,l[kt]=e}catch(n){st(t,t.return,n)}}var $l=!1,Rt=!1,qc=!1,gr=typeof WeakSet=="function"?WeakSet:Set,Yt=null;function ky(t,l){if(t=t.containerInfo,tf=Zn,t=Ds(t),_i(t)){if("selectionStart"in t)var e={start:t.selectionStart,end:t.selectionEnd};else t:{e=(e=t.ownerDocument)&&e.defaultView||window;var a=e.getSelection&&e.getSelection();if(a&&a.rangeCount!==0){e=a.anchorNode;var u=a.anchorOffset,n=a.focusNode;a=a.focusOffset;try{e.nodeType,n.nodeType}catch{e=null;break t}var i=0,c=-1,o=-1,S=0,z=0,M=t,b=null;l:for(;;){for(var E;M!==e||u!==0&&M.nodeType!==3||(c=i+u),M!==n||a!==0&&M.nodeType!==3||(o=i+a),M.nodeType===3&&(i+=M.nodeValue.length),(E=M.firstChild)!==null;)b=M,M=E;for(;;){if(M===t)break l;if(b===e&&++S===u&&(c=i),b===n&&++z===a&&(o=i),(E=M.nextSibling)!==null)break;M=b,b=M.parentNode}M=E}e=c===-1||o===-1?null:{start:c,end:o}}else e=null}e=e||{start:0,end:0}}else e=null;for(lf={focusedElem:t,selectionRange:e},Zn=!1,Yt=l;Yt!==null;)if(l=Yt,t=l.child,(l.subtreeFlags&1028)!==0&&t!==null)t.return=l,Yt=t;else for(;Yt!==null;){switch(l=Yt,n=l.alternate,t=l.flags,l.tag){case 0:if((t&4)!==0&&(t=l.updateQueue,t=t!==null?t.events:null,t!==null))for(e=0;e<t.length;e++)u=t[e],u.ref.impl=u.nextImpl;break;case 11:case 15:break;case 1:if((t&1024)!==0&&n!==null){t=void 0,e=l,u=n.memoizedProps,n=n.memoizedState,a=e.stateNode;try{var B=Ze(e.type,u);t=a.getSnapshotBeforeUpdate(B,n),a.__reactInternalSnapshotBeforeUpdate=t}catch(X){st(e,e.return,X)}}break;case 3:if((t&1024)!==0){if(t=l.stateNode.containerInfo,e=t.nodeType,e===9)uf(t);else if(e===1)switch(t.nodeName){case"HEAD":case"HTML":case"BODY":uf(t);break;default:t.textContent=""}}break;case 5:case 26:case 27:case 6:case 4:case 17:break;default:if((t&1024)!==0)throw Error(s(163))}if(t=l.sibling,t!==null){t.return=l.return,Yt=t;break}Yt=l.return}}function Sr(t,l,e){var a=e.flags;switch(e.tag){case 0:case 11:case 15:kl(t,e),a&4&&ou(5,e);break;case 1:if(kl(t,e),a&4)if(t=e.stateNode,l===null)try{t.componentDidMount()}catch(i){st(e,e.return,i)}else{var u=Ze(e.type,l.memoizedProps);l=l.memoizedState;try{t.componentDidUpdate(u,l,t.__reactInternalSnapshotBeforeUpdate)}catch(i){st(e,e.return,i)}}a&64&&dr(e),a&512&&ru(e,e.return);break;case 3:if(kl(t,e),a&64&&(t=e.updateQueue,t!==null)){if(l=null,e.child!==null)switch(e.child.tag){case 27:case 5:l=e.child.stateNode;break;case 1:l=e.child.stateNode}try{lo(t,l)}catch(i){st(e,e.return,i)}}break;case 27:l===null&&a&4&&hr(e);case 26:case 5:kl(t,e),l===null&&a&4&&yr(e),a&512&&ru(e,e.return);break;case 12:kl(t,e);break;case 31:kl(t,e),a&4&&Er(t,e);break;case 13:kl(t,e),a&4&&Ar(t,e),a&64&&(t=e.memoizedState,t!==null&&(t=t.dehydrated,t!==null&&(e=nv.bind(null,e),Ov(t,e))));break;case 22:if(a=e.memoizedState!==null||$l,!a){l=l!==null&&l.memoizedState!==null||Rt,u=$l;var n=Rt;$l=a,(Rt=l)&&!n?Fl(t,e,(e.subtreeFlags&8772)!==0):kl(t,e),$l=u,Rt=n}break;case 30:break;default:kl(t,e)}}function br(t){var l=t.alternate;l!==null&&(t.alternate=null,br(l)),t.child=null,t.deletions=null,t.sibling=null,t.tag===5&&(l=t.stateNode,l!==null&&si(l)),t.stateNode=null,t.return=null,t.dependencies=null,t.memoizedProps=null,t.memoizedState=null,t.pendingProps=null,t.stateNode=null,t.updateQueue=null}var Et=null,It=!1;function Wl(t,l,e){for(e=e.child;e!==null;)Tr(t,l,e),e=e.sibling}function Tr(t,l,e){if(il&&typeof il.onCommitFiberUnmount=="function")try{il.onCommitFiberUnmount(Ba,e)}catch{}switch(e.tag){case 26:Rt||Bl(e,l),Wl(t,l,e),e.memoizedState?e.memoizedState.count--:e.stateNode&&(e=e.stateNode,e.parentNode.removeChild(e));break;case 27:Rt||Bl(e,l);var a=Et,u=It;Te(e.type)&&(Et=e.stateNode,It=!1),Wl(t,l,e),Tu(e.stateNode),Et=a,It=u;break;case 5:Rt||Bl(e,l);case 6:if(a=Et,u=It,Et=null,Wl(t,l,e),Et=a,It=u,Et!==null)if(It)try{(Et.nodeType===9?Et.body:Et.nodeName==="HTML"?Et.ownerDocument.body:Et).removeChild(e.stateNode)}catch(n){st(e,l,n)}else try{Et.removeChild(e.stateNode)}catch(n){st(e,l,n)}break;case 18:Et!==null&&(It?(t=Et,dd(t.nodeType===9?t.body:t.nodeName==="HTML"?t.ownerDocument.body:t,e.stateNode),Ua(t)):dd(Et,e.stateNode));break;case 4:a=Et,u=It,Et=e.stateNode.containerInfo,It=!0,Wl(t,l,e),Et=a,It=u;break;case 0:case 11:case 14:case 15:me(2,e,l),Rt||me(4,e,l),Wl(t,l,e);break;case 1:Rt||(Bl(e,l),a=e.stateNode,typeof a.componentWillUnmount=="function"&&mr(e,l,a)),Wl(t,l,e);break;case 21:Wl(t,l,e);break;case 22:Rt=(a=Rt)||e.memoizedState!==null,Wl(t,l,e),Rt=a;break;default:Wl(t,l,e)}}function Er(t,l){if(l.memoizedState===null&&(t=l.alternate,t!==null&&(t=t.memoizedState,t!==null))){t=t.dehydrated;try{Ua(t)}catch(e){st(l,l.return,e)}}}function Ar(t,l){if(l.memoizedState===null&&(t=l.alternate,t!==null&&(t=t.memoizedState,t!==null&&(t=t.dehydrated,t!==null))))try{Ua(t)}catch(e){st(l,l.return,e)}}function Fy(t){switch(t.tag){case 31:case 13:case 19:var l=t.stateNode;return l===null&&(l=t.stateNode=new gr),l;case 22:return t=t.stateNode,l=t._retryCache,l===null&&(l=t._retryCache=new gr),l;default:throw Error(s(435,t.tag))}}function pn(t,l){var e=Fy(t);l.forEach(function(a){if(!e.has(a)){e.add(a);var u=iv.bind(null,t,a);a.then(u,u)}})}function Pt(t,l){var e=l.deletions;if(e!==null)for(var a=0;a<e.length;a++){var u=e[a],n=t,i=l,c=i;t:for(;c!==null;){switch(c.tag){case 27:if(Te(c.type)){Et=c.stateNode,It=!1;break t}break;case 5:Et=c.stateNode,It=!1;break t;case 3:case 4:Et=c.stateNode.containerInfo,It=!0;break t}c=c.return}if(Et===null)throw Error(s(160));Tr(n,i,u),Et=null,It=!1,n=u.alternate,n!==null&&(n.return=null),u.return=null}if(l.subtreeFlags&13886)for(l=l.child;l!==null;)pr(l,t),l=l.sibling}var Hl=null;function pr(t,l){var e=t.alternate,a=t.flags;switch(t.tag){case 0:case 11:case 14:case 15:Pt(l,t),tl(t),a&4&&(me(3,t,t.return),ou(3,t),me(5,t,t.return));break;case 1:Pt(l,t),tl(t),a&512&&(Rt||e===null||Bl(e,e.return)),a&64&&$l&&(t=t.updateQueue,t!==null&&(a=t.callbacks,a!==null&&(e=t.shared.hiddenCallbacks,t.shared.hiddenCallbacks=e===null?a:e.concat(a))));break;case 26:var u=Hl;if(Pt(l,t),tl(t),a&512&&(Rt||e===null||Bl(e,e.return)),a&4){var n=e!==null?e.memoizedState:null;if(a=t.memoizedState,e===null)if(a===null)if(t.stateNode===null){t:{a=t.type,e=t.memoizedProps,u=u.ownerDocument||u;l:switch(a){case"title":n=u.getElementsByTagName("title")[0],(!n||n[Ga]||n[xt]||n.namespaceURI==="http://www.w3.org/2000/svg"||n.hasAttribute("itemprop"))&&(n=u.createElement(a),u.head.insertBefore(n,u.querySelector("head > title"))),Vt(n,a,e),n[xt]=t,jt(n),a=n;break t;case"link":var i=pd("link","href",u).get(a+(e.href||""));if(i){for(var c=0;c<i.length;c++)if(n=i[c],n.getAttribute("href")===(e.href==null||e.href===""?null:e.href)&&n.getAttribute("rel")===(e.rel==null?null:e.rel)&&n.getAttribute("title")===(e.title==null?null:e.title)&&n.getAttribute("crossorigin")===(e.crossOrigin==null?null:e.crossOrigin)){i.splice(c,1);break l}}n=u.createElement(a),Vt(n,a,e),u.head.appendChild(n);break;case"meta":if(i=pd("meta","content",u).get(a+(e.content||""))){for(c=0;c<i.length;c++)if(n=i[c],n.getAttribute("content")===(e.content==null?null:""+e.content)&&n.getAttribute("name")===(e.name==null?null:e.name)&&n.getAttribute("property")===(e.property==null?null:e.property)&&n.getAttribute("http-equiv")===(e.httpEquiv==null?null:e.httpEquiv)&&n.getAttribute("charset")===(e.charSet==null?null:e.charSet)){i.splice(c,1);break l}}n=u.createElement(a),Vt(n,a,e),u.head.appendChild(n);break;default:throw Error(s(468,a))}n[xt]=t,jt(n),a=n}t.stateNode=a}else zd(u,t.type,t.stateNode);else t.stateNode=Ad(u,a,t.memoizedProps);else n!==a?(n===null?e.stateNode!==null&&(e=e.stateNode,e.parentNode.removeChild(e)):n.count--,a===null?zd(u,t.type,t.stateNode):Ad(u,a,t.memoizedProps)):a===null&&t.stateNode!==null&&Hc(t,t.memoizedProps,e.memoizedProps)}break;case 27:Pt(l,t),tl(t),a&512&&(Rt||e===null||Bl(e,e.return)),e!==null&&a&4&&Hc(t,t.memoizedProps,e.memoizedProps);break;case 5:if(Pt(l,t),tl(t),a&512&&(Rt||e===null||Bl(e,e.return)),t.flags&32){u=t.stateNode;try{Ie(u,"")}catch(B){st(t,t.return,B)}}a&4&&t.stateNode!=null&&(u=t.memoizedProps,Hc(t,u,e!==null?e.memoizedProps:u)),a&1024&&(qc=!0);break;case 6:if(Pt(l,t),tl(t),a&4){if(t.stateNode===null)throw Error(s(162));a=t.memoizedProps,e=t.stateNode;try{e.nodeValue=a}catch(B){st(t,t.return,B)}}break;case 3:if(Xn=null,u=Hl,Hl=Yn(l.containerInfo),Pt(l,t),Hl=u,tl(t),a&4&&e!==null&&e.memoizedState.isDehydrated)try{Ua(l.containerInfo)}catch(B){st(t,t.return,B)}qc&&(qc=!1,zr(t));break;case 4:a=Hl,Hl=Yn(t.stateNode.containerInfo),Pt(l,t),tl(t),Hl=a;break;case 12:Pt(l,t),tl(t);break;case 31:Pt(l,t),tl(t),a&4&&(a=t.updateQueue,a!==null&&(t.updateQueue=null,pn(t,a)));break;case 13:Pt(l,t),tl(t),t.child.flags&8192&&t.memoizedState!==null!=(e!==null&&e.memoizedState!==null)&&(On=nl()),a&4&&(a=t.updateQueue,a!==null&&(t.updateQueue=null,pn(t,a)));break;case 22:u=t.memoizedState!==null;var o=e!==null&&e.memoizedState!==null,S=$l,z=Rt;if($l=S||u,Rt=z||o,Pt(l,t),Rt=z,$l=S,tl(t),a&8192)t:for(l=t.stateNode,l._visibility=u?l._visibility&-2:l._visibility|1,u&&(e===null||o||$l||Rt||Ve(t)),e=null,l=t;;){if(l.tag===5||l.tag===26){if(e===null){o=e=l;try{if(n=o.stateNode,u)i=n.style,typeof i.setProperty=="function"?i.setProperty("display","none","important"):i.display="none";else{c=o.stateNode;var M=o.memoizedProps.style,b=M!=null&&M.hasOwnProperty("display")?M.display:null;c.style.display=b==null||typeof b=="boolean"?"":(""+b).trim()}}catch(B){st(o,o.return,B)}}}else if(l.tag===6){if(e===null){o=l;try{o.stateNode.nodeValue=u?"":o.memoizedProps}catch(B){st(o,o.return,B)}}}else if(l.tag===18){if(e===null){o=l;try{var E=o.stateNode;u?md(E,!0):md(o.stateNode,!1)}catch(B){st(o,o.return,B)}}}else if((l.tag!==22&&l.tag!==23||l.memoizedState===null||l===t)&&l.child!==null){l.child.return=l,l=l.child;continue}if(l===t)break t;for(;l.sibling===null;){if(l.return===null||l.return===t)break t;e===l&&(e=null),l=l.return}e===l&&(e=null),l.sibling.return=l.return,l=l.sibling}a&4&&(a=t.updateQueue,a!==null&&(e=a.retryQueue,e!==null&&(a.retryQueue=null,pn(t,e))));break;case 19:Pt(l,t),tl(t),a&4&&(a=t.updateQueue,a!==null&&(t.updateQueue=null,pn(t,a)));break;case 30:break;case 21:break;default:Pt(l,t),tl(t)}}function tl(t){var l=t.flags;if(l&2){try{for(var e,a=t.return;a!==null;){if(vr(a)){e=a;break}a=a.return}if(e==null)throw Error(s(160));switch(e.tag){case 27:var u=e.stateNode,n=Cc(t);An(t,n,u);break;case 5:var i=e.stateNode;e.flags&32&&(Ie(i,""),e.flags&=-33);var c=Cc(t);An(t,c,i);break;case 3:case 4:var o=e.stateNode.containerInfo,S=Cc(t);Rc(t,S,o);break;default:throw Error(s(161))}}catch(z){st(t,t.return,z)}t.flags&=-3}l&4096&&(t.flags&=-4097)}function zr(t){if(t.subtreeFlags&1024)for(t=t.child;t!==null;){var l=t;zr(l),l.tag===5&&l.flags&1024&&l.stateNode.reset(),t=t.sibling}}function kl(t,l){if(l.subtreeFlags&8772)for(l=l.child;l!==null;)Sr(t,l.alternate,l),l=l.sibling}function Ve(t){for(t=t.child;t!==null;){var l=t;switch(l.tag){case 0:case 11:case 14:case 15:me(4,l,l.return),Ve(l);break;case 1:Bl(l,l.return);var e=l.stateNode;typeof e.componentWillUnmount=="function"&&mr(l,l.return,e),Ve(l);break;case 27:Tu(l.stateNode);case 26:case 5:Bl(l,l.return),Ve(l);break;case 22:l.memoizedState===null&&Ve(l);break;case 30:Ve(l);break;default:Ve(l)}t=t.sibling}}function Fl(t,l,e){for(e=e&&(l.subtreeFlags&8772)!==0,l=l.child;l!==null;){var a=l.alternate,u=t,n=l,i=n.flags;switch(n.tag){case 0:case 11:case 15:Fl(u,n,e),ou(4,n);break;case 1:if(Fl(u,n,e),a=n,u=a.stateNode,typeof u.componentDidMount=="function")try{u.componentDidMount()}catch(S){st(a,a.return,S)}if(a=n,u=a.updateQueue,u!==null){var c=a.stateNode;try{var o=u.shared.hiddenCallbacks;if(o!==null)for(u.shared.hiddenCallbacks=null,u=0;u<o.length;u++)to(o[u],c)}catch(S){st(a,a.return,S)}}e&&i&64&&dr(n),ru(n,n.return);break;case 27:hr(n);case 26:case 5:Fl(u,n,e),e&&a===null&&i&4&&yr(n),ru(n,n.return);break;case 12:Fl(u,n,e);break;case 31:Fl(u,n,e),e&&i&4&&Er(u,n);break;case 13:Fl(u,n,e),e&&i&4&&Ar(u,n);break;case 22:n.memoizedState===null&&Fl(u,n,e),ru(n,n.return);break;case 30:break;default:Fl(u,n,e)}l=l.sibling}}function Nc(t,l){var e=null;t!==null&&t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(e=t.memoizedState.cachePool.pool),t=null,l.memoizedState!==null&&l.memoizedState.cachePool!==null&&(t=l.memoizedState.cachePool.pool),t!==e&&(t!=null&&t.refCount++,e!=null&&Fa(e))}function Bc(t,l){t=null,l.alternate!==null&&(t=l.alternate.memoizedState.cache),l=l.memoizedState.cache,l!==t&&(l.refCount++,t!=null&&Fa(t))}function Cl(t,l,e,a){if(l.subtreeFlags&10256)for(l=l.child;l!==null;)Or(t,l,e,a),l=l.sibling}function Or(t,l,e,a){var u=l.flags;switch(l.tag){case 0:case 11:case 15:Cl(t,l,e,a),u&2048&&ou(9,l);break;case 1:Cl(t,l,e,a);break;case 3:Cl(t,l,e,a),u&2048&&(t=null,l.alternate!==null&&(t=l.alternate.memoizedState.cache),l=l.memoizedState.cache,l!==t&&(l.refCount++,t!=null&&Fa(t)));break;case 12:if(u&2048){Cl(t,l,e,a),t=l.stateNode;try{var n=l.memoizedProps,i=n.id,c=n.onPostCommit;typeof c=="function"&&c(i,l.alternate===null?"mount":"update",t.passiveEffectDuration,-0)}catch(o){st(l,l.return,o)}}else Cl(t,l,e,a);break;case 31:Cl(t,l,e,a);break;case 13:Cl(t,l,e,a);break;case 23:break;case 22:n=l.stateNode,i=l.alternate,l.memoizedState!==null?n._visibility&2?Cl(t,l,e,a):du(t,l):n._visibility&2?Cl(t,l,e,a):(n._visibility|=2,Sa(t,l,e,a,(l.subtreeFlags&10256)!==0||!1)),u&2048&&Nc(i,l);break;case 24:Cl(t,l,e,a),u&2048&&Bc(l.alternate,l);break;default:Cl(t,l,e,a)}}function Sa(t,l,e,a,u){for(u=u&&((l.subtreeFlags&10256)!==0||!1),l=l.child;l!==null;){var n=t,i=l,c=e,o=a,S=i.flags;switch(i.tag){case 0:case 11:case 15:Sa(n,i,c,o,u),ou(8,i);break;case 23:break;case 22:var z=i.stateNode;i.memoizedState!==null?z._visibility&2?Sa(n,i,c,o,u):du(n,i):(z._visibility|=2,Sa(n,i,c,o,u)),u&&S&2048&&Nc(i.alternate,i);break;case 24:Sa(n,i,c,o,u),u&&S&2048&&Bc(i.alternate,i);break;default:Sa(n,i,c,o,u)}l=l.sibling}}function du(t,l){if(l.subtreeFlags&10256)for(l=l.child;l!==null;){var e=t,a=l,u=a.flags;switch(a.tag){case 22:du(e,a),u&2048&&Nc(a.alternate,a);break;case 24:du(e,a),u&2048&&Bc(a.alternate,a);break;default:du(e,a)}l=l.sibling}}var mu=8192;function ba(t,l,e){if(t.subtreeFlags&mu)for(t=t.child;t!==null;)_r(t,l,e),t=t.sibling}function _r(t,l,e){switch(t.tag){case 26:ba(t,l,e),t.flags&mu&&t.memoizedState!==null&&Yv(e,Hl,t.memoizedState,t.memoizedProps);break;case 5:ba(t,l,e);break;case 3:case 4:var a=Hl;Hl=Yn(t.stateNode.containerInfo),ba(t,l,e),Hl=a;break;case 22:t.memoizedState===null&&(a=t.alternate,a!==null&&a.memoizedState!==null?(a=mu,mu=16777216,ba(t,l,e),mu=a):ba(t,l,e));break;default:ba(t,l,e)}}function Mr(t){var l=t.alternate;if(l!==null&&(t=l.child,t!==null)){l.child=null;do l=t.sibling,t.sibling=null,t=l;while(t!==null)}}function yu(t){var l=t.deletions;if((t.flags&16)!==0){if(l!==null)for(var e=0;e<l.length;e++){var a=l[e];Yt=a,Ur(a,t)}Mr(t)}if(t.subtreeFlags&10256)for(t=t.child;t!==null;)Dr(t),t=t.sibling}function Dr(t){switch(t.tag){case 0:case 11:case 15:yu(t),t.flags&2048&&me(9,t,t.return);break;case 3:yu(t);break;case 12:yu(t);break;case 22:var l=t.stateNode;t.memoizedState!==null&&l._visibility&2&&(t.return===null||t.return.tag!==13)?(l._visibility&=-3,zn(t)):yu(t);break;default:yu(t)}}function zn(t){var l=t.deletions;if((t.flags&16)!==0){if(l!==null)for(var e=0;e<l.length;e++){var a=l[e];Yt=a,Ur(a,t)}Mr(t)}for(t=t.child;t!==null;){switch(l=t,l.tag){case 0:case 11:case 15:me(8,l,l.return),zn(l);break;case 22:e=l.stateNode,e._visibility&2&&(e._visibility&=-3,zn(l));break;default:zn(l)}t=t.sibling}}function Ur(t,l){for(;Yt!==null;){var e=Yt;switch(e.tag){case 0:case 11:case 15:me(8,e,l);break;case 23:case 22:if(e.memoizedState!==null&&e.memoizedState.cachePool!==null){var a=e.memoizedState.cachePool.pool;a!=null&&a.refCount++}break;case 24:Fa(e.memoizedState.cache)}if(a=e.child,a!==null)a.return=e,Yt=a;else t:for(e=t;Yt!==null;){a=Yt;var u=a.sibling,n=a.return;if(br(a),a===e){Yt=null;break t}if(u!==null){u.return=n,Yt=u;break t}Yt=n}}}var Iy={getCacheForType:function(t){var l=Lt(Ut),e=l.data.get(t);return e===void 0&&(e=t(),l.data.set(t,e)),e},cacheSignal:function(){return Lt(Ut).controller.signal}},Py=typeof WeakMap=="function"?WeakMap:Map,nt=0,gt=null,k=null,I=0,ft=0,dl=null,ye=!1,Ta=!1,jc=!1,Il=0,Ot=0,ve=0,Ke=0,Yc=0,ml=0,Ea=0,vu=null,ll=null,Gc=!1,On=0,Hr=0,_n=1/0,Mn=null,he=null,Bt=0,ge=null,Aa=null,Pl=0,Xc=0,xc=null,Cr=null,hu=0,Qc=null;function yl(){return(nt&2)!==0&&I!==0?I&-I:O.T!==null?wc():Jf()}function Rr(){if(ml===0)if((I&536870912)===0||lt){var t=Nu;Nu<<=1,(Nu&3932160)===0&&(Nu=262144),ml=t}else ml=536870912;return t=ol.current,t!==null&&(t.flags|=32),ml}function el(t,l,e){(t===gt&&(ft===2||ft===9)||t.cancelPendingCommit!==null)&&(pa(t,0),Se(t,I,ml,!1)),Ya(t,e),((nt&2)===0||t!==gt)&&(t===gt&&((nt&2)===0&&(Ke|=e),Ot===4&&Se(t,I,ml,!1)),jl(t))}function qr(t,l,e){if((nt&6)!==0)throw Error(s(327));var a=!e&&(l&127)===0&&(l&t.expiredLanes)===0||ja(t,l),u=a?ev(t,l):Zc(t,l,!0),n=a;do{if(u===0){Ta&&!a&&Se(t,l,0,!1);break}else{if(e=t.current.alternate,n&&!tv(e)){u=Zc(t,l,!1),n=!1;continue}if(u===2){if(n=l,t.errorRecoveryDisabledLanes&n)var i=0;else i=t.pendingLanes&-536870913,i=i!==0?i:i&536870912?536870912:0;if(i!==0){l=i;t:{var c=t;u=vu;var o=c.current.memoizedState.isDehydrated;if(o&&(pa(c,i).flags|=256),i=Zc(c,i,!1),i!==2){if(jc&&!o){c.errorRecoveryDisabledLanes|=n,Ke|=n,u=4;break t}n=ll,ll=u,n!==null&&(ll===null?ll=n:ll.push.apply(ll,n))}u=i}if(n=!1,u!==2)continue}}if(u===1){pa(t,0),Se(t,l,0,!0);break}t:{switch(a=t,n=u,n){case 0:case 1:throw Error(s(345));case 4:if((l&4194048)!==l)break;case 6:Se(a,l,ml,!ye);break t;case 2:ll=null;break;case 3:case 5:break;default:throw Error(s(329))}if((l&62914560)===l&&(u=On+300-nl(),10<u)){if(Se(a,l,ml,!ye),ju(a,0,!0)!==0)break t;Pl=l,a.timeoutHandle=od(Nr.bind(null,a,e,ll,Mn,Gc,l,ml,Ke,Ea,ye,n,"Throttled",-0,0),u);break t}Nr(a,e,ll,Mn,Gc,l,ml,Ke,Ea,ye,n,null,-0,0)}}break}while(!0);jl(t)}function Nr(t,l,e,a,u,n,i,c,o,S,z,M,b,E){if(t.timeoutHandle=-1,M=l.subtreeFlags,M&8192||(M&16785408)===16785408){M={stylesheets:null,count:0,imgCount:0,imgBytes:0,suspenseyImages:[],waitingForImages:!0,waitingForViewTransition:!1,unsuspend:Gl},_r(l,n,M);var B=(n&62914560)===n?On-nl():(n&4194048)===n?Hr-nl():0;if(B=Gv(M,B),B!==null){Pl=n,t.cancelPendingCommit=B(Lr.bind(null,t,l,n,e,a,u,i,c,o,z,M,null,b,E)),Se(t,n,i,!S);return}}Lr(t,l,n,e,a,u,i,c,o)}function tv(t){for(var l=t;;){var e=l.tag;if((e===0||e===11||e===15)&&l.flags&16384&&(e=l.updateQueue,e!==null&&(e=e.stores,e!==null)))for(var a=0;a<e.length;a++){var u=e[a],n=u.getSnapshot;u=u.value;try{if(!fl(n(),u))return!1}catch{return!1}}if(e=l.child,l.subtreeFlags&16384&&e!==null)e.return=l,l=e;else{if(l===t)break;for(;l.sibling===null;){if(l.return===null||l.return===t)return!0;l=l.return}l.sibling.return=l.return,l=l.sibling}}return!0}function Se(t,l,e,a){l&=~Yc,l&=~Ke,t.suspendedLanes|=l,t.pingedLanes&=~l,a&&(t.warmLanes|=l),a=t.expirationTimes;for(var u=l;0<u;){var n=31-cl(u),i=1<<n;a[n]=-1,u&=~i}e!==0&&Zf(t,e,l)}function Dn(){return(nt&6)===0?(gu(0),!1):!0}function Lc(){if(k!==null){if(ft===0)var t=k.return;else t=k,Ll=je=null,uc(t),ma=null,Pa=0,t=k;for(;t!==null;)rr(t.alternate,t),t=t.return;k=null}}function pa(t,l){var e=t.timeoutHandle;e!==-1&&(t.timeoutHandle=-1,Tv(e)),e=t.cancelPendingCommit,e!==null&&(t.cancelPendingCommit=null,e()),Pl=0,Lc(),gt=t,k=e=xl(t.current,null),I=l,ft=0,dl=null,ye=!1,Ta=ja(t,l),jc=!1,Ea=ml=Yc=Ke=ve=Ot=0,ll=vu=null,Gc=!1,(l&8)!==0&&(l|=l&32);var a=t.entangledLanes;if(a!==0)for(t=t.entanglements,a&=l;0<a;){var u=31-cl(a),n=1<<u;l|=t[u],a&=~n}return Il=l,$u(),e}function Br(t,l){V=null,O.H=cu,l===da||l===en?(l=ks(),ft=3):l===Ji?(l=ks(),ft=4):ft=l===Tc?8:l!==null&&typeof l=="object"&&typeof l.then=="function"?6:1,dl=l,k===null&&(Ot=1,gn(t,bl(l,t.current)))}function jr(){var t=ol.current;return t===null?!0:(I&4194048)===I?pl===null:(I&62914560)===I||(I&536870912)!==0?t===pl:!1}function Yr(){var t=O.H;return O.H=cu,t===null?cu:t}function Gr(){var t=O.A;return O.A=Iy,t}function Un(){Ot=4,ye||(I&4194048)!==I&&ol.current!==null||(Ta=!0),(ve&134217727)===0&&(Ke&134217727)===0||gt===null||Se(gt,I,ml,!1)}function Zc(t,l,e){var a=nt;nt|=2;var u=Yr(),n=Gr();(gt!==t||I!==l)&&(Mn=null,pa(t,l)),l=!1;var i=Ot;t:do try{if(ft!==0&&k!==null){var c=k,o=dl;switch(ft){case 8:Lc(),i=6;break t;case 3:case 2:case 9:case 6:ol.current===null&&(l=!0);var S=ft;if(ft=0,dl=null,za(t,c,o,S),e&&Ta){i=0;break t}break;default:S=ft,ft=0,dl=null,za(t,c,o,S)}}lv(),i=Ot;break}catch(z){Br(t,z)}while(!0);return l&&t.shellSuspendCounter++,Ll=je=null,nt=a,O.H=u,O.A=n,k===null&&(gt=null,I=0,$u()),i}function lv(){for(;k!==null;)Xr(k)}function ev(t,l){var e=nt;nt|=2;var a=Yr(),u=Gr();gt!==t||I!==l?(Mn=null,_n=nl()+500,pa(t,l)):Ta=ja(t,l);t:do try{if(ft!==0&&k!==null){l=k;var n=dl;l:switch(ft){case 1:ft=0,dl=null,za(t,l,n,1);break;case 2:case 9:if($s(n)){ft=0,dl=null,xr(l);break}l=function(){ft!==2&&ft!==9||gt!==t||(ft=7),jl(t)},n.then(l,l);break t;case 3:ft=7;break t;case 4:ft=5;break t;case 7:$s(n)?(ft=0,dl=null,xr(l)):(ft=0,dl=null,za(t,l,n,7));break;case 5:var i=null;switch(k.tag){case 26:i=k.memoizedState;case 5:case 27:var c=k;if(i?Od(i):c.stateNode.complete){ft=0,dl=null;var o=c.sibling;if(o!==null)k=o;else{var S=c.return;S!==null?(k=S,Hn(S)):k=null}break l}}ft=0,dl=null,za(t,l,n,5);break;case 6:ft=0,dl=null,za(t,l,n,6);break;case 8:Lc(),Ot=6;break t;default:throw Error(s(462))}}av();break}catch(z){Br(t,z)}while(!0);return Ll=je=null,O.H=a,O.A=u,nt=e,k!==null?0:(gt=null,I=0,$u(),Ot)}function av(){for(;k!==null&&!_m();)Xr(k)}function Xr(t){var l=sr(t.alternate,t,Il);t.memoizedProps=t.pendingProps,l===null?Hn(t):k=l}function xr(t){var l=t,e=l.alternate;switch(l.tag){case 15:case 0:l=ar(e,l,l.pendingProps,l.type,void 0,I);break;case 11:l=ar(e,l,l.pendingProps,l.type.render,l.ref,I);break;case 5:uc(l);default:rr(e,l),l=k=Ys(l,Il),l=sr(e,l,Il)}t.memoizedProps=t.pendingProps,l===null?Hn(t):k=l}function za(t,l,e,a){Ll=je=null,uc(l),ma=null,Pa=0;var u=l.return;try{if(Ky(t,u,l,e,I)){Ot=1,gn(t,bl(e,t.current)),k=null;return}}catch(n){if(u!==null)throw k=u,n;Ot=1,gn(t,bl(e,t.current)),k=null;return}l.flags&32768?(lt||a===1?t=!0:Ta||(I&536870912)!==0?t=!1:(ye=t=!0,(a===2||a===9||a===3||a===6)&&(a=ol.current,a!==null&&a.tag===13&&(a.flags|=16384))),Qr(l,t)):Hn(l)}function Hn(t){var l=t;do{if((l.flags&32768)!==0){Qr(l,ye);return}t=l.return;var e=$y(l.alternate,l,Il);if(e!==null){k=e;return}if(l=l.sibling,l!==null){k=l;return}k=l=t}while(l!==null);Ot===0&&(Ot=5)}function Qr(t,l){do{var e=Wy(t.alternate,t);if(e!==null){e.flags&=32767,k=e;return}if(e=t.return,e!==null&&(e.flags|=32768,e.subtreeFlags=0,e.deletions=null),!l&&(t=t.sibling,t!==null)){k=t;return}k=t=e}while(t!==null);Ot=6,k=null}function Lr(t,l,e,a,u,n,i,c,o){t.cancelPendingCommit=null;do Cn();while(Bt!==0);if((nt&6)!==0)throw Error(s(327));if(l!==null){if(l===t.current)throw Error(s(177));if(n=l.lanes|l.childLanes,n|=Ci,jm(t,e,n,i,c,o),t===gt&&(k=gt=null,I=0),Aa=l,ge=t,Pl=e,Xc=n,xc=u,Cr=a,(l.subtreeFlags&10256)!==0||(l.flags&10256)!==0?(t.callbackNode=null,t.callbackPriority=0,cv(Ru,function(){return wr(),null})):(t.callbackNode=null,t.callbackPriority=0),a=(l.flags&13878)!==0,(l.subtreeFlags&13878)!==0||a){a=O.T,O.T=null,u=C.p,C.p=2,i=nt,nt|=4;try{ky(t,l,e)}finally{nt=i,C.p=u,O.T=a}}Bt=1,Zr(),Vr(),Kr()}}function Zr(){if(Bt===1){Bt=0;var t=ge,l=Aa,e=(l.flags&13878)!==0;if((l.subtreeFlags&13878)!==0||e){e=O.T,O.T=null;var a=C.p;C.p=2;var u=nt;nt|=4;try{pr(l,t);var n=lf,i=Ds(t.containerInfo),c=n.focusedElem,o=n.selectionRange;if(i!==c&&c&&c.ownerDocument&&Ms(c.ownerDocument.documentElement,c)){if(o!==null&&_i(c)){var S=o.start,z=o.end;if(z===void 0&&(z=S),"selectionStart"in c)c.selectionStart=S,c.selectionEnd=Math.min(z,c.value.length);else{var M=c.ownerDocument||document,b=M&&M.defaultView||window;if(b.getSelection){var E=b.getSelection(),B=c.textContent.length,X=Math.min(o.start,B),mt=o.end===void 0?X:Math.min(o.end,B);!E.extend&&X>mt&&(i=mt,mt=X,X=i);var h=_s(c,X),d=_s(c,mt);if(h&&d&&(E.rangeCount!==1||E.anchorNode!==h.node||E.anchorOffset!==h.offset||E.focusNode!==d.node||E.focusOffset!==d.offset)){var g=M.createRange();g.setStart(h.node,h.offset),E.removeAllRanges(),X>mt?(E.addRange(g),E.extend(d.node,d.offset)):(g.setEnd(d.node,d.offset),E.addRange(g))}}}}for(M=[],E=c;E=E.parentNode;)E.nodeType===1&&M.push({element:E,left:E.scrollLeft,top:E.scrollTop});for(typeof c.focus=="function"&&c.focus(),c=0;c<M.length;c++){var _=M[c];_.element.scrollLeft=_.left,_.element.scrollTop=_.top}}Zn=!!tf,lf=tf=null}finally{nt=u,C.p=a,O.T=e}}t.current=l,Bt=2}}function Vr(){if(Bt===2){Bt=0;var t=ge,l=Aa,e=(l.flags&8772)!==0;if((l.subtreeFlags&8772)!==0||e){e=O.T,O.T=null;var a=C.p;C.p=2;var u=nt;nt|=4;try{Sr(t,l.alternate,l)}finally{nt=u,C.p=a,O.T=e}}Bt=3}}function Kr(){if(Bt===4||Bt===3){Bt=0,Mm();var t=ge,l=Aa,e=Pl,a=Cr;(l.subtreeFlags&10256)!==0||(l.flags&10256)!==0?Bt=5:(Bt=0,Aa=ge=null,Jr(t,t.pendingLanes));var u=t.pendingLanes;if(u===0&&(he=null),ci(e),l=l.stateNode,il&&typeof il.onCommitFiberRoot=="function")try{il.onCommitFiberRoot(Ba,l,void 0,(l.current.flags&128)===128)}catch{}if(a!==null){l=O.T,u=C.p,C.p=2,O.T=null;try{for(var n=t.onRecoverableError,i=0;i<a.length;i++){var c=a[i];n(c.value,{componentStack:c.stack})}}finally{O.T=l,C.p=u}}(Pl&3)!==0&&Cn(),jl(t),u=t.pendingLanes,(e&261930)!==0&&(u&42)!==0?t===Qc?hu++:(hu=0,Qc=t):hu=0,gu(0)}}function Jr(t,l){(t.pooledCacheLanes&=l)===0&&(l=t.pooledCache,l!=null&&(t.pooledCache=null,Fa(l)))}function Cn(){return Zr(),Vr(),Kr(),wr()}function wr(){if(Bt!==5)return!1;var t=ge,l=Xc;Xc=0;var e=ci(Pl),a=O.T,u=C.p;try{C.p=32>e?32:e,O.T=null,e=xc,xc=null;var n=ge,i=Pl;if(Bt=0,Aa=ge=null,Pl=0,(nt&6)!==0)throw Error(s(331));var c=nt;if(nt|=4,Dr(n.current),Or(n,n.current,i,e),nt=c,gu(0,!1),il&&typeof il.onPostCommitFiberRoot=="function")try{il.onPostCommitFiberRoot(Ba,n)}catch{}return!0}finally{C.p=u,O.T=a,Jr(t,l)}}function $r(t,l,e){l=bl(e,l),l=bc(t.stateNode,l,2),t=oe(t,l,2),t!==null&&(Ya(t,2),jl(t))}function st(t,l,e){if(t.tag===3)$r(t,t,e);else for(;l!==null;){if(l.tag===3){$r(l,t,e);break}else if(l.tag===1){var a=l.stateNode;if(typeof l.type.getDerivedStateFromError=="function"||typeof a.componentDidCatch=="function"&&(he===null||!he.has(a))){t=bl(e,t),e=Wo(2),a=oe(l,e,2),a!==null&&(ko(e,a,l,t),Ya(a,2),jl(a));break}}l=l.return}}function Vc(t,l,e){var a=t.pingCache;if(a===null){a=t.pingCache=new Py;var u=new Set;a.set(l,u)}else u=a.get(l),u===void 0&&(u=new Set,a.set(l,u));u.has(e)||(jc=!0,u.add(e),t=uv.bind(null,t,l,e),l.then(t,t))}function uv(t,l,e){var a=t.pingCache;a!==null&&a.delete(l),t.pingedLanes|=t.suspendedLanes&e,t.warmLanes&=~e,gt===t&&(I&e)===e&&(Ot===4||Ot===3&&(I&62914560)===I&&300>nl()-On?(nt&2)===0&&pa(t,0):Yc|=e,Ea===I&&(Ea=0)),jl(t)}function Wr(t,l){l===0&&(l=Lf()),t=qe(t,l),t!==null&&(Ya(t,l),jl(t))}function nv(t){var l=t.memoizedState,e=0;l!==null&&(e=l.retryLane),Wr(t,e)}function iv(t,l){var e=0;switch(t.tag){case 31:case 13:var a=t.stateNode,u=t.memoizedState;u!==null&&(e=u.retryLane);break;case 19:a=t.stateNode;break;case 22:a=t.stateNode._retryCache;break;default:throw Error(s(314))}a!==null&&a.delete(l),Wr(t,e)}function cv(t,l){return ai(t,l)}var Rn=null,Oa=null,Kc=!1,qn=!1,Jc=!1,be=0;function jl(t){t!==Oa&&t.next===null&&(Oa===null?Rn=Oa=t:Oa=Oa.next=t),qn=!0,Kc||(Kc=!0,sv())}function gu(t,l){if(!Jc&&qn){Jc=!0;do for(var e=!1,a=Rn;a!==null;){if(t!==0){var u=a.pendingLanes;if(u===0)var n=0;else{var i=a.suspendedLanes,c=a.pingedLanes;n=(1<<31-cl(42|t)+1)-1,n&=u&~(i&~c),n=n&201326741?n&201326741|1:n?n|2:0}n!==0&&(e=!0,Pr(a,n))}else n=I,n=ju(a,a===gt?n:0,a.cancelPendingCommit!==null||a.timeoutHandle!==-1),(n&3)===0||ja(a,n)||(e=!0,Pr(a,n));a=a.next}while(e);Jc=!1}}function fv(){kr()}function kr(){qn=Kc=!1;var t=0;be!==0&&bv()&&(t=be);for(var l=nl(),e=null,a=Rn;a!==null;){var u=a.next,n=Fr(a,l);n===0?(a.next=null,e===null?Rn=u:e.next=u,u===null&&(Oa=e)):(e=a,(t!==0||(n&3)!==0)&&(qn=!0)),a=u}Bt!==0&&Bt!==5||gu(t),be!==0&&(be=0)}function Fr(t,l){for(var e=t.suspendedLanes,a=t.pingedLanes,u=t.expirationTimes,n=t.pendingLanes&-62914561;0<n;){var i=31-cl(n),c=1<<i,o=u[i];o===-1?((c&e)===0||(c&a)!==0)&&(u[i]=Bm(c,l)):o<=l&&(t.expiredLanes|=c),n&=~c}if(l=gt,e=I,e=ju(t,t===l?e:0,t.cancelPendingCommit!==null||t.timeoutHandle!==-1),a=t.callbackNode,e===0||t===l&&(ft===2||ft===9)||t.cancelPendingCommit!==null)return a!==null&&a!==null&&ui(a),t.callbackNode=null,t.callbackPriority=0;if((e&3)===0||ja(t,e)){if(l=e&-e,l===t.callbackPriority)return l;switch(a!==null&&ui(a),ci(e)){case 2:case 8:e=xf;break;case 32:e=Ru;break;case 268435456:e=Qf;break;default:e=Ru}return a=Ir.bind(null,t),e=ai(e,a),t.callbackPriority=l,t.callbackNode=e,l}return a!==null&&a!==null&&ui(a),t.callbackPriority=2,t.callbackNode=null,2}function Ir(t,l){if(Bt!==0&&Bt!==5)return t.callbackNode=null,t.callbackPriority=0,null;var e=t.callbackNode;if(Cn()&&t.callbackNode!==e)return null;var a=I;return a=ju(t,t===gt?a:0,t.cancelPendingCommit!==null||t.timeoutHandle!==-1),a===0?null:(qr(t,a,l),Fr(t,nl()),t.callbackNode!=null&&t.callbackNode===e?Ir.bind(null,t):null)}function Pr(t,l){if(Cn())return null;qr(t,l,!0)}function sv(){Ev(function(){(nt&6)!==0?ai(Xf,fv):kr()})}function wc(){if(be===0){var t=oa;t===0&&(t=qu,qu<<=1,(qu&261888)===0&&(qu=256)),be=t}return be}function td(t){return t==null||typeof t=="symbol"||typeof t=="boolean"?null:typeof t=="function"?t:xu(""+t)}function ld(t,l){var e=l.ownerDocument.createElement("input");return e.name=l.name,e.value=l.value,t.id&&e.setAttribute("form",t.id),l.parentNode.insertBefore(e,l),t=new FormData(t),e.parentNode.removeChild(e),t}function ov(t,l,e,a,u){if(l==="submit"&&e&&e.stateNode===u){var n=td((u[kt]||null).action),i=a.submitter;i&&(l=(l=i[kt]||null)?td(l.formAction):i.getAttribute("formAction"),l!==null&&(n=l,i=null));var c=new Vu("action","action",null,a,u);t.push({event:c,listeners:[{instance:null,listener:function(){if(a.defaultPrevented){if(be!==0){var o=i?ld(u,i):new FormData(u);mc(e,{pending:!0,data:o,method:u.method,action:n},null,o)}}else typeof n=="function"&&(c.preventDefault(),o=i?ld(u,i):new FormData(u),mc(e,{pending:!0,data:o,method:u.method,action:n},n,o))},currentTarget:u}]})}}for(var $c=0;$c<Hi.length;$c++){var Wc=Hi[$c],rv=Wc.toLowerCase(),dv=Wc[0].toUpperCase()+Wc.slice(1);Ul(rv,"on"+dv)}Ul(Cs,"onAnimationEnd"),Ul(Rs,"onAnimationIteration"),Ul(qs,"onAnimationStart"),Ul("dblclick","onDoubleClick"),Ul("focusin","onFocus"),Ul("focusout","onBlur"),Ul(Dy,"onTransitionRun"),Ul(Uy,"onTransitionStart"),Ul(Hy,"onTransitionCancel"),Ul(Ns,"onTransitionEnd"),ke("onMouseEnter",["mouseout","mouseover"]),ke("onMouseLeave",["mouseout","mouseover"]),ke("onPointerEnter",["pointerout","pointerover"]),ke("onPointerLeave",["pointerout","pointerover"]),Ue("onChange","change click focusin focusout input keydown keyup selectionchange".split(" ")),Ue("onSelect","focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" ")),Ue("onBeforeInput",["compositionend","keypress","textInput","paste"]),Ue("onCompositionEnd","compositionend focusout keydown keypress keyup mousedown".split(" ")),Ue("onCompositionStart","compositionstart focusout keydown keypress keyup mousedown".split(" ")),Ue("onCompositionUpdate","compositionupdate focusout keydown keypress keyup mousedown".split(" "));var Su="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange resize seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),mv=new Set("beforetoggle cancel close invalid load scroll scrollend toggle".split(" ").concat(Su));function ed(t,l){l=(l&4)!==0;for(var e=0;e<t.length;e++){var a=t[e],u=a.event;a=a.listeners;t:{var n=void 0;if(l)for(var i=a.length-1;0<=i;i--){var c=a[i],o=c.instance,S=c.currentTarget;if(c=c.listener,o!==n&&u.isPropagationStopped())break t;n=c,u.currentTarget=S;try{n(u)}catch(z){wu(z)}u.currentTarget=null,n=o}else for(i=0;i<a.length;i++){if(c=a[i],o=c.instance,S=c.currentTarget,c=c.listener,o!==n&&u.isPropagationStopped())break t;n=c,u.currentTarget=S;try{n(u)}catch(z){wu(z)}u.currentTarget=null,n=o}}}}function F(t,l){var e=l[fi];e===void 0&&(e=l[fi]=new Set);var a=t+"__bubble";e.has(a)||(ad(l,t,2,!1),e.add(a))}function kc(t,l,e){var a=0;l&&(a|=4),ad(e,t,a,l)}var Nn="_reactListening"+Math.random().toString(36).slice(2);function Fc(t){if(!t[Nn]){t[Nn]=!0,Wf.forEach(function(e){e!=="selectionchange"&&(mv.has(e)||kc(e,!1,t),kc(e,!0,t))});var l=t.nodeType===9?t:t.ownerDocument;l===null||l[Nn]||(l[Nn]=!0,kc("selectionchange",!1,l))}}function ad(t,l,e,a){switch(Rd(l)){case 2:var u=Qv;break;case 8:u=Lv;break;default:u=mf}e=u.bind(null,l,e,t),u=void 0,!gi||l!=="touchstart"&&l!=="touchmove"&&l!=="wheel"||(u=!0),a?u!==void 0?t.addEventListener(l,e,{capture:!0,passive:u}):t.addEventListener(l,e,!0):u!==void 0?t.addEventListener(l,e,{passive:u}):t.addEventListener(l,e,!1)}function Ic(t,l,e,a,u){var n=a;if((l&1)===0&&(l&2)===0&&a!==null)t:for(;;){if(a===null)return;var i=a.tag;if(i===3||i===4){var c=a.stateNode.containerInfo;if(c===u)break;if(i===4)for(i=a.return;i!==null;){var o=i.tag;if((o===3||o===4)&&i.stateNode.containerInfo===u)return;i=i.return}for(;c!==null;){if(i=we(c),i===null)return;if(o=i.tag,o===5||o===6||o===26||o===27){a=n=i;continue t}c=c.parentNode}}a=a.return}cs(function(){var S=n,z=vi(e),M=[];t:{var b=Bs.get(t);if(b!==void 0){var E=Vu,B=t;switch(t){case"keypress":if(Lu(e)===0)break t;case"keydown":case"keyup":E=iy;break;case"focusin":B="focus",E=Ei;break;case"focusout":B="blur",E=Ei;break;case"beforeblur":case"afterblur":E=Ei;break;case"click":if(e.button===2)break t;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":E=os;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":E=$m;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":E=sy;break;case Cs:case Rs:case qs:E=Fm;break;case Ns:E=ry;break;case"scroll":case"scrollend":E=Jm;break;case"wheel":E=my;break;case"copy":case"cut":case"paste":E=Pm;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":E=ds;break;case"toggle":case"beforetoggle":E=vy}var X=(l&4)!==0,mt=!X&&(t==="scroll"||t==="scrollend"),h=X?b!==null?b+"Capture":null:b;X=[];for(var d=S,g;d!==null;){var _=d;if(g=_.stateNode,_=_.tag,_!==5&&_!==26&&_!==27||g===null||h===null||(_=xa(d,h),_!=null&&X.push(bu(d,_,g))),mt)break;d=d.return}0<X.length&&(b=new E(b,B,null,e,z),M.push({event:b,listeners:X}))}}if((l&7)===0){t:{if(b=t==="mouseover"||t==="pointerover",E=t==="mouseout"||t==="pointerout",b&&e!==yi&&(B=e.relatedTarget||e.fromElement)&&(we(B)||B[Je]))break t;if((E||b)&&(b=z.window===z?z:(b=z.ownerDocument)?b.defaultView||b.parentWindow:window,E?(B=e.relatedTarget||e.toElement,E=S,B=B?we(B):null,B!==null&&(mt=p(B),X=B.tag,B!==mt||X!==5&&X!==27&&X!==6)&&(B=null)):(E=null,B=S),E!==B)){if(X=os,_="onMouseLeave",h="onMouseEnter",d="mouse",(t==="pointerout"||t==="pointerover")&&(X=ds,_="onPointerLeave",h="onPointerEnter",d="pointer"),mt=E==null?b:Xa(E),g=B==null?b:Xa(B),b=new X(_,d+"leave",E,e,z),b.target=mt,b.relatedTarget=g,_=null,we(z)===S&&(X=new X(h,d+"enter",B,e,z),X.target=g,X.relatedTarget=mt,_=X),mt=_,E&&B)l:{for(X=yv,h=E,d=B,g=0,_=h;_;_=X(_))g++;_=0;for(var G=d;G;G=X(G))_++;for(;0<g-_;)h=X(h),g--;for(;0<_-g;)d=X(d),_--;for(;g--;){if(h===d||d!==null&&h===d.alternate){X=h;break l}h=X(h),d=X(d)}X=null}else X=null;E!==null&&ud(M,b,E,X,!1),B!==null&&mt!==null&&ud(M,mt,B,X,!0)}}t:{if(b=S?Xa(S):window,E=b.nodeName&&b.nodeName.toLowerCase(),E==="select"||E==="input"&&b.type==="file")var at=Ts;else if(Ss(b))if(Es)at=Oy;else{at=py;var Y=Ay}else E=b.nodeName,!E||E.toLowerCase()!=="input"||b.type!=="checkbox"&&b.type!=="radio"?S&&mi(S.elementType)&&(at=Ts):at=zy;if(at&&(at=at(t,S))){bs(M,at,e,z);break t}Y&&Y(t,b,S),t==="focusout"&&S&&b.type==="number"&&S.memoizedProps.value!=null&&di(b,"number",b.value)}switch(Y=S?Xa(S):window,t){case"focusin":(Ss(Y)||Y.contentEditable==="true")&&(ea=Y,Mi=S,$a=null);break;case"focusout":$a=Mi=ea=null;break;case"mousedown":Di=!0;break;case"contextmenu":case"mouseup":case"dragend":Di=!1,Us(M,e,z);break;case"selectionchange":if(My)break;case"keydown":case"keyup":Us(M,e,z)}var J;if(pi)t:{switch(t){case"compositionstart":var P="onCompositionStart";break t;case"compositionend":P="onCompositionEnd";break t;case"compositionupdate":P="onCompositionUpdate";break t}P=void 0}else la?hs(t,e)&&(P="onCompositionEnd"):t==="keydown"&&e.keyCode===229&&(P="onCompositionStart");P&&(ms&&e.locale!=="ko"&&(la||P!=="onCompositionStart"?P==="onCompositionEnd"&&la&&(J=fs()):(ae=z,Si="value"in ae?ae.value:ae.textContent,la=!0)),Y=Bn(S,P),0<Y.length&&(P=new rs(P,t,null,e,z),M.push({event:P,listeners:Y}),J?P.data=J:(J=gs(e),J!==null&&(P.data=J)))),(J=gy?Sy(t,e):by(t,e))&&(P=Bn(S,"onBeforeInput"),0<P.length&&(Y=new rs("onBeforeInput","beforeinput",null,e,z),M.push({event:Y,listeners:P}),Y.data=J)),ov(M,t,S,e,z)}ed(M,l)})}function bu(t,l,e){return{instance:t,listener:l,currentTarget:e}}function Bn(t,l){for(var e=l+"Capture",a=[];t!==null;){var u=t,n=u.stateNode;if(u=u.tag,u!==5&&u!==26&&u!==27||n===null||(u=xa(t,e),u!=null&&a.unshift(bu(t,u,n)),u=xa(t,l),u!=null&&a.push(bu(t,u,n))),t.tag===3)return a;t=t.return}return[]}function yv(t){if(t===null)return null;do t=t.return;while(t&&t.tag!==5&&t.tag!==27);return t||null}function ud(t,l,e,a,u){for(var n=l._reactName,i=[];e!==null&&e!==a;){var c=e,o=c.alternate,S=c.stateNode;if(c=c.tag,o!==null&&o===a)break;c!==5&&c!==26&&c!==27||S===null||(o=S,u?(S=xa(e,n),S!=null&&i.unshift(bu(e,S,o))):u||(S=xa(e,n),S!=null&&i.push(bu(e,S,o)))),e=e.return}i.length!==0&&t.push({event:l,listeners:i})}var vv=/\r\n?/g,hv=/\u0000|\uFFFD/g;function nd(t){return(typeof t=="string"?t:""+t).replace(vv,`
`).replace(hv,"")}function id(t,l){return l=nd(l),nd(t)===l}function dt(t,l,e,a,u,n){switch(e){case"children":typeof a=="string"?l==="body"||l==="textarea"&&a===""||Ie(t,a):(typeof a=="number"||typeof a=="bigint")&&l!=="body"&&Ie(t,""+a);break;case"className":Gu(t,"class",a);break;case"tabIndex":Gu(t,"tabindex",a);break;case"dir":case"role":case"viewBox":case"width":case"height":Gu(t,e,a);break;case"style":ns(t,a,n);break;case"data":if(l!=="object"){Gu(t,"data",a);break}case"src":case"href":if(a===""&&(l!=="a"||e!=="href")){t.removeAttribute(e);break}if(a==null||typeof a=="function"||typeof a=="symbol"||typeof a=="boolean"){t.removeAttribute(e);break}a=xu(""+a),t.setAttribute(e,a);break;case"action":case"formAction":if(typeof a=="function"){t.setAttribute(e,"javascript:throw new Error('A React form was unexpectedly submitted. If you called form.submit() manually, consider using form.requestSubmit() instead. If you\\'re trying to use event.stopPropagation() in a submit event handler, consider also calling event.preventDefault().')");break}else typeof n=="function"&&(e==="formAction"?(l!=="input"&&dt(t,l,"name",u.name,u,null),dt(t,l,"formEncType",u.formEncType,u,null),dt(t,l,"formMethod",u.formMethod,u,null),dt(t,l,"formTarget",u.formTarget,u,null)):(dt(t,l,"encType",u.encType,u,null),dt(t,l,"method",u.method,u,null),dt(t,l,"target",u.target,u,null)));if(a==null||typeof a=="symbol"||typeof a=="boolean"){t.removeAttribute(e);break}a=xu(""+a),t.setAttribute(e,a);break;case"onClick":a!=null&&(t.onclick=Gl);break;case"onScroll":a!=null&&F("scroll",t);break;case"onScrollEnd":a!=null&&F("scrollend",t);break;case"dangerouslySetInnerHTML":if(a!=null){if(typeof a!="object"||!("__html"in a))throw Error(s(61));if(e=a.__html,e!=null){if(u.children!=null)throw Error(s(60));t.innerHTML=e}}break;case"multiple":t.multiple=a&&typeof a!="function"&&typeof a!="symbol";break;case"muted":t.muted=a&&typeof a!="function"&&typeof a!="symbol";break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"defaultValue":case"defaultChecked":case"innerHTML":case"ref":break;case"autoFocus":break;case"xlinkHref":if(a==null||typeof a=="function"||typeof a=="boolean"||typeof a=="symbol"){t.removeAttribute("xlink:href");break}e=xu(""+a),t.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href",e);break;case"contentEditable":case"spellCheck":case"draggable":case"value":case"autoReverse":case"externalResourcesRequired":case"focusable":case"preserveAlpha":a!=null&&typeof a!="function"&&typeof a!="symbol"?t.setAttribute(e,""+a):t.removeAttribute(e);break;case"inert":case"allowFullScreen":case"async":case"autoPlay":case"controls":case"default":case"defer":case"disabled":case"disablePictureInPicture":case"disableRemotePlayback":case"formNoValidate":case"hidden":case"loop":case"noModule":case"noValidate":case"open":case"playsInline":case"readOnly":case"required":case"reversed":case"scoped":case"seamless":case"itemScope":a&&typeof a!="function"&&typeof a!="symbol"?t.setAttribute(e,""):t.removeAttribute(e);break;case"capture":case"download":a===!0?t.setAttribute(e,""):a!==!1&&a!=null&&typeof a!="function"&&typeof a!="symbol"?t.setAttribute(e,a):t.removeAttribute(e);break;case"cols":case"rows":case"size":case"span":a!=null&&typeof a!="function"&&typeof a!="symbol"&&!isNaN(a)&&1<=a?t.setAttribute(e,a):t.removeAttribute(e);break;case"rowSpan":case"start":a==null||typeof a=="function"||typeof a=="symbol"||isNaN(a)?t.removeAttribute(e):t.setAttribute(e,a);break;case"popover":F("beforetoggle",t),F("toggle",t),Yu(t,"popover",a);break;case"xlinkActuate":Yl(t,"http://www.w3.org/1999/xlink","xlink:actuate",a);break;case"xlinkArcrole":Yl(t,"http://www.w3.org/1999/xlink","xlink:arcrole",a);break;case"xlinkRole":Yl(t,"http://www.w3.org/1999/xlink","xlink:role",a);break;case"xlinkShow":Yl(t,"http://www.w3.org/1999/xlink","xlink:show",a);break;case"xlinkTitle":Yl(t,"http://www.w3.org/1999/xlink","xlink:title",a);break;case"xlinkType":Yl(t,"http://www.w3.org/1999/xlink","xlink:type",a);break;case"xmlBase":Yl(t,"http://www.w3.org/XML/1998/namespace","xml:base",a);break;case"xmlLang":Yl(t,"http://www.w3.org/XML/1998/namespace","xml:lang",a);break;case"xmlSpace":Yl(t,"http://www.w3.org/XML/1998/namespace","xml:space",a);break;case"is":Yu(t,"is",a);break;case"innerText":case"textContent":break;default:(!(2<e.length)||e[0]!=="o"&&e[0]!=="O"||e[1]!=="n"&&e[1]!=="N")&&(e=Vm.get(e)||e,Yu(t,e,a))}}function Pc(t,l,e,a,u,n){switch(e){case"style":ns(t,a,n);break;case"dangerouslySetInnerHTML":if(a!=null){if(typeof a!="object"||!("__html"in a))throw Error(s(61));if(e=a.__html,e!=null){if(u.children!=null)throw Error(s(60));t.innerHTML=e}}break;case"children":typeof a=="string"?Ie(t,a):(typeof a=="number"||typeof a=="bigint")&&Ie(t,""+a);break;case"onScroll":a!=null&&F("scroll",t);break;case"onScrollEnd":a!=null&&F("scrollend",t);break;case"onClick":a!=null&&(t.onclick=Gl);break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"innerHTML":case"ref":break;case"innerText":case"textContent":break;default:if(!kf.hasOwnProperty(e))t:{if(e[0]==="o"&&e[1]==="n"&&(u=e.endsWith("Capture"),l=e.slice(2,u?e.length-7:void 0),n=t[kt]||null,n=n!=null?n[e]:null,typeof n=="function"&&t.removeEventListener(l,n,u),typeof a=="function")){typeof n!="function"&&n!==null&&(e in t?t[e]=null:t.hasAttribute(e)&&t.removeAttribute(e)),t.addEventListener(l,a,u);break t}e in t?t[e]=a:a===!0?t.setAttribute(e,""):Yu(t,e,a)}}}function Vt(t,l,e){switch(l){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"img":F("error",t),F("load",t);var a=!1,u=!1,n;for(n in e)if(e.hasOwnProperty(n)){var i=e[n];if(i!=null)switch(n){case"src":a=!0;break;case"srcSet":u=!0;break;case"children":case"dangerouslySetInnerHTML":throw Error(s(137,l));default:dt(t,l,n,i,e,null)}}u&&dt(t,l,"srcSet",e.srcSet,e,null),a&&dt(t,l,"src",e.src,e,null);return;case"input":F("invalid",t);var c=n=i=u=null,o=null,S=null;for(a in e)if(e.hasOwnProperty(a)){var z=e[a];if(z!=null)switch(a){case"name":u=z;break;case"type":i=z;break;case"checked":o=z;break;case"defaultChecked":S=z;break;case"value":n=z;break;case"defaultValue":c=z;break;case"children":case"dangerouslySetInnerHTML":if(z!=null)throw Error(s(137,l));break;default:dt(t,l,a,z,e,null)}}ls(t,n,c,o,S,i,u,!1);return;case"select":F("invalid",t),a=i=n=null;for(u in e)if(e.hasOwnProperty(u)&&(c=e[u],c!=null))switch(u){case"value":n=c;break;case"defaultValue":i=c;break;case"multiple":a=c;default:dt(t,l,u,c,e,null)}l=n,e=i,t.multiple=!!a,l!=null?Fe(t,!!a,l,!1):e!=null&&Fe(t,!!a,e,!0);return;case"textarea":F("invalid",t),n=u=a=null;for(i in e)if(e.hasOwnProperty(i)&&(c=e[i],c!=null))switch(i){case"value":a=c;break;case"defaultValue":u=c;break;case"children":n=c;break;case"dangerouslySetInnerHTML":if(c!=null)throw Error(s(91));break;default:dt(t,l,i,c,e,null)}as(t,a,u,n);return;case"option":for(o in e)if(e.hasOwnProperty(o)&&(a=e[o],a!=null))switch(o){case"selected":t.selected=a&&typeof a!="function"&&typeof a!="symbol";break;default:dt(t,l,o,a,e,null)}return;case"dialog":F("beforetoggle",t),F("toggle",t),F("cancel",t),F("close",t);break;case"iframe":case"object":F("load",t);break;case"video":case"audio":for(a=0;a<Su.length;a++)F(Su[a],t);break;case"image":F("error",t),F("load",t);break;case"details":F("toggle",t);break;case"embed":case"source":case"link":F("error",t),F("load",t);case"area":case"base":case"br":case"col":case"hr":case"keygen":case"meta":case"param":case"track":case"wbr":case"menuitem":for(S in e)if(e.hasOwnProperty(S)&&(a=e[S],a!=null))switch(S){case"children":case"dangerouslySetInnerHTML":throw Error(s(137,l));default:dt(t,l,S,a,e,null)}return;default:if(mi(l)){for(z in e)e.hasOwnProperty(z)&&(a=e[z],a!==void 0&&Pc(t,l,z,a,e,void 0));return}}for(c in e)e.hasOwnProperty(c)&&(a=e[c],a!=null&&dt(t,l,c,a,e,null))}function gv(t,l,e,a){switch(l){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"input":var u=null,n=null,i=null,c=null,o=null,S=null,z=null;for(E in e){var M=e[E];if(e.hasOwnProperty(E)&&M!=null)switch(E){case"checked":break;case"value":break;case"defaultValue":o=M;default:a.hasOwnProperty(E)||dt(t,l,E,null,a,M)}}for(var b in a){var E=a[b];if(M=e[b],a.hasOwnProperty(b)&&(E!=null||M!=null))switch(b){case"type":n=E;break;case"name":u=E;break;case"checked":S=E;break;case"defaultChecked":z=E;break;case"value":i=E;break;case"defaultValue":c=E;break;case"children":case"dangerouslySetInnerHTML":if(E!=null)throw Error(s(137,l));break;default:E!==M&&dt(t,l,b,E,a,M)}}ri(t,i,c,o,S,z,n,u);return;case"select":E=i=c=b=null;for(n in e)if(o=e[n],e.hasOwnProperty(n)&&o!=null)switch(n){case"value":break;case"multiple":E=o;default:a.hasOwnProperty(n)||dt(t,l,n,null,a,o)}for(u in a)if(n=a[u],o=e[u],a.hasOwnProperty(u)&&(n!=null||o!=null))switch(u){case"value":b=n;break;case"defaultValue":c=n;break;case"multiple":i=n;default:n!==o&&dt(t,l,u,n,a,o)}l=c,e=i,a=E,b!=null?Fe(t,!!e,b,!1):!!a!=!!e&&(l!=null?Fe(t,!!e,l,!0):Fe(t,!!e,e?[]:"",!1));return;case"textarea":E=b=null;for(c in e)if(u=e[c],e.hasOwnProperty(c)&&u!=null&&!a.hasOwnProperty(c))switch(c){case"value":break;case"children":break;default:dt(t,l,c,null,a,u)}for(i in a)if(u=a[i],n=e[i],a.hasOwnProperty(i)&&(u!=null||n!=null))switch(i){case"value":b=u;break;case"defaultValue":E=u;break;case"children":break;case"dangerouslySetInnerHTML":if(u!=null)throw Error(s(91));break;default:u!==n&&dt(t,l,i,u,a,n)}es(t,b,E);return;case"option":for(var B in e)if(b=e[B],e.hasOwnProperty(B)&&b!=null&&!a.hasOwnProperty(B))switch(B){case"selected":t.selected=!1;break;default:dt(t,l,B,null,a,b)}for(o in a)if(b=a[o],E=e[o],a.hasOwnProperty(o)&&b!==E&&(b!=null||E!=null))switch(o){case"selected":t.selected=b&&typeof b!="function"&&typeof b!="symbol";break;default:dt(t,l,o,b,a,E)}return;case"img":case"link":case"area":case"base":case"br":case"col":case"embed":case"hr":case"keygen":case"meta":case"param":case"source":case"track":case"wbr":case"menuitem":for(var X in e)b=e[X],e.hasOwnProperty(X)&&b!=null&&!a.hasOwnProperty(X)&&dt(t,l,X,null,a,b);for(S in a)if(b=a[S],E=e[S],a.hasOwnProperty(S)&&b!==E&&(b!=null||E!=null))switch(S){case"children":case"dangerouslySetInnerHTML":if(b!=null)throw Error(s(137,l));break;default:dt(t,l,S,b,a,E)}return;default:if(mi(l)){for(var mt in e)b=e[mt],e.hasOwnProperty(mt)&&b!==void 0&&!a.hasOwnProperty(mt)&&Pc(t,l,mt,void 0,a,b);for(z in a)b=a[z],E=e[z],!a.hasOwnProperty(z)||b===E||b===void 0&&E===void 0||Pc(t,l,z,b,a,E);return}}for(var h in e)b=e[h],e.hasOwnProperty(h)&&b!=null&&!a.hasOwnProperty(h)&&dt(t,l,h,null,a,b);for(M in a)b=a[M],E=e[M],!a.hasOwnProperty(M)||b===E||b==null&&E==null||dt(t,l,M,b,a,E)}function cd(t){switch(t){case"css":case"script":case"font":case"img":case"image":case"input":case"link":return!0;default:return!1}}function Sv(){if(typeof performance.getEntriesByType=="function"){for(var t=0,l=0,e=performance.getEntriesByType("resource"),a=0;a<e.length;a++){var u=e[a],n=u.transferSize,i=u.initiatorType,c=u.duration;if(n&&c&&cd(i)){for(i=0,c=u.responseEnd,a+=1;a<e.length;a++){var o=e[a],S=o.startTime;if(S>c)break;var z=o.transferSize,M=o.initiatorType;z&&cd(M)&&(o=o.responseEnd,i+=z*(o<c?1:(c-S)/(o-S)))}if(--a,l+=8*(n+i)/(u.duration/1e3),t++,10<t)break}}if(0<t)return l/t/1e6}return navigator.connection&&(t=navigator.connection.downlink,typeof t=="number")?t:5}var tf=null,lf=null;function jn(t){return t.nodeType===9?t:t.ownerDocument}function fd(t){switch(t){case"http://www.w3.org/2000/svg":return 1;case"http://www.w3.org/1998/Math/MathML":return 2;default:return 0}}function sd(t,l){if(t===0)switch(l){case"svg":return 1;case"math":return 2;default:return 0}return t===1&&l==="foreignObject"?0:t}function ef(t,l){return t==="textarea"||t==="noscript"||typeof l.children=="string"||typeof l.children=="number"||typeof l.children=="bigint"||typeof l.dangerouslySetInnerHTML=="object"&&l.dangerouslySetInnerHTML!==null&&l.dangerouslySetInnerHTML.__html!=null}var af=null;function bv(){var t=window.event;return t&&t.type==="popstate"?t===af?!1:(af=t,!0):(af=null,!1)}var od=typeof setTimeout=="function"?setTimeout:void 0,Tv=typeof clearTimeout=="function"?clearTimeout:void 0,rd=typeof Promise=="function"?Promise:void 0,Ev=typeof queueMicrotask=="function"?queueMicrotask:typeof rd<"u"?function(t){return rd.resolve(null).then(t).catch(Av)}:od;function Av(t){setTimeout(function(){throw t})}function Te(t){return t==="head"}function dd(t,l){var e=l,a=0;do{var u=e.nextSibling;if(t.removeChild(e),u&&u.nodeType===8)if(e=u.data,e==="/$"||e==="/&"){if(a===0){t.removeChild(u),Ua(l);return}a--}else if(e==="$"||e==="$?"||e==="$~"||e==="$!"||e==="&")a++;else if(e==="html")Tu(t.ownerDocument.documentElement);else if(e==="head"){e=t.ownerDocument.head,Tu(e);for(var n=e.firstChild;n;){var i=n.nextSibling,c=n.nodeName;n[Ga]||c==="SCRIPT"||c==="STYLE"||c==="LINK"&&n.rel.toLowerCase()==="stylesheet"||e.removeChild(n),n=i}}else e==="body"&&Tu(t.ownerDocument.body);e=u}while(e);Ua(l)}function md(t,l){var e=t;t=0;do{var a=e.nextSibling;if(e.nodeType===1?l?(e._stashedDisplay=e.style.display,e.style.display="none"):(e.style.display=e._stashedDisplay||"",e.getAttribute("style")===""&&e.removeAttribute("style")):e.nodeType===3&&(l?(e._stashedText=e.nodeValue,e.nodeValue=""):e.nodeValue=e._stashedText||""),a&&a.nodeType===8)if(e=a.data,e==="/$"){if(t===0)break;t--}else e!=="$"&&e!=="$?"&&e!=="$~"&&e!=="$!"||t++;e=a}while(e)}function uf(t){var l=t.firstChild;for(l&&l.nodeType===10&&(l=l.nextSibling);l;){var e=l;switch(l=l.nextSibling,e.nodeName){case"HTML":case"HEAD":case"BODY":uf(e),si(e);continue;case"SCRIPT":case"STYLE":continue;case"LINK":if(e.rel.toLowerCase()==="stylesheet")continue}t.removeChild(e)}}function pv(t,l,e,a){for(;t.nodeType===1;){var u=e;if(t.nodeName.toLowerCase()!==l.toLowerCase()){if(!a&&(t.nodeName!=="INPUT"||t.type!=="hidden"))break}else if(a){if(!t[Ga])switch(l){case"meta":if(!t.hasAttribute("itemprop"))break;return t;case"link":if(n=t.getAttribute("rel"),n==="stylesheet"&&t.hasAttribute("data-precedence"))break;if(n!==u.rel||t.getAttribute("href")!==(u.href==null||u.href===""?null:u.href)||t.getAttribute("crossorigin")!==(u.crossOrigin==null?null:u.crossOrigin)||t.getAttribute("title")!==(u.title==null?null:u.title))break;return t;case"style":if(t.hasAttribute("data-precedence"))break;return t;case"script":if(n=t.getAttribute("src"),(n!==(u.src==null?null:u.src)||t.getAttribute("type")!==(u.type==null?null:u.type)||t.getAttribute("crossorigin")!==(u.crossOrigin==null?null:u.crossOrigin))&&n&&t.hasAttribute("async")&&!t.hasAttribute("itemprop"))break;return t;default:return t}}else if(l==="input"&&t.type==="hidden"){var n=u.name==null?null:""+u.name;if(u.type==="hidden"&&t.getAttribute("name")===n)return t}else return t;if(t=zl(t.nextSibling),t===null)break}return null}function zv(t,l,e){if(l==="")return null;for(;t.nodeType!==3;)if((t.nodeType!==1||t.nodeName!=="INPUT"||t.type!=="hidden")&&!e||(t=zl(t.nextSibling),t===null))return null;return t}function yd(t,l){for(;t.nodeType!==8;)if((t.nodeType!==1||t.nodeName!=="INPUT"||t.type!=="hidden")&&!l||(t=zl(t.nextSibling),t===null))return null;return t}function nf(t){return t.data==="$?"||t.data==="$~"}function cf(t){return t.data==="$!"||t.data==="$?"&&t.ownerDocument.readyState!=="loading"}function Ov(t,l){var e=t.ownerDocument;if(t.data==="$~")t._reactRetry=l;else if(t.data!=="$?"||e.readyState!=="loading")l();else{var a=function(){l(),e.removeEventListener("DOMContentLoaded",a)};e.addEventListener("DOMContentLoaded",a),t._reactRetry=a}}function zl(t){for(;t!=null;t=t.nextSibling){var l=t.nodeType;if(l===1||l===3)break;if(l===8){if(l=t.data,l==="$"||l==="$!"||l==="$?"||l==="$~"||l==="&"||l==="F!"||l==="F")break;if(l==="/$"||l==="/&")return null}}return t}var ff=null;function vd(t){t=t.nextSibling;for(var l=0;t;){if(t.nodeType===8){var e=t.data;if(e==="/$"||e==="/&"){if(l===0)return zl(t.nextSibling);l--}else e!=="$"&&e!=="$!"&&e!=="$?"&&e!=="$~"&&e!=="&"||l++}t=t.nextSibling}return null}function hd(t){t=t.previousSibling;for(var l=0;t;){if(t.nodeType===8){var e=t.data;if(e==="$"||e==="$!"||e==="$?"||e==="$~"||e==="&"){if(l===0)return t;l--}else e!=="/$"&&e!=="/&"||l++}t=t.previousSibling}return null}function gd(t,l,e){switch(l=jn(e),t){case"html":if(t=l.documentElement,!t)throw Error(s(452));return t;case"head":if(t=l.head,!t)throw Error(s(453));return t;case"body":if(t=l.body,!t)throw Error(s(454));return t;default:throw Error(s(451))}}function Tu(t){for(var l=t.attributes;l.length;)t.removeAttributeNode(l[0]);si(t)}var Ol=new Map,Sd=new Set;function Yn(t){return typeof t.getRootNode=="function"?t.getRootNode():t.nodeType===9?t:t.ownerDocument}var te=C.d;C.d={f:_v,r:Mv,D:Dv,C:Uv,L:Hv,m:Cv,X:qv,S:Rv,M:Nv};function _v(){var t=te.f(),l=Dn();return t||l}function Mv(t){var l=$e(t);l!==null&&l.tag===5&&l.type==="form"?Bo(l):te.r(t)}var _a=typeof document>"u"?null:document;function bd(t,l,e){var a=_a;if(a&&typeof l=="string"&&l){var u=gl(l);u='link[rel="'+t+'"][href="'+u+'"]',typeof e=="string"&&(u+='[crossorigin="'+e+'"]'),Sd.has(u)||(Sd.add(u),t={rel:t,crossOrigin:e,href:l},a.querySelector(u)===null&&(l=a.createElement("link"),Vt(l,"link",t),jt(l),a.head.appendChild(l)))}}function Dv(t){te.D(t),bd("dns-prefetch",t,null)}function Uv(t,l){te.C(t,l),bd("preconnect",t,l)}function Hv(t,l,e){te.L(t,l,e);var a=_a;if(a&&t&&l){var u='link[rel="preload"][as="'+gl(l)+'"]';l==="image"&&e&&e.imageSrcSet?(u+='[imagesrcset="'+gl(e.imageSrcSet)+'"]',typeof e.imageSizes=="string"&&(u+='[imagesizes="'+gl(e.imageSizes)+'"]')):u+='[href="'+gl(t)+'"]';var n=u;switch(l){case"style":n=Ma(t);break;case"script":n=Da(t)}Ol.has(n)||(t=N({rel:"preload",href:l==="image"&&e&&e.imageSrcSet?void 0:t,as:l},e),Ol.set(n,t),a.querySelector(u)!==null||l==="style"&&a.querySelector(Eu(n))||l==="script"&&a.querySelector(Au(n))||(l=a.createElement("link"),Vt(l,"link",t),jt(l),a.head.appendChild(l)))}}function Cv(t,l){te.m(t,l);var e=_a;if(e&&t){var a=l&&typeof l.as=="string"?l.as:"script",u='link[rel="modulepreload"][as="'+gl(a)+'"][href="'+gl(t)+'"]',n=u;switch(a){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":n=Da(t)}if(!Ol.has(n)&&(t=N({rel:"modulepreload",href:t},l),Ol.set(n,t),e.querySelector(u)===null)){switch(a){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":if(e.querySelector(Au(n)))return}a=e.createElement("link"),Vt(a,"link",t),jt(a),e.head.appendChild(a)}}}function Rv(t,l,e){te.S(t,l,e);var a=_a;if(a&&t){var u=We(a).hoistableStyles,n=Ma(t);l=l||"default";var i=u.get(n);if(!i){var c={loading:0,preload:null};if(i=a.querySelector(Eu(n)))c.loading=5;else{t=N({rel:"stylesheet",href:t,"data-precedence":l},e),(e=Ol.get(n))&&sf(t,e);var o=i=a.createElement("link");jt(o),Vt(o,"link",t),o._p=new Promise(function(S,z){o.onload=S,o.onerror=z}),o.addEventListener("load",function(){c.loading|=1}),o.addEventListener("error",function(){c.loading|=2}),c.loading|=4,Gn(i,l,a)}i={type:"stylesheet",instance:i,count:1,state:c},u.set(n,i)}}}function qv(t,l){te.X(t,l);var e=_a;if(e&&t){var a=We(e).hoistableScripts,u=Da(t),n=a.get(u);n||(n=e.querySelector(Au(u)),n||(t=N({src:t,async:!0},l),(l=Ol.get(u))&&of(t,l),n=e.createElement("script"),jt(n),Vt(n,"link",t),e.head.appendChild(n)),n={type:"script",instance:n,count:1,state:null},a.set(u,n))}}function Nv(t,l){te.M(t,l);var e=_a;if(e&&t){var a=We(e).hoistableScripts,u=Da(t),n=a.get(u);n||(n=e.querySelector(Au(u)),n||(t=N({src:t,async:!0,type:"module"},l),(l=Ol.get(u))&&of(t,l),n=e.createElement("script"),jt(n),Vt(n,"link",t),e.head.appendChild(n)),n={type:"script",instance:n,count:1,state:null},a.set(u,n))}}function Td(t,l,e,a){var u=(u=W.current)?Yn(u):null;if(!u)throw Error(s(446));switch(t){case"meta":case"title":return null;case"style":return typeof e.precedence=="string"&&typeof e.href=="string"?(l=Ma(e.href),e=We(u).hoistableStyles,a=e.get(l),a||(a={type:"style",instance:null,count:0,state:null},e.set(l,a)),a):{type:"void",instance:null,count:0,state:null};case"link":if(e.rel==="stylesheet"&&typeof e.href=="string"&&typeof e.precedence=="string"){t=Ma(e.href);var n=We(u).hoistableStyles,i=n.get(t);if(i||(u=u.ownerDocument||u,i={type:"stylesheet",instance:null,count:0,state:{loading:0,preload:null}},n.set(t,i),(n=u.querySelector(Eu(t)))&&!n._p&&(i.instance=n,i.state.loading=5),Ol.has(t)||(e={rel:"preload",as:"style",href:e.href,crossOrigin:e.crossOrigin,integrity:e.integrity,media:e.media,hrefLang:e.hrefLang,referrerPolicy:e.referrerPolicy},Ol.set(t,e),n||Bv(u,t,e,i.state))),l&&a===null)throw Error(s(528,""));return i}if(l&&a!==null)throw Error(s(529,""));return null;case"script":return l=e.async,e=e.src,typeof e=="string"&&l&&typeof l!="function"&&typeof l!="symbol"?(l=Da(e),e=We(u).hoistableScripts,a=e.get(l),a||(a={type:"script",instance:null,count:0,state:null},e.set(l,a)),a):{type:"void",instance:null,count:0,state:null};default:throw Error(s(444,t))}}function Ma(t){return'href="'+gl(t)+'"'}function Eu(t){return'link[rel="stylesheet"]['+t+"]"}function Ed(t){return N({},t,{"data-precedence":t.precedence,precedence:null})}function Bv(t,l,e,a){t.querySelector('link[rel="preload"][as="style"]['+l+"]")?a.loading=1:(l=t.createElement("link"),a.preload=l,l.addEventListener("load",function(){return a.loading|=1}),l.addEventListener("error",function(){return a.loading|=2}),Vt(l,"link",e),jt(l),t.head.appendChild(l))}function Da(t){return'[src="'+gl(t)+'"]'}function Au(t){return"script[async]"+t}function Ad(t,l,e){if(l.count++,l.instance===null)switch(l.type){case"style":var a=t.querySelector('style[data-href~="'+gl(e.href)+'"]');if(a)return l.instance=a,jt(a),a;var u=N({},e,{"data-href":e.href,"data-precedence":e.precedence,href:null,precedence:null});return a=(t.ownerDocument||t).createElement("style"),jt(a),Vt(a,"style",u),Gn(a,e.precedence,t),l.instance=a;case"stylesheet":u=Ma(e.href);var n=t.querySelector(Eu(u));if(n)return l.state.loading|=4,l.instance=n,jt(n),n;a=Ed(e),(u=Ol.get(u))&&sf(a,u),n=(t.ownerDocument||t).createElement("link"),jt(n);var i=n;return i._p=new Promise(function(c,o){i.onload=c,i.onerror=o}),Vt(n,"link",a),l.state.loading|=4,Gn(n,e.precedence,t),l.instance=n;case"script":return n=Da(e.src),(u=t.querySelector(Au(n)))?(l.instance=u,jt(u),u):(a=e,(u=Ol.get(n))&&(a=N({},e),of(a,u)),t=t.ownerDocument||t,u=t.createElement("script"),jt(u),Vt(u,"link",a),t.head.appendChild(u),l.instance=u);case"void":return null;default:throw Error(s(443,l.type))}else l.type==="stylesheet"&&(l.state.loading&4)===0&&(a=l.instance,l.state.loading|=4,Gn(a,e.precedence,t));return l.instance}function Gn(t,l,e){for(var a=e.querySelectorAll('link[rel="stylesheet"][data-precedence],style[data-precedence]'),u=a.length?a[a.length-1]:null,n=u,i=0;i<a.length;i++){var c=a[i];if(c.dataset.precedence===l)n=c;else if(n!==u)break}n?n.parentNode.insertBefore(t,n.nextSibling):(l=e.nodeType===9?e.head:e,l.insertBefore(t,l.firstChild))}function sf(t,l){t.crossOrigin==null&&(t.crossOrigin=l.crossOrigin),t.referrerPolicy==null&&(t.referrerPolicy=l.referrerPolicy),t.title==null&&(t.title=l.title)}function of(t,l){t.crossOrigin==null&&(t.crossOrigin=l.crossOrigin),t.referrerPolicy==null&&(t.referrerPolicy=l.referrerPolicy),t.integrity==null&&(t.integrity=l.integrity)}var Xn=null;function pd(t,l,e){if(Xn===null){var a=new Map,u=Xn=new Map;u.set(e,a)}else u=Xn,a=u.get(e),a||(a=new Map,u.set(e,a));if(a.has(t))return a;for(a.set(t,null),e=e.getElementsByTagName(t),u=0;u<e.length;u++){var n=e[u];if(!(n[Ga]||n[xt]||t==="link"&&n.getAttribute("rel")==="stylesheet")&&n.namespaceURI!=="http://www.w3.org/2000/svg"){var i=n.getAttribute(l)||"";i=t+i;var c=a.get(i);c?c.push(n):a.set(i,[n])}}return a}function zd(t,l,e){t=t.ownerDocument||t,t.head.insertBefore(e,l==="title"?t.querySelector("head > title"):null)}function jv(t,l,e){if(e===1||l.itemProp!=null)return!1;switch(t){case"meta":case"title":return!0;case"style":if(typeof l.precedence!="string"||typeof l.href!="string"||l.href==="")break;return!0;case"link":if(typeof l.rel!="string"||typeof l.href!="string"||l.href===""||l.onLoad||l.onError)break;switch(l.rel){case"stylesheet":return t=l.disabled,typeof l.precedence=="string"&&t==null;default:return!0}case"script":if(l.async&&typeof l.async!="function"&&typeof l.async!="symbol"&&!l.onLoad&&!l.onError&&l.src&&typeof l.src=="string")return!0}return!1}function Od(t){return!(t.type==="stylesheet"&&(t.state.loading&3)===0)}function Yv(t,l,e,a){if(e.type==="stylesheet"&&(typeof a.media!="string"||matchMedia(a.media).matches!==!1)&&(e.state.loading&4)===0){if(e.instance===null){var u=Ma(a.href),n=l.querySelector(Eu(u));if(n){l=n._p,l!==null&&typeof l=="object"&&typeof l.then=="function"&&(t.count++,t=xn.bind(t),l.then(t,t)),e.state.loading|=4,e.instance=n,jt(n);return}n=l.ownerDocument||l,a=Ed(a),(u=Ol.get(u))&&sf(a,u),n=n.createElement("link"),jt(n);var i=n;i._p=new Promise(function(c,o){i.onload=c,i.onerror=o}),Vt(n,"link",a),e.instance=n}t.stylesheets===null&&(t.stylesheets=new Map),t.stylesheets.set(e,l),(l=e.state.preload)&&(e.state.loading&3)===0&&(t.count++,e=xn.bind(t),l.addEventListener("load",e),l.addEventListener("error",e))}}var rf=0;function Gv(t,l){return t.stylesheets&&t.count===0&&Ln(t,t.stylesheets),0<t.count||0<t.imgCount?function(e){var a=setTimeout(function(){if(t.stylesheets&&Ln(t,t.stylesheets),t.unsuspend){var n=t.unsuspend;t.unsuspend=null,n()}},6e4+l);0<t.imgBytes&&rf===0&&(rf=62500*Sv());var u=setTimeout(function(){if(t.waitingForImages=!1,t.count===0&&(t.stylesheets&&Ln(t,t.stylesheets),t.unsuspend)){var n=t.unsuspend;t.unsuspend=null,n()}},(t.imgBytes>rf?50:800)+l);return t.unsuspend=e,function(){t.unsuspend=null,clearTimeout(a),clearTimeout(u)}}:null}function xn(){if(this.count--,this.count===0&&(this.imgCount===0||!this.waitingForImages)){if(this.stylesheets)Ln(this,this.stylesheets);else if(this.unsuspend){var t=this.unsuspend;this.unsuspend=null,t()}}}var Qn=null;function Ln(t,l){t.stylesheets=null,t.unsuspend!==null&&(t.count++,Qn=new Map,l.forEach(Xv,t),Qn=null,xn.call(t))}function Xv(t,l){if(!(l.state.loading&4)){var e=Qn.get(t);if(e)var a=e.get(null);else{e=new Map,Qn.set(t,e);for(var u=t.querySelectorAll("link[data-precedence],style[data-precedence]"),n=0;n<u.length;n++){var i=u[n];(i.nodeName==="LINK"||i.getAttribute("media")!=="not all")&&(e.set(i.dataset.precedence,i),a=i)}a&&e.set(null,a)}u=l.instance,i=u.getAttribute("data-precedence"),n=e.get(i)||a,n===a&&e.set(null,u),e.set(i,u),this.count++,a=xn.bind(this),u.addEventListener("load",a),u.addEventListener("error",a),n?n.parentNode.insertBefore(u,n.nextSibling):(t=t.nodeType===9?t.head:t,t.insertBefore(u,t.firstChild)),l.state.loading|=4}}var pu={$$typeof:At,Provider:null,Consumer:null,_currentValue:x,_currentValue2:x,_threadCount:0};function xv(t,l,e,a,u,n,i,c,o){this.tag=1,this.containerInfo=t,this.pingCache=this.current=this.pendingChildren=null,this.timeoutHandle=-1,this.callbackNode=this.next=this.pendingContext=this.context=this.cancelPendingCommit=null,this.callbackPriority=0,this.expirationTimes=ni(-1),this.entangledLanes=this.shellSuspendCounter=this.errorRecoveryDisabledLanes=this.expiredLanes=this.warmLanes=this.pingedLanes=this.suspendedLanes=this.pendingLanes=0,this.entanglements=ni(0),this.hiddenUpdates=ni(null),this.identifierPrefix=a,this.onUncaughtError=u,this.onCaughtError=n,this.onRecoverableError=i,this.pooledCache=null,this.pooledCacheLanes=0,this.formState=o,this.incompleteTransitions=new Map}function _d(t,l,e,a,u,n,i,c,o,S,z,M){return t=new xv(t,l,e,i,o,S,z,M,c),l=1,n===!0&&(l|=24),n=sl(3,null,null,l),t.current=n,n.stateNode=t,l=Zi(),l.refCount++,t.pooledCache=l,l.refCount++,n.memoizedState={element:a,isDehydrated:e,cache:l},wi(n),t}function Md(t){return t?(t=na,t):na}function Dd(t,l,e,a,u,n){u=Md(u),a.context===null?a.context=u:a.pendingContext=u,a=se(l),a.payload={element:e},n=n===void 0?null:n,n!==null&&(a.callback=n),e=oe(t,a,l),e!==null&&(el(e,t,l),lu(e,t,l))}function Ud(t,l){if(t=t.memoizedState,t!==null&&t.dehydrated!==null){var e=t.retryLane;t.retryLane=e!==0&&e<l?e:l}}function df(t,l){Ud(t,l),(t=t.alternate)&&Ud(t,l)}function Hd(t){if(t.tag===13||t.tag===31){var l=qe(t,67108864);l!==null&&el(l,t,67108864),df(t,67108864)}}function Cd(t){if(t.tag===13||t.tag===31){var l=yl();l=ii(l);var e=qe(t,l);e!==null&&el(e,t,l),df(t,l)}}var Zn=!0;function Qv(t,l,e,a){var u=O.T;O.T=null;var n=C.p;try{C.p=2,mf(t,l,e,a)}finally{C.p=n,O.T=u}}function Lv(t,l,e,a){var u=O.T;O.T=null;var n=C.p;try{C.p=8,mf(t,l,e,a)}finally{C.p=n,O.T=u}}function mf(t,l,e,a){if(Zn){var u=yf(a);if(u===null)Ic(t,l,a,Vn,e),qd(t,a);else if(Vv(u,t,l,e,a))a.stopPropagation();else if(qd(t,a),l&4&&-1<Zv.indexOf(t)){for(;u!==null;){var n=$e(u);if(n!==null)switch(n.tag){case 3:if(n=n.stateNode,n.current.memoizedState.isDehydrated){var i=De(n.pendingLanes);if(i!==0){var c=n;for(c.pendingLanes|=2,c.entangledLanes|=2;i;){var o=1<<31-cl(i);c.entanglements[1]|=o,i&=~o}jl(n),(nt&6)===0&&(_n=nl()+500,gu(0))}}break;case 31:case 13:c=qe(n,2),c!==null&&el(c,n,2),Dn(),df(n,2)}if(n=yf(a),n===null&&Ic(t,l,a,Vn,e),n===u)break;u=n}u!==null&&a.stopPropagation()}else Ic(t,l,a,null,e)}}function yf(t){return t=vi(t),vf(t)}var Vn=null;function vf(t){if(Vn=null,t=we(t),t!==null){var l=p(t);if(l===null)t=null;else{var e=l.tag;if(e===13){if(t=U(l),t!==null)return t;t=null}else if(e===31){if(t=H(l),t!==null)return t;t=null}else if(e===3){if(l.stateNode.current.memoizedState.isDehydrated)return l.tag===3?l.stateNode.containerInfo:null;t=null}else l!==t&&(t=null)}}return Vn=t,null}function Rd(t){switch(t){case"beforetoggle":case"cancel":case"click":case"close":case"contextmenu":case"copy":case"cut":case"auxclick":case"dblclick":case"dragend":case"dragstart":case"drop":case"focusin":case"focusout":case"input":case"invalid":case"keydown":case"keypress":case"keyup":case"mousedown":case"mouseup":case"paste":case"pause":case"play":case"pointercancel":case"pointerdown":case"pointerup":case"ratechange":case"reset":case"resize":case"seeked":case"submit":case"toggle":case"touchcancel":case"touchend":case"touchstart":case"volumechange":case"change":case"selectionchange":case"textInput":case"compositionstart":case"compositionend":case"compositionupdate":case"beforeblur":case"afterblur":case"beforeinput":case"blur":case"fullscreenchange":case"focus":case"hashchange":case"popstate":case"select":case"selectstart":return 2;case"drag":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"mousemove":case"mouseout":case"mouseover":case"pointermove":case"pointerout":case"pointerover":case"scroll":case"touchmove":case"wheel":case"mouseenter":case"mouseleave":case"pointerenter":case"pointerleave":return 8;case"message":switch(Dm()){case Xf:return 2;case xf:return 8;case Ru:case Um:return 32;case Qf:return 268435456;default:return 32}default:return 32}}var hf=!1,Ee=null,Ae=null,pe=null,zu=new Map,Ou=new Map,ze=[],Zv="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset".split(" ");function qd(t,l){switch(t){case"focusin":case"focusout":Ee=null;break;case"dragenter":case"dragleave":Ae=null;break;case"mouseover":case"mouseout":pe=null;break;case"pointerover":case"pointerout":zu.delete(l.pointerId);break;case"gotpointercapture":case"lostpointercapture":Ou.delete(l.pointerId)}}function _u(t,l,e,a,u,n){return t===null||t.nativeEvent!==n?(t={blockedOn:l,domEventName:e,eventSystemFlags:a,nativeEvent:n,targetContainers:[u]},l!==null&&(l=$e(l),l!==null&&Hd(l)),t):(t.eventSystemFlags|=a,l=t.targetContainers,u!==null&&l.indexOf(u)===-1&&l.push(u),t)}function Vv(t,l,e,a,u){switch(l){case"focusin":return Ee=_u(Ee,t,l,e,a,u),!0;case"dragenter":return Ae=_u(Ae,t,l,e,a,u),!0;case"mouseover":return pe=_u(pe,t,l,e,a,u),!0;case"pointerover":var n=u.pointerId;return zu.set(n,_u(zu.get(n)||null,t,l,e,a,u)),!0;case"gotpointercapture":return n=u.pointerId,Ou.set(n,_u(Ou.get(n)||null,t,l,e,a,u)),!0}return!1}function Nd(t){var l=we(t.target);if(l!==null){var e=p(l);if(e!==null){if(l=e.tag,l===13){if(l=U(e),l!==null){t.blockedOn=l,wf(t.priority,function(){Cd(e)});return}}else if(l===31){if(l=H(e),l!==null){t.blockedOn=l,wf(t.priority,function(){Cd(e)});return}}else if(l===3&&e.stateNode.current.memoizedState.isDehydrated){t.blockedOn=e.tag===3?e.stateNode.containerInfo:null;return}}}t.blockedOn=null}function Kn(t){if(t.blockedOn!==null)return!1;for(var l=t.targetContainers;0<l.length;){var e=yf(t.nativeEvent);if(e===null){e=t.nativeEvent;var a=new e.constructor(e.type,e);yi=a,e.target.dispatchEvent(a),yi=null}else return l=$e(e),l!==null&&Hd(l),t.blockedOn=e,!1;l.shift()}return!0}function Bd(t,l,e){Kn(t)&&e.delete(l)}function Kv(){hf=!1,Ee!==null&&Kn(Ee)&&(Ee=null),Ae!==null&&Kn(Ae)&&(Ae=null),pe!==null&&Kn(pe)&&(pe=null),zu.forEach(Bd),Ou.forEach(Bd)}function Jn(t,l){t.blockedOn===l&&(t.blockedOn=null,hf||(hf=!0,f.unstable_scheduleCallback(f.unstable_NormalPriority,Kv)))}var wn=null;function jd(t){wn!==t&&(wn=t,f.unstable_scheduleCallback(f.unstable_NormalPriority,function(){wn===t&&(wn=null);for(var l=0;l<t.length;l+=3){var e=t[l],a=t[l+1],u=t[l+2];if(typeof a!="function"){if(vf(a||e)===null)continue;break}var n=$e(e);n!==null&&(t.splice(l,3),l-=3,mc(n,{pending:!0,data:u,method:e.method,action:a},a,u))}}))}function Ua(t){function l(o){return Jn(o,t)}Ee!==null&&Jn(Ee,t),Ae!==null&&Jn(Ae,t),pe!==null&&Jn(pe,t),zu.forEach(l),Ou.forEach(l);for(var e=0;e<ze.length;e++){var a=ze[e];a.blockedOn===t&&(a.blockedOn=null)}for(;0<ze.length&&(e=ze[0],e.blockedOn===null);)Nd(e),e.blockedOn===null&&ze.shift();if(e=(t.ownerDocument||t).$$reactFormReplay,e!=null)for(a=0;a<e.length;a+=3){var u=e[a],n=e[a+1],i=u[kt]||null;if(typeof n=="function")i||jd(e);else if(i){var c=null;if(n&&n.hasAttribute("formAction")){if(u=n,i=n[kt]||null)c=i.formAction;else if(vf(u)!==null)continue}else c=i.action;typeof c=="function"?e[a+1]=c:(e.splice(a,3),a-=3),jd(e)}}}function Yd(){function t(n){n.canIntercept&&n.info==="react-transition"&&n.intercept({handler:function(){return new Promise(function(i){return u=i})},focusReset:"manual",scroll:"manual"})}function l(){u!==null&&(u(),u=null),a||setTimeout(e,20)}function e(){if(!a&&!navigation.transition){var n=navigation.currentEntry;n&&n.url!=null&&navigation.navigate(n.url,{state:n.getState(),info:"react-transition",history:"replace"})}}if(typeof navigation=="object"){var a=!1,u=null;return navigation.addEventListener("navigate",t),navigation.addEventListener("navigatesuccess",l),navigation.addEventListener("navigateerror",l),setTimeout(e,100),function(){a=!0,navigation.removeEventListener("navigate",t),navigation.removeEventListener("navigatesuccess",l),navigation.removeEventListener("navigateerror",l),u!==null&&(u(),u=null)}}}function gf(t){this._internalRoot=t}$n.prototype.render=gf.prototype.render=function(t){var l=this._internalRoot;if(l===null)throw Error(s(409));var e=l.current,a=yl();Dd(e,a,t,l,null,null)},$n.prototype.unmount=gf.prototype.unmount=function(){var t=this._internalRoot;if(t!==null){this._internalRoot=null;var l=t.containerInfo;Dd(t.current,2,null,t,null,null),Dn(),l[Je]=null}};function $n(t){this._internalRoot=t}$n.prototype.unstable_scheduleHydration=function(t){if(t){var l=Jf();t={blockedOn:null,target:t,priority:l};for(var e=0;e<ze.length&&l!==0&&l<ze[e].priority;e++);ze.splice(e,0,t),e===0&&Nd(t)}};var Gd=r.version;if(Gd!=="19.2.4")throw Error(s(527,Gd,"19.2.4"));C.findDOMNode=function(t){var l=t._reactInternals;if(l===void 0)throw typeof t.render=="function"?Error(s(188)):(t=Object.keys(t).join(","),Error(s(268,t)));return t=v(l),t=t!==null?R(t):null,t=t===null?null:t.stateNode,t};var Jv={bundleType:0,version:"19.2.4",rendererPackageName:"react-dom",currentDispatcherRef:O,reconcilerVersion:"19.2.4"};if(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__<"u"){var Wn=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(!Wn.isDisabled&&Wn.supportsFiber)try{Ba=Wn.inject(Jv),il=Wn}catch{}}return Mu.createRoot=function(t,l){if(!A(t))throw Error(s(299));var e=!1,a="",u=Ko,n=Jo,i=wo;return l!=null&&(l.unstable_strictMode===!0&&(e=!0),l.identifierPrefix!==void 0&&(a=l.identifierPrefix),l.onUncaughtError!==void 0&&(u=l.onUncaughtError),l.onCaughtError!==void 0&&(n=l.onCaughtError),l.onRecoverableError!==void 0&&(i=l.onRecoverableError)),l=_d(t,1,!1,null,null,e,a,null,u,n,i,Yd),t[Je]=l.current,Fc(t),new gf(l)},Mu.hydrateRoot=function(t,l,e){if(!A(t))throw Error(s(299));var a=!1,u="",n=Ko,i=Jo,c=wo,o=null;return e!=null&&(e.unstable_strictMode===!0&&(a=!0),e.identifierPrefix!==void 0&&(u=e.identifierPrefix),e.onUncaughtError!==void 0&&(n=e.onUncaughtError),e.onCaughtError!==void 0&&(i=e.onCaughtError),e.onRecoverableError!==void 0&&(c=e.onRecoverableError),e.formState!==void 0&&(o=e.formState)),l=_d(t,1,!0,l,e??null,a,u,o,n,i,c,Yd),l.context=Md(null),e=l.current,a=yl(),a=ii(a),u=se(a),u.callback=null,oe(e,u,a),e=a,l.current.lanes=e,Ya(l,e),jl(l),t[Je]=l.current,Fc(t),new $n(l)},Mu.version="19.2.4",Mu}var Jd;function l0(){if(Jd)return Sf.exports;Jd=1;function f(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(f)}catch(r){console.error(r)}}return f(),Sf.exports=t0(),Sf.exports}var e0=l0();const wd=qa(e0);function In(f){return f instanceof Function||typeof f=="function"}const a0=["Head","bodyAttributes","htmlAttributes"];function u0(f,r){return f.includes(r)}function $d(f,r){const m=r._configViaHook?.[f],s=r.config[f],A=p=>In(p)?p(r):p;return u0(a0,f)?[...(s??[]).map(A),...m??[]]:m!==void 0?m:A(s)}var yt=Nf();const pt=qa(yt),eh=dm({__proto__:null,default:pt},[yt]);function mm(f,r){const m=globalThis[Wd]=globalThis[Wd]||{};return m[f]=m[f]||r}const Wd="_vike_react",ym=mm("usePageContext.tsx",{reactContext:pt.createContext(void 0)});function n0({pageContext:f,children:r}){const{reactContext:m}=ym;return pt.createElement(m.Provider,{value:f},r)}function ah(){const{reactContext:f}=ym;return yt.useContext(f)}function i0(f){const{Page:r,config:{Loading:m}}=f;let s=r?pt.createElement(r,null):null;const A=T=>m?.layout?pt.createElement(yt.Suspense,{fallback:pt.createElement(m.layout,null)},s):T;s=A(s),[...f.config.Layout||[],...f.config.Wrapper||[]].forEach(T=>{s=pt.createElement(T,null,s),s=A(s)}),s=pt.createElement(n0,{pageContext:f},s);let p,U,H=new Promise((T,v)=>{p=T,U=v});return s=pt.createElement(c0,{renderPromiseResolve:p},s),f.config.reactStrictMode!==!1&&(s=pt.createElement(pt.StrictMode,null,s)),{page:s,renderPromise:H,renderPromiseReject:U}}function c0({children:f,renderPromiseResolve:r}){return yt.useEffect(r),f}async function kd(f,r){if(!f)return[];const m=f.map(A=>In(A)?($v(r),A(r)):A);return await Promise.all(m)}function f0(f,r){f!==void 0&&(document.title=f||""),r!==void 0&&(document.documentElement.lang=r||"en")}function Fd(f){return Object.entries(f)}function s0(f){const r={};return(f.config.react??[]).forEach(m=>{const s=In(m)?m(f):m;s&&Fd(s).forEach(([A,p])=>{p&&(r[A]??(r[A]={}),Fd(p).forEach(([U,H])=>{var T;if(!In(H))(T=r[A])[U]??(T[U]=H);else{const v=r[A][U];r[A][U]=(...R)=>{v?.(...R),H(...R)}}}))})}),r}function vm(f){return typeof f=="object"&&f!==null}function o0(f,r){return Object.assign(f,r),f}const _e=mm("onRenderClient.tsx",{});async function uh(f){f._headAlreadySet=f.isHydration,await kd(f.config.onBeforeRenderClient,f);const{page:r,renderPromise:m,renderPromiseReject:s}=i0(f);f.page=r,_e.onUncaughtErrorLocal=H=>{s(H)};const A=document.getElementById("root"),{hydrateRootOptions:p,createRootOptions:U}=s0(f);f.isHydration&&A.innerHTML!==""?_e.root=wd.hydrateRoot(A,r,{...p,onUncaughtError(...H){Id.call(this,H,p)}}):(_e.root||(_e.root=wd.createRoot(A,{...U,onUncaughtError(...H){Id.call(this,H,U)}})),_e.root.render(r)),f.root=_e.root;try{await m}finally{delete _e.onUncaughtErrorLocal}f.isHydration||(f._headAlreadySet=!0,r0(f)),await kd(f.config.onAfterRenderClient,f)}function r0(f){const r=$d("title",f),m=$d("lang",f);f0(r,m)}function Id(f,r){const[m,s]=f,A=d0(m,s);console.error(A),vm(A)&&(A.isAlreadyLogged=!0),_e.onUncaughtErrorLocal?.(A),r?.onUncaughtError?.call(this,A,s)}function d0(f,r){if(!r?.componentStack||!vm(f))return f;const m=String(f.stack).split(`
`);let s=m.findIndex(v=>v.includes("react_stack_bottom_frame"));if(s===-1&&(s=m.findIndex(v=>v.includes("node_modules")&&v.includes("react"))),s===-1)return f;const A=m.slice(0,s),p=m.slice(s),U=r.componentStack.split(`
`).filter(Boolean);U[0]===A.at(-1)&&U.shift();const H=[...A,...U,...p].join(`
`);return o0(f,{stack:H})}const m0=async()=>{document.querySelector("body")?.classList.add("page-is-transitioning")},nh=Object.freeze(Object.defineProperty({__proto__:null,onPageTransitionStart:m0},Symbol.toStringTag,{value:"Module"})),y0=async()=>{document.querySelector("body")?.classList.remove("page-is-transitioning")},ih=Object.freeze(Object.defineProperty({__proto__:null,onPageTransitionEnd:y0},Symbol.toStringTag,{value:"Module"}));var pf={exports:{}},Du={};/**
 * @license React
 * react-jsx-runtime.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Pd;function v0(){if(Pd)return Du;Pd=1;var f=Symbol.for("react.transitional.element"),r=Symbol.for("react.fragment");function m(s,A,p){var U=null;if(p!==void 0&&(U=""+p),A.key!==void 0&&(U=""+A.key),"key"in A){p={};for(var H in A)H!=="key"&&(p[H]=A[H])}else p=A;return A=p.ref,{$$typeof:f,type:s,key:U,ref:A!==void 0?A:null,props:p}}return Du.Fragment=r,Du.jsx=m,Du.jsxs=m,Du}var tm;function h0(){return tm||(tm=1,pf.exports=v0()),pf.exports}var al=h0();const g0=qa(al),ch=dm({__proto__:null,default:g0},[al]);var zf,lm;function S0(){if(lm)return zf;lm=1;var f=typeof Element<"u",r=typeof Map=="function",m=typeof Set=="function",s=typeof ArrayBuffer=="function"&&!!ArrayBuffer.isView;function A(p,U){if(p===U)return!0;if(p&&U&&typeof p=="object"&&typeof U=="object"){if(p.constructor!==U.constructor)return!1;var H,T,v;if(Array.isArray(p)){if(H=p.length,H!=U.length)return!1;for(T=H;T--!==0;)if(!A(p[T],U[T]))return!1;return!0}var R;if(r&&p instanceof Map&&U instanceof Map){if(p.size!==U.size)return!1;for(R=p.entries();!(T=R.next()).done;)if(!U.has(T.value[0]))return!1;for(R=p.entries();!(T=R.next()).done;)if(!A(T.value[1],U.get(T.value[0])))return!1;return!0}if(m&&p instanceof Set&&U instanceof Set){if(p.size!==U.size)return!1;for(R=p.entries();!(T=R.next()).done;)if(!U.has(T.value[0]))return!1;return!0}if(s&&ArrayBuffer.isView(p)&&ArrayBuffer.isView(U)){if(H=p.length,H!=U.length)return!1;for(T=H;T--!==0;)if(p[T]!==U[T])return!1;return!0}if(p.constructor===RegExp)return p.source===U.source&&p.flags===U.flags;if(p.valueOf!==Object.prototype.valueOf&&typeof p.valueOf=="function"&&typeof U.valueOf=="function")return p.valueOf()===U.valueOf();if(p.toString!==Object.prototype.toString&&typeof p.toString=="function"&&typeof U.toString=="function")return p.toString()===U.toString();if(v=Object.keys(p),H=v.length,H!==Object.keys(U).length)return!1;for(T=H;T--!==0;)if(!Object.prototype.hasOwnProperty.call(U,v[T]))return!1;if(f&&p instanceof Element)return!1;for(T=H;T--!==0;)if(!((v[T]==="_owner"||v[T]==="__v"||v[T]==="__o")&&p.$$typeof)&&!A(p[v[T]],U[v[T]]))return!1;return!0}return p!==p&&U!==U}return zf=function(U,H){try{return A(U,H)}catch(T){if((T.message||"").match(/stack|recursion/i))return console.warn("react-fast-compare cannot handle circular refs"),!1;throw T}},zf}var b0=S0();const T0=qa(b0);var Of,em;function E0(){if(em)return Of;em=1;var f=function(r,m,s,A,p,U,H,T){if(!r){var v;if(m===void 0)v=new Error("Minified exception occurred; use the non-minified dev environment for the full error message and additional helpful warnings.");else{var R=[s,A,p,U,H,T],N=0;v=new Error(m.replace(/%s/g,function(){return R[N++]})),v.name="Invariant Violation"}throw v.framesToPop=1,v}};return Of=f,Of}var A0=E0();const am=qa(A0);var _f,um;function p0(){return um||(um=1,_f=function(r,m,s,A){var p=s?s.call(A,r,m):void 0;if(p!==void 0)return!!p;if(r===m)return!0;if(typeof r!="object"||!r||typeof m!="object"||!m)return!1;var U=Object.keys(r),H=Object.keys(m);if(U.length!==H.length)return!1;for(var T=Object.prototype.hasOwnProperty.bind(m),v=0;v<U.length;v++){var R=U[v];if(!T(R))return!1;var N=r[R],K=m[R];if(p=s?s.call(A,N,K,R):void 0,p===!1||p===void 0&&N!==K)return!1}return!0}),_f}var z0=p0();const O0=qa(z0);var hm=(f=>(f.BASE="base",f.BODY="body",f.HEAD="head",f.HTML="html",f.LINK="link",f.META="meta",f.NOSCRIPT="noscript",f.SCRIPT="script",f.STYLE="style",f.TITLE="title",f.FRAGMENT="Symbol(react.fragment)",f))(hm||{}),Mf={link:{rel:["amphtml","canonical","alternate"]},script:{type:["application/ld+json"]},meta:{charset:"",name:["generator","robots","description"],property:["og:type","og:title","og:url","og:image","og:image:alt","og:description","twitter:url","twitter:title","twitter:description","twitter:image","twitter:image:alt","twitter:card","twitter:site"]}},nm=Object.values(hm),Bf={accesskey:"accessKey",charset:"charSet",class:"className",contenteditable:"contentEditable",contextmenu:"contextMenu","http-equiv":"httpEquiv",itemprop:"itemProp",tabindex:"tabIndex"},_0=Object.entries(Bf).reduce((f,[r,m])=>(f[m]=r,f),{}),Rl="data-rh",Ca={DEFAULT_TITLE:"defaultTitle",DEFER:"defer",ENCODE_SPECIAL_CHARACTERS:"encodeSpecialCharacters",ON_CHANGE_CLIENT_STATE:"onChangeClientState",TITLE_TEMPLATE:"titleTemplate",PRIORITIZE_SEO_TAGS:"prioritizeSeoTags"},Ra=(f,r)=>{for(let m=f.length-1;m>=0;m-=1){const s=f[m];if(Object.prototype.hasOwnProperty.call(s,r))return s[r]}return null},M0=f=>{let r=Ra(f,"title");const m=Ra(f,Ca.TITLE_TEMPLATE);if(Array.isArray(r)&&(r=r.join("")),m&&r)return m.replace(/%s/g,()=>r);const s=Ra(f,Ca.DEFAULT_TITLE);return r||s||void 0},D0=f=>Ra(f,Ca.ON_CHANGE_CLIENT_STATE)||(()=>{}),Df=(f,r)=>r.filter(m=>typeof m[f]<"u").map(m=>m[f]).reduce((m,s)=>({...m,...s}),{}),U0=(f,r)=>r.filter(m=>typeof m.base<"u").map(m=>m.base).reverse().reduce((m,s)=>{if(!m.length){const A=Object.keys(s);for(let p=0;p<A.length;p+=1){const H=A[p].toLowerCase();if(f.indexOf(H)!==-1&&s[H])return m.concat(s)}}return m},[]),H0=f=>console&&typeof console.warn=="function"&&console.warn(f),Uu=(f,r,m)=>{const s={};return m.filter(A=>Array.isArray(A[f])?!0:(typeof A[f]<"u"&&H0(`Helmet: ${f} should be of type "Array". Instead found type "${typeof A[f]}"`),!1)).map(A=>A[f]).reverse().reduce((A,p)=>{const U={};p.filter(T=>{let v;const R=Object.keys(T);for(let K=0;K<R.length;K+=1){const tt=R[K],Q=tt.toLowerCase();r.indexOf(Q)!==-1&&!(v==="rel"&&T[v].toLowerCase()==="canonical")&&!(Q==="rel"&&T[Q].toLowerCase()==="stylesheet")&&(v=Q),r.indexOf(tt)!==-1&&(tt==="innerHTML"||tt==="cssText"||tt==="itemprop")&&(v=tt)}if(!v||!T[v])return!1;const N=T[v].toLowerCase();return s[v]||(s[v]={}),U[v]||(U[v]={}),s[v][N]?!1:(U[v][N]=!0,!0)}).reverse().forEach(T=>A.push(T));const H=Object.keys(U);for(let T=0;T<H.length;T+=1){const v=H[T],R={...s[v],...U[v]};s[v]=R}return A},[]).reverse()},C0=(f,r)=>{if(Array.isArray(f)&&f.length){for(let m=0;m<f.length;m+=1)if(f[m][r])return!0}return!1},R0=f=>({baseTag:U0(["href"],f),bodyAttributes:Df("bodyAttributes",f),defer:Ra(f,Ca.DEFER),encode:Ra(f,Ca.ENCODE_SPECIAL_CHARACTERS),htmlAttributes:Df("htmlAttributes",f),linkTags:Uu("link",["rel","href"],f),metaTags:Uu("meta",["name","charset","http-equiv","property","itemprop"],f),noscriptTags:Uu("noscript",["innerHTML"],f),onChangeClientState:D0(f),scriptTags:Uu("script",["src","innerHTML"],f),styleTags:Uu("style",["cssText"],f),title:M0(f),titleAttributes:Df("titleAttributes",f),prioritizeSeoTags:C0(f,Ca.PRIORITIZE_SEO_TAGS)}),gm=f=>Array.isArray(f)?f.join(""):f,q0=(f,r)=>{const m=Object.keys(f);for(let s=0;s<m.length;s+=1)if(r[m[s]]&&r[m[s]].includes(f[m[s]]))return!0;return!1},Uf=(f,r)=>Array.isArray(f)?f.reduce((m,s)=>(q0(s,r)?m.priority.push(s):m.default.push(s),m),{priority:[],default:[]}):{default:f,priority:[]},im=(f,r)=>({...f,[r]:void 0}),N0=["noscript","script","style"],Hf=(f,r=!0)=>r===!1?String(f):String(f).replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;").replace(/"/g,"&quot;").replace(/'/g,"&#x27;"),Sm=f=>Object.keys(f).reduce((r,m)=>{const s=typeof f[m]<"u"?`${m}="${f[m]}"`:`${m}`;return r?`${r} ${s}`:s},""),B0=(f,r,m,s)=>{const A=Sm(m),p=gm(r);return A?`<${f} ${Rl}="true" ${A}>${Hf(p,s)}</${f}>`:`<${f} ${Rl}="true">${Hf(p,s)}</${f}>`},j0=(f,r,m=!0)=>r.reduce((s,A)=>{const p=A,U=Object.keys(p).filter(v=>!(v==="innerHTML"||v==="cssText")).reduce((v,R)=>{const N=typeof p[R]>"u"?R:`${R}="${Hf(p[R],m)}"`;return v?`${v} ${N}`:N},""),H=p.innerHTML||p.cssText||"",T=N0.indexOf(f)===-1;return`${s}<${f} ${Rl}="true" ${U}${T?"/>":`>${H}</${f}>`}`},""),bm=(f,r={})=>Object.keys(f).reduce((m,s)=>{const A=Bf[s];return m[A||s]=f[s],m},r),Y0=(f,r,m)=>{const s={key:r,[Rl]:!0},A=bm(m,s);return[pt.createElement("title",A,r)]},Fn=(f,r)=>r.map((m,s)=>{const A={key:s,[Rl]:!0};return Object.keys(m).forEach(p=>{const H=Bf[p]||p;if(H==="innerHTML"||H==="cssText"){const T=m.innerHTML||m.cssText;A.dangerouslySetInnerHTML={__html:T}}else A[H]=m[p]}),pt.createElement(f,A)}),_l=(f,r,m=!0)=>{switch(f){case"title":return{toComponent:()=>Y0(f,r.title,r.titleAttributes),toString:()=>B0(f,r.title,r.titleAttributes,m)};case"bodyAttributes":case"htmlAttributes":return{toComponent:()=>bm(r),toString:()=>Sm(r)};default:return{toComponent:()=>Fn(f,r),toString:()=>j0(f,r,m)}}},G0=({metaTags:f,linkTags:r,scriptTags:m,encode:s})=>{const A=Uf(f,Mf.meta),p=Uf(r,Mf.link),U=Uf(m,Mf.script);return{priorityMethods:{toComponent:()=>[...Fn("meta",A.priority),...Fn("link",p.priority),...Fn("script",U.priority)],toString:()=>`${_l("meta",A.priority,s)} ${_l("link",p.priority,s)} ${_l("script",U.priority,s)}`},metaTags:A.default,linkTags:p.default,scriptTags:U.default}},X0=f=>{const{baseTag:r,bodyAttributes:m,encode:s=!0,htmlAttributes:A,noscriptTags:p,styleTags:U,title:H="",titleAttributes:T,prioritizeSeoTags:v}=f;let{linkTags:R,metaTags:N,scriptTags:K}=f,tt={toComponent:()=>{},toString:()=>""};return v&&({priorityMethods:tt,linkTags:R,metaTags:N,scriptTags:K}=G0(f)),{priority:tt,base:_l("base",r,s),bodyAttributes:_l("bodyAttributes",m,s),htmlAttributes:_l("htmlAttributes",A,s),link:_l("link",R,s),meta:_l("meta",N,s),noscript:_l("noscript",p,s),script:_l("script",K,s),style:_l("style",U,s),title:_l("title",{title:H,titleAttributes:T},s)}},Cf=X0,kn=[],Tm=!!(typeof window<"u"&&window.document&&window.document.createElement),Rf=class{instances=[];canUseDOM=Tm;context;value={setHelmet:f=>{this.context.helmet=f},helmetInstances:{get:()=>this.canUseDOM?kn:this.instances,add:f=>{(this.canUseDOM?kn:this.instances).push(f)},remove:f=>{const r=(this.canUseDOM?kn:this.instances).indexOf(f);(this.canUseDOM?kn:this.instances).splice(r,1)}}};constructor(f,r){this.context=f,this.canUseDOM=r||!1,r||(f.helmet=Cf({baseTag:[],bodyAttributes:{},htmlAttributes:{},linkTags:[],metaTags:[],noscriptTags:[],scriptTags:[],styleTags:[],title:"",titleAttributes:{}}))}},x0={},Em=pt.createContext(x0),Am=class pm extends yt.Component{static canUseDOM=Tm;helmetData;constructor(r){super(r),this.helmetData=new Rf(this.props.context||{},pm.canUseDOM)}render(){return pt.createElement(Em.Provider,{value:this.helmetData.value},this.props.children)}},Ha=(f,r)=>{const m=document.head||document.querySelector("head"),s=m.querySelectorAll(`${f}[${Rl}]`),A=[].slice.call(s),p=[];let U;return r&&r.length&&r.forEach(H=>{const T=document.createElement(f);for(const v in H)if(Object.prototype.hasOwnProperty.call(H,v))if(v==="innerHTML")T.innerHTML=H.innerHTML;else if(v==="cssText")T.styleSheet?T.styleSheet.cssText=H.cssText:T.appendChild(document.createTextNode(H.cssText));else{const R=v,N=typeof H[R]>"u"?"":H[R];T.setAttribute(v,N)}T.setAttribute(Rl,"true"),A.some((v,R)=>(U=R,T.isEqualNode(v)))?A.splice(U,1):p.push(T)}),A.forEach(H=>H.parentNode?.removeChild(H)),p.forEach(H=>m.appendChild(H)),{oldTags:A,newTags:p}},qf=(f,r)=>{const m=document.getElementsByTagName(f)[0];if(!m)return;const s=m.getAttribute(Rl),A=s?s.split(","):[],p=[...A],U=Object.keys(r);for(const H of U){const T=r[H]||"";m.getAttribute(H)!==T&&m.setAttribute(H,T),A.indexOf(H)===-1&&A.push(H);const v=p.indexOf(H);v!==-1&&p.splice(v,1)}for(let H=p.length-1;H>=0;H-=1)m.removeAttribute(p[H]);A.length===p.length?m.removeAttribute(Rl):m.getAttribute(Rl)!==U.join(",")&&m.setAttribute(Rl,U.join(","))},Q0=(f,r)=>{typeof f<"u"&&document.title!==f&&(document.title=gm(f)),qf("title",r)},cm=(f,r)=>{const{baseTag:m,bodyAttributes:s,htmlAttributes:A,linkTags:p,metaTags:U,noscriptTags:H,onChangeClientState:T,scriptTags:v,styleTags:R,title:N,titleAttributes:K}=f;qf("body",s),qf("html",A),Q0(N,K);const tt={baseTag:Ha("base",m),linkTags:Ha("link",p),metaTags:Ha("meta",U),noscriptTags:Ha("noscript",H),scriptTags:Ha("script",v),styleTags:Ha("style",R)},Q={},et={};Object.keys(tt).forEach(vt=>{const{newTags:it,oldTags:qt}=tt[vt];it.length&&(Q[vt]=it),qt.length&&(et[vt]=tt[vt].oldTags)}),r&&r(),T(f,Q,et)},Hu=null,L0=f=>{Hu&&cancelAnimationFrame(Hu),f.defer?Hu=requestAnimationFrame(()=>{cm(f,()=>{Hu=null})}):(cm(f),Hu=null)},Z0=L0,fm=class extends yt.Component{rendered=!1;shouldComponentUpdate(f){return!O0(f,this.props)}componentDidUpdate(){this.emitChange()}componentWillUnmount(){const{helmetInstances:f}=this.props.context;f.remove(this),this.emitChange()}emitChange(){const{helmetInstances:f,setHelmet:r}=this.props.context;let m=null;const s=R0(f.get().map(A=>{const p={...A.props};return delete p.context,p}));Am.canUseDOM?Z0(s):Cf&&(m=Cf(s)),r(m)}init(){if(this.rendered)return;this.rendered=!0;const{helmetInstances:f}=this.props.context;f.add(this),this.emitChange()}render(){return this.init(),null}},V0=class extends yt.Component{static defaultProps={defer:!0,encodeSpecialCharacters:!0,prioritizeSeoTags:!1};shouldComponentUpdate(f){return!T0(im(this.props,"helmetData"),im(f,"helmetData"))}mapNestedChildrenToProps(f,r){if(!r)return null;switch(f.type){case"script":case"noscript":return{innerHTML:r};case"style":return{cssText:r};default:throw new Error(`<${f.type} /> elements are self-closing and can not contain children. Refer to our API for more information.`)}}flattenArrayTypeChildren(f,r,m,s){return{...r,[f.type]:[...r[f.type]||[],{...m,...this.mapNestedChildrenToProps(f,s)}]}}mapObjectTypeChildren(f,r,m,s){switch(f.type){case"title":return{...r,[f.type]:s,titleAttributes:{...m}};case"body":return{...r,bodyAttributes:{...m}};case"html":return{...r,htmlAttributes:{...m}};default:return{...r,[f.type]:{...m}}}}mapArrayTypeChildrenToProps(f,r){let m={...r};return Object.keys(f).forEach(s=>{m={...m,[s]:f[s]}}),m}warnOnInvalidChildren(f,r){return am(nm.some(m=>f.type===m),typeof f.type=="function"?"You may be attempting to nest <Helmet> components within each other, which is not allowed. Refer to our API for more information.":`Only elements types ${nm.join(", ")} are allowed. Helmet does not support rendering <${f.type}> elements. Refer to our API for more information.`),am(!r||typeof r=="string"||Array.isArray(r)&&!r.some(m=>typeof m!="string"),`Helmet expects a string as a child of <${f.type}>. Did you forget to wrap your children in braces? ( <${f.type}>{\`\`}</${f.type}> ) Refer to our API for more information.`),!0}mapChildrenToProps(f,r){let m={};return pt.Children.forEach(f,s=>{if(!s||!s.props)return;const{children:A,...p}=s.props,U=Object.keys(p).reduce((T,v)=>(T[_0[v]||v]=p[v],T),{});let{type:H}=s;switch(typeof H=="symbol"?H=H.toString():this.warnOnInvalidChildren(s,A),H){case"Symbol(react.fragment)":r=this.mapChildrenToProps(A,r);break;case"link":case"meta":case"noscript":case"script":case"style":m=this.flattenArrayTypeChildren(s,m,U,A);break;default:r=this.mapObjectTypeChildren(s,r,U,A);break}}),this.mapArrayTypeChildrenToProps(m,r)}render(){const{children:f,...r}=this.props;let m={...r},{helmetData:s}=r;if(f&&(m=this.mapChildrenToProps(f,m)),s&&!(s instanceof Rf)){const A=s;s=new Rf(A.context,!0),delete m.helmetData}return s?pt.createElement(fm,{...m,context:s.value}):pt.createElement(Em.Consumer,null,A=>pt.createElement(fm,{...m,context:A}))}},K0=(f,r,m,s,A,p,U,H,T)=>{const v=document.documentElement;function R(Q){(Array.isArray(s)?s:[s]).forEach(vt=>{const it=vt==="class",qt=it&&p?A.map(At=>p[At]||At):A;it?(v.classList.remove(...qt),v.classList.add(p&&p[Q]?p[Q]:Q)):v.setAttribute(vt,Q)}),N(Q)}function N(Q){if(T){const et=Q.includes("dark");v.style.colorScheme=et?"dark":"light"}}function K(Q,et,vt){let it;return Q==="system"?it=vt?"dark":"light":it=Q,et?`liquid-${it}`:it}function tt(){try{const Q=localStorage.getItem(`${m}-base`),et=localStorage.getItem(`${m}-liquid`);return Q!==null&&et!==null?{baseTheme:Q,isLiquid:et==="true"}:{baseTheme:f,isLiquid:r}}catch{return{baseTheme:"system",isLiquid:!0}}}if(U)R(U);else{const{baseTheme:Q,isLiquid:et}=tt(),vt=localStorage.getItem(`${m}-base`)!==null,it=localStorage.getItem(`${m}-liquid`)!==null;if(!vt||!it)try{vt||localStorage.setItem(`${m}-base`,Q),it||localStorage.setItem(`${m}-liquid`,String(et))}catch{}const qt=window.matchMedia("(prefers-color-scheme: dark)").matches,At=K(Q,et,qt);R(At)}},J0=yt.memo(({forcedTheme:f,storageKey:r,attribute:m,enableSystem:s,enableColorScheme:A,defaultBaseTheme:p,defaultIsLiquid:U,value:H,themes:T,nonce:v,scriptProps:R})=>{const N=JSON.stringify([p,U,r,m,T,H,f,s,A]).slice(1,-1);return al.jsx("script",{...R,suppressHydrationWarning:!0,nonce:typeof window>"u"?v:"",dangerouslySetInnerHTML:{__html:`(${K0.toString()})(${N})`}})}),sm="(prefers-color-scheme: dark)",zm=typeof window>"u",jf=yt.createContext(void 0),om=(f,r)=>{try{localStorage.setItem(f,r),document.cookie=`vuer:theme=${r}; path=/; max-age=3153600000;`}catch{}},rm=(f,r)=>{if(zm)return r;try{return localStorage.getItem(f)||r}catch{return r}},w0=(f,r,m)=>{let s;return f==="system"?s=m?"dark":"light":s=f,r?`liquid-${s}`:s};function fh(f){const r=yt.useContext(jf);if(r===void 0)throw new Error("useTheme must be used within a ThemeProvider");return yt.useMemo(()=>f?typeof f=="function"?f(r):r[f]:r,[r])}var $0=f=>yt.useContext(jf)?al.jsx(al.Fragment,{children:f.children}):al.jsx(k0,{...f}),W0=["light","dark","liquid-light","liquid-dark"],k0=({forcedTheme:f,disableTransitionOnChange:r=!1,enableSystem:m=!0,enableColorScheme:s=!1,storageKey:A="vuer-uikit-theme",defaultBaseTheme:p=m?"system":"light",defaultIsLiquid:U=!0,attribute:H="class",value:T,children:v,nonce:R,scriptProps:N})=>{const K=W0,tt=yt.useMemo(()=>({baseTheme:p,isLiquid:U}),[p,U]),[Q,et]=yt.useState(()=>{const w=rm(`${A}-base`,"");return w&&["light","dark","system"].includes(w)?w:tt.baseTheme}),[vt,it]=yt.useState(()=>{const w=rm(`${A}-liquid`,"");return w?w==="true":tt.isLiquid}),[qt,At]=yt.useState(()=>zm?!1:window.matchMedia(sm).matches),Jt=T?Object.values(T):K,wt=yt.useMemo(()=>w0(Q,vt,qt),[Q,vt,qt]),Nt=yt.useMemo(()=>f||wt,[f,wt]),$=yt.useCallback(w=>{if(!w)return;const Tt=T?T[w]:w,Dl=r?F0(R):null,Xt=document.documentElement,O=C=>{C==="class"?(Xt.classList.remove(...Jt),Tt&&Xt.classList.add(Tt)):C.startsWith("data-")&&(Tt?Xt.setAttribute(C,Tt):Xt.removeAttribute(C))};if(Array.isArray(H)?H.forEach(O):O(H),s){const C=w.includes("dark");Xt.style.colorScheme=C?"dark":"light"}Dl?.()},[H,Jt,r,s,R,T]),Gt=yt.useCallback(w=>{et(w),om(`${A}-base`,w)},[A]),ul=yt.useCallback(()=>{it(w=>{const Tt=!w;return om(`${A}-liquid`,String(Tt)),Tt})},[A]),Ml=yt.useCallback(w=>{const Tt=w.matches;At(Tt)},[]);yt.useEffect(()=>{const w=window.matchMedia(sm);return w.addListener(Ml),Ml(w),()=>w.removeListener(Ml)},[Ml]),yt.useEffect(()=>{const w=Tt=>{Tt.key===`${A}-base`&&Tt.newValue?et(Tt.newValue):Tt.key===`${A}-liquid`&&Tt.newValue!==null&&it(Tt.newValue==="true")};return window.addEventListener("storage",w),()=>window.removeEventListener("storage",w)},[A]),yt.useEffect(()=>{$(Nt)},[$,Nt]);const vl=yt.useMemo(()=>({baseTheme:Q,setBaseTheme:Gt,isLiquid:vt,toggleLiquid:ul,computedTheme:wt,resolvedTheme:Nt,systemTheme:m?qt?"dark":"light":void 0,forcedTheme:f,storageKey:A}),[Q,Gt,vt,ul,wt,Nt,qt,m,f,A]);return al.jsxs(jf.Provider,{value:vl,children:[al.jsx(J0,{forcedTheme:f,storageKey:A,attribute:H,enableSystem:m,enableColorScheme:s,defaultBaseTheme:tt.baseTheme,defaultIsLiquid:tt.isLiquid,value:T,themes:K,nonce:R,scriptProps:N}),v]})},F0=f=>{const r=document.createElement("style");return f&&r.setAttribute("nonce",f),r.appendChild(document.createTextNode("*,*::before,*::after{-webkit-transition:none!important;-moz-transition:none!important;-o-transition:none!important;-ms-transition:none!important;transition:none!important}")),document.head.appendChild(r),()=>{window.getComputedStyle(document.body),setTimeout(()=>{r&&r.parentNode&&r.parentNode.removeChild(r)},1)}};function I0({children:f}){return al.jsx(Am,{children:al.jsxs(yt.StrictMode,{children:[al.jsxs(V0,{children:[al.jsx("title",{children:"Vuer by FortyFive Labs"}),al.jsx("meta",{name:"description",content:"Modern Physical AI Studio for XR"})]}),al.jsx($0,{defaultBaseTheme:"system",storageKey:"vuer-web-theme",children:f})]})})}const sh=Object.freeze(Object.defineProperty({__proto__:null,default:I0},Symbol.toStringTag,{value:"Module"})),oh={component:P0};function P0(){return pt.createElement(pt.Fragment,null,pt.createElement("div",{style:{width:"100%",height:"100%",maxHeight:"100%",background:"linear-gradient(110deg, #ececec 8%, #f5f5f5 18%, #ececec 33%)",borderRadius:"5px",backgroundSize:"200% 100%",animation:"1.3s vike-react-loading linear infinite",aspectRatio:"2.5/1"}}),pt.createElement("style",{href:"vike-react-loading",precedence:"default"},`
          @keyframes vike-react-loading {
            to {
              background-position-x: -200%;
            }
          }
        `))}export{V0 as H,ch as J,eh as M,pt as P,sh as a,ih as b,nh as c,Am as d,Pv as e,fh as f,qa as g,Nf as h,oh as i,al as j,kv as k,e0 as l,uh as o,yt as r,ah as u};
